@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.03/RCS/ed.defns.c,v 3.15 1992/10/05 02:41:30 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.04/RCS/ed.defns.c,v 3.17 1993/04/26 21:13:10 christos Exp $ */
 /*
  * ed.defns.c: Editor function definitions and initialization
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: ed.defns.c,v 3.15 1992/10/05 02:41:30 christos Exp $")
+RCSID("$Id: ed.defns.c,v 3.17 1993/04/26 21:13:10 christos Exp $")
 
 #include "ed.h"
 
@@ -259,8 +259,12 @@ PFCmd   CcFuncTbl[] = {		/* table of available commands */
 #define		F_COMPLETE_ALL	106
     e_list_all,
 #define		F_LIST_ALL	107
+    e_complete_fwd,
+#define		F_COMPLETE_FWD	108
+    e_complete_back,
+#define		F_COMPLETE_BACK	109
     0				/* DUMMY VALUE */
-#define		F_NUM_FNS	108
+#define		F_NUM_FNS	110
 };
 
 KEYCMD  NumFuns = F_NUM_FNS;
@@ -1091,232 +1095,236 @@ KEYCMD  CcViCmdMap[] = {
 
 
 struct KeyFuncs FuncNames[] = {
-    "backward-char", F_CHARBACK,
-    "Move back a character",
-    "backward-delete-char", F_DELPREV,
-    "Delete the character behind cursor",
-    "backward-delete-word", F_DELWORDPREV,
-    "Cut from beginning of current word to cursor - saved in cut buffer",
-    "backward-kill-line", F_KILLBEG,
-    "Cut from beginning of line to cursor - save in cut buffer",
-    "backward-word", F_WORDBACK,
-    "Move to beginning of current word",
-    "beginning-of-line", F_TOBEG,
-    "Move to beginning of line",
-    "capitalize-word", F_CASECAPITAL,
-    "Capitalize the characters from cursor to end of current word",
-    "change-case", V_CHGCASE,
-    "Vi change case of character under cursor and advance one character",
-    "change-till-end-of-line", V_CHGTOEND,	/* backwards compat. */
-    "Vi change to end of line",
-    "clear-screen", F_CLEARDISP,
-    "Clear screen leaving current line on top",
-    "complete-word", F_COMPLETE,
-    "Complete current word",
-    "complete-word-raw", F_COMPLETE_ALL,
-    "Complete current word ignoring programmable completions",
-    "copy-prev-word", F_COPYPREV,
-    "Copy current word to cursor",
-    "copy-region-as-kill", F_COPYREGION,
-    "Copy area between mark and cursor to cut buffer",
-    "delete-char", F_DELNEXT,
-    "Delete character under cursor",
-    "delete-char-or-eof", F_DELNEXT_EOF,
-    "Delete character under cursor or end of file if there is no character",	
-    "delete-char-or-list", F_LIST_DELNEXT,
-    "Delete character under cursor or list completions if at end of line",
-    "delete-word", F_DELWORDNEXT,
-    "Cut from cursor to end of current word - save in cut buffer",
-    "digit", F_DIGIT,
-    "Adds to argument if started or enters digit",
-    "digit-argument", F_ARGDIGIT,
-    "Digit that starts argument",
-    "down-history", F_DOWN_HIST,
-    "Move to next history line",
-    "downcase-word", F_CASELOWER,
-    "Lowercase the characters from cursor to end of current word",
-    "end-of-file", F_SEND_EOF,
-    "Indicate end of file",
-    "end-of-line", F_TOEND,
-    "Move cursor to end of line",
-    "exchange-point-and-mark", F_EXCHANGE_MARK,
-    "Exchange the cursor and mark",
-    "expand-glob", F_EXPAND_GLOB,
-    "Expand file name wildcards",
-    "expand-history", F_EXPAND_HISTORY,
-    "Expand history escapes",
-    "expand-line", F_EXPAND,
-    "Expand the history escapes in a line",
-    "expand-variables", F_EXPAND_VARS,
-    "Expand variables",
-    "forward-char", F_CHARFWD,
-    "Move forward one character",
-    "forward-word", F_WORDFWD,
-    "Move forward to end of current word",
-    "gosmacs-transpose-chars", F_GCHARSWITCH,
-    "Exchange the two characters before the cursor",
-    "history-search-backward", F_UP_SEARCH_HIST,
-    "Search in history backwards for line beginning as current",
-    "history-search-forward", F_DOWN_SEARCH_HIST,
-    "Search in history forward for line beginning as current",
-    "insert-last-word", F_LAST_ITEM,
-    "Insert last item of previous command",
-    "i-search-fwd", F_INC_FWD,
-    "Incremental search forward",
-    "i-search-back", F_INC_BACK,
-    "Incremental search backwards",
-    "keyboard-quit", F_STARTOVER,
-    "Clear line",
-    "kill-line", F_KILLEND,
-    "Cut to end of line and save in cut buffer",
-    "kill-region", F_KILLREGION,
-    "Cut area between mark and cursor and save in cut buffer",
-    "kill-whole-line", F_KILLALL,
-    "Cut the entire line and save in cut buffer",
-    "list-choices", F_LIST_CHOICES,
-    "List choices for completion",
-    "list-choices-raw", F_LIST_ALL,
-    "List choices for completion overriding programmable completion",
-    "list-glob", F_LIST_GLOB,
-    "List file name wildcard matches",
-    "list-or-eof", F_LIST_EOF,
-    "List choices for completion or indicate end of file if empty line",
-    "load-average", F_LOAD_AVERAGE,
-    "Display load average and current process status",
-    "magic-space", F_MAGIC_SPACE,
-    "Expand history escapes and insert a space",
-    "newline", F_NEWLINE,
-    "Execute command",
-    "normalize-path", F_PATH_NORM,
-    "Expand pathnames, eliminating leading .'s and ..'s",
-    "overwrite-mode", F_INSOVR,
-    "Switch from insert to overwrite mode or vice versa",
-    "prefix-meta", F_METANEXT,
-    "Add 8th bit to next character typed",
-    "quoted-insert", F_QUOTE,
-    "Add the next character typed to the line verbatim",
-    "redisplay", F_REDISP,
-    "Redisplay everything",
-    "run-fg-editor", F_RUN_FG_EDITOR,
-    "Restart stopped editor",
-    "run-help", F_HELPME,
-    "Look for help on current command",
-    "self-insert-command", F_INSERT,
-    "This character is added to the line",
-    "sequence-lead-in", F_XKEY,
-    "This character is the first in a character sequence",
-    "set-mark-command", F_SET_MARK,
-    "Set the mark at cursor",
-    "spell-word", F_CORRECT,
-    "Correct the spelling of current word",
-    "spell-line", F_CORRECT_L,
-    "Correct the spelling of entire line",
-    "stuff-char", F_STUFF_CHAR,
-    "Send character to tty in cooked mode",
-    "toggle-literal-history", F_TOGGLE_HIST,
-    "Toggle between literal and lexical current history line",
-    "transpose-chars", F_CHARSWITCH,
-    "Exchange the character to the left of the cursor with the one under",
-    "transpose-gosling", F_GCHARSWITCH,
-    "Exchange the two characters before the cursor",
-    /* EGS: make Convex Users happy */
-    "tty-dsusp", F_TTY_DSUSP,
-    "Tty delayed suspend character",
-    "tty-flush-output", F_TTY_FLUSHO,
-    "Tty flush output character",
-    "tty-sigintr", F_TTY_INT,
-    "Tty interrupt character",
-    "tty-sigquit", F_TTY_QUIT,
-    "Tty quit character",
-    "tty-sigtsusp", F_TTY_TSUSP,
-    "Tty suspend character",
-    "tty-start-output", F_TTY_STARTO,
-    "Tty allow output character",
-    "tty-stop-output", F_TTY_STOPO,
-    "Tty disallow output character",
-    "undefined-key", F_UNASSIGNED,
-    "Indicates unbound character",
-    "universal-argument", F_ARGFOUR,
-    "Emacs universal argument (argument times 4)",
-    "up-history", F_UP_HIST,
-    "Move to previous history line",
-    "upcase-word", F_CASEUPPER,
-    "Uppercase the characters from cursor to end of current word",
-    "vi-beginning-of-next-word", V_WORDBEGNEXT,
-    "Vi goto the beginning of next word",
-    "vi-add", V_ADD,
-    "Vi enter insert mode after the cursor",
-    "vi-add-at-eol", V_ADDEND,
-    "Vi enter insert mode at end of line",
-    "vi-chg-case", V_CHGCASE,
-    "Vi change case of character under cursor and advance one character",
-    "vi-chg-meta", V_CHGMETA,
-    "Vi change prefix command",
-    "vi-chg-to-eol", V_CHGTOEND,
-    "Vi change to end of line",
-    "vi-cmd-mode", V_CMD_MODE,
-    "Enter vi command mode (use alternative key bindings)",
-    "vi-cmd-mode-complete", V_CM_COMPLETE,
-    "Vi command mode complete current word",
-    "vi-delprev", V_DELPREV,
-    "Vi move to previous character (backspace)",
-    "vi-delmeta", V_DELMETA,
-    "Vi delete prefix command",
-    "vi-endword", V_ENDWORD,
-    "Vi move to the end of the current space delimited word",
-    "vi-eword", V_EWORD,
-    "Vi move to the end of the current word",
-    "vi-char-back", V_CHAR_BACK,
-    "Vi move to the character specified backwards",
-    "vi-char-fwd", V_CHAR_FWD,
-    "Vi move to the character specified forward",
-    "vi-charto-back", V_CHARTO_BACK,
-    "Vi move up to the character specified backwards",
-    "vi-charto-fwd", V_CHARTO_FWD,
-    "Vi move up to the character specified forward",
-    "vi-insert", V_INSERT,
-    "Enter vi insert mode",
-    "vi-insert-at-bol", V_INSBEG,
-    "Enter vi insert mode at beginning of line",
-    "vi-repeat-char-fwd", V_RCHAR_FWD,
-    "Vi repeat current character search in the same search direction",
-    "vi-repeat-char-back", V_RCHAR_BACK,
-    "Vi repeat current character search in the opposite search direction",
-    "vi-repeat-search-fwd", V_RSRCH_FWD,
-    "Vi repeat current search in the same search direction",
-    "vi-repeat-search-back", V_RSRCH_BACK,
-    "Vi repeat current search in the opposite search direction",
-    "vi-replace-char", V_REPLONE,
-    "Vi replace character under the cursor with the next character typed",
-    "vi-replace-mode", V_REPLMODE,
-    "Vi replace mode",
-    "vi-search-back", V_USH_META,
-    "Vi search history backwards",
-    "vi-search-fwd", V_DSH_META,
-    "Vi search history forward",
-    "vi-substitute-char", V_SUBSTCHAR,
-    "Vi replace character under the cursor and enter insert mode",
-    "vi-substitute-line", V_SUBSTLINE,
-    "Vi replace entire line",
-    "vi-word-back", V_WORDBACK,
-    "Vi move to the previous word",
-    "vi-word-fwd", V_WORDFWD,
-    "Vi move to the next word",
-    "vi-undo", V_UNDO,
-    "Vi undo last change",
-    "vi-zero", V_ZERO,
-    "Vi goto the beginning of line",
-    "which-command", F_WHICH,
-    "Perform which of current command",
-    "yank", F_YANK_KILL,
-    "Paste cut buffer at cursor position",
-    0, 0
+    { "backward-char", F_CHARBACK,
+      "Move back a character" },
+    { "backward-delete-char", F_DELPREV,
+      "Delete the character behind cursor" },
+    { "backward-delete-word", F_DELWORDPREV,
+      "Cut from beginning of current word to cursor - saved in cut buffer" },
+    { "backward-kill-line", F_KILLBEG,
+      "Cut from beginning of line to cursor - save in cut buffer" },
+    { "backward-word", F_WORDBACK,
+      "Move to beginning of current word" },
+    { "beginning-of-line", F_TOBEG,
+      "Move to beginning of line" },
+    { "capitalize-word", F_CASECAPITAL,
+      "Capitalize the characters from cursor to end of current word" },
+    { "change-case", V_CHGCASE,
+      "Vi change case of character under cursor and advance one character" },
+    { "change-till-end-of-line", V_CHGTOEND,	/* backwards compat. */
+      "Vi change to end of line" },
+    { "clear-screen", F_CLEARDISP,
+      "Clear screen leaving current line on top" },
+    { "complete-word", F_COMPLETE,
+      "Complete current word" },
+    { "complete-word-fwd", F_COMPLETE_FWD,
+      "Tab forward through files" },
+    { "complete-word-back", F_COMPLETE_BACK,
+      "Tab backwards through files" },
+    { "complete-word-raw", F_COMPLETE_ALL,
+      "Complete current word ignoring programmable completions" },
+    { "copy-prev-word", F_COPYPREV,
+      "Copy current word to cursor" },
+    { "copy-region-as-kill", F_COPYREGION,
+      "Copy area between mark and cursor to cut buffer" },
+    { "delete-char", F_DELNEXT,
+      "Delete character under cursor" },
+    { "delete-char-or-eof", F_DELNEXT_EOF,
+      "Delete character under cursor or end of file if there is no character" },
+    { "delete-char-or-list", F_LIST_DELNEXT,
+      "Delete character under cursor or list completions if at end of line" },
+    { "delete-word", F_DELWORDNEXT,
+      "Cut from cursor to end of current word - save in cut buffer" },
+    { "digit", F_DIGIT,
+      "Adds to argument if started or enters digit" },
+    { "digit-argument", F_ARGDIGIT,
+      "Digit that starts argument" },
+    { "down-history", F_DOWN_HIST,
+      "Move to next history line" },
+    { "downcase-word", F_CASELOWER,
+      "Lowercase the characters from cursor to end of current word" },
+    { "end-of-file", F_SEND_EOF,
+      "Indicate end of file" },
+    { "end-of-line", F_TOEND,
+      "Move cursor to end of line" },
+    { "exchange-point-and-mark", F_EXCHANGE_MARK,
+      "Exchange the cursor and mark" },
+    { "expand-glob", F_EXPAND_GLOB,
+      "Expand file name wildcards" },
+    { "expand-history", F_EXPAND_HISTORY,
+      "Expand history escapes" },
+    { "expand-line", F_EXPAND,
+      "Expand the history escapes in a line" },
+    { "expand-variables", F_EXPAND_VARS,
+      "Expand variables" },
+    { "forward-char", F_CHARFWD,
+      "Move forward one character" },
+    { "forward-word", F_WORDFWD,
+      "Move forward to end of current word" },
+    { "gosmacs-transpose-chars", F_GCHARSWITCH,
+      "Exchange the two characters before the cursor" },
+    { "history-search-backward", F_UP_SEARCH_HIST,
+      "Search in history backwards for line beginning as current" },
+    { "history-search-forward", F_DOWN_SEARCH_HIST,
+      "Search in history forward for line beginning as current" },
+    { "insert-last-word", F_LAST_ITEM,
+      "Insert last item of previous command" },
+    { "i-search-fwd", F_INC_FWD,
+      "Incremental search forward" },
+    { "i-search-back", F_INC_BACK,
+      "Incremental search backwards" },
+    { "keyboard-quit", F_STARTOVER,
+      "Clear line" },
+    { "kill-line", F_KILLEND,
+      "Cut to end of line and save in cut buffer" },
+    { "kill-region", F_KILLREGION,
+      "Cut area between mark and cursor and save in cut buffer" },
+    { "kill-whole-line", F_KILLALL,
+      "Cut the entire line and save in cut buffer" },
+    { "list-choices", F_LIST_CHOICES,
+      "List choices for completion" },
+    { "list-choices-raw", F_LIST_ALL,
+      "List choices for completion overriding programmable completion" },
+    { "list-glob", F_LIST_GLOB,
+      "List file name wildcard matches" },
+    { "list-or-eof", F_LIST_EOF,
+      "List choices for completion or indicate end of file if empty line" },
+    { "load-average", F_LOAD_AVERAGE,
+      "Display load average and current process status" },
+    { "magic-space", F_MAGIC_SPACE,
+      "Expand history escapes and insert a space" },
+    { "newline", F_NEWLINE,
+      "Execute command" },
+    { "normalize-path", F_PATH_NORM,
+      "Expand pathnames, eliminating leading .'s and ..'s" },
+    { "overwrite-mode", F_INSOVR,
+      "Switch from insert to overwrite mode or vice versa" },
+    { "prefix-meta", F_METANEXT,
+      "Add 8th bit to next character typed" },
+    { "quoted-insert", F_QUOTE,
+      "Add the next character typed to the line verbatim" },
+    { "redisplay", F_REDISP,
+      "Redisplay everything" },
+    { "run-fg-editor", F_RUN_FG_EDITOR,
+      "Restart stopped editor" },
+    { "run-help", F_HELPME,
+      "Look for help on current command" },
+    { "self-insert-command", F_INSERT,
+      "This character is added to the line" },
+    { "sequence-lead-in", F_XKEY,
+      "This character is the first in a character sequence" },
+    { "set-mark-command", F_SET_MARK,
+      "Set the mark at cursor" },
+    { "spell-word", F_CORRECT,
+      "Correct the spelling of current word" },
+    { "spell-line", F_CORRECT_L,
+      "Correct the spelling of entire line" },
+    { "stuff-char", F_STUFF_CHAR,
+      "Send character to tty in cooked mode" },
+    { "toggle-literal-history", F_TOGGLE_HIST,
+      "Toggle between literal and lexical current history line" },
+    { "transpose-chars", F_CHARSWITCH,
+      "Exchange the character to the left of the cursor with the one under" },
+    { "transpose-gosling", F_GCHARSWITCH,
+      "Exchange the two characters before the cursor" },
+    { "tty-dsusp", F_TTY_DSUSP,
+      "Tty delayed suspend character" },
+    { "tty-flush-output", F_TTY_FLUSHO,
+      "Tty flush output character" },
+    { "tty-sigintr", F_TTY_INT,
+      "Tty interrupt character" },
+    { "tty-sigquit", F_TTY_QUIT,
+      "Tty quit character" },
+    { "tty-sigtsusp", F_TTY_TSUSP,
+      "Tty suspend character" },
+    { "tty-start-output", F_TTY_STARTO,
+      "Tty allow output character" },
+    { "tty-stop-output", F_TTY_STOPO,
+      "Tty disallow output character" },
+    { "undefined-key", F_UNASSIGNED,
+      "Indicates unbound character" },
+    { "universal-argument", F_ARGFOUR,
+      "Emacs universal argument (argument times 4)" },
+    { "up-history", F_UP_HIST,
+      "Move to previous history line" },
+    { "upcase-word", F_CASEUPPER,
+      "Uppercase the characters from cursor to end of current word" },
+    { "vi-beginning-of-next-word", V_WORDBEGNEXT,
+      "Vi goto the beginning of next word" },
+    { "vi-add", V_ADD,
+      "Vi enter insert mode after the cursor" },
+    { "vi-add-at-eol", V_ADDEND,
+      "Vi enter insert mode at end of line" },
+    { "vi-chg-case", V_CHGCASE,
+      "Vi change case of character under cursor and advance one character" },
+    { "vi-chg-meta", V_CHGMETA,
+      "Vi change prefix command" },
+    { "vi-chg-to-eol", V_CHGTOEND,
+      "Vi change to end of line" },
+    { "vi-cmd-mode", V_CMD_MODE,
+      "Enter vi command mode (use alternative key bindings)" },
+    { "vi-cmd-mode-complete", V_CM_COMPLETE,
+      "Vi command mode complete current word" },
+    { "vi-delprev", V_DELPREV,
+      "Vi move to previous character (backspace)" },
+    { "vi-delmeta", V_DELMETA,
+      "Vi delete prefix command" },
+    { "vi-endword", V_ENDWORD,
+      "Vi move to the end of the current space delimited word" },
+    { "vi-eword", V_EWORD,
+      "Vi move to the end of the current word" },
+    { "vi-char-back", V_CHAR_BACK,
+      "Vi move to the character specified backwards" },
+    { "vi-char-fwd", V_CHAR_FWD,
+      "Vi move to the character specified forward" },
+    { "vi-charto-back", V_CHARTO_BACK,
+      "Vi move up to the character specified backwards" },
+    { "vi-charto-fwd", V_CHARTO_FWD,
+      "Vi move up to the character specified forward" },
+    { "vi-insert", V_INSERT,
+      "Enter vi insert mode" },
+    { "vi-insert-at-bol", V_INSBEG,
+      "Enter vi insert mode at beginning of line" },
+    { "vi-repeat-char-fwd", V_RCHAR_FWD,
+      "Vi repeat current character search in the same search direction" },
+    { "vi-repeat-char-back", V_RCHAR_BACK,
+      "Vi repeat current character search in the opposite search direction" },
+    { "vi-repeat-search-fwd", V_RSRCH_FWD,
+      "Vi repeat current search in the same search direction" },
+    { "vi-repeat-search-back", V_RSRCH_BACK,
+      "Vi repeat current search in the opposite search direction" },
+    { "vi-replace-char", V_REPLONE,
+      "Vi replace character under the cursor with the next character typed" },
+    { "vi-replace-mode", V_REPLMODE,
+      "Vi replace mode" },
+    { "vi-search-back", V_USH_META,
+      "Vi search history backwards" },
+    { "vi-search-fwd", V_DSH_META,
+      "Vi search history forward" },
+    { "vi-substitute-char", V_SUBSTCHAR,
+      "Vi replace character under the cursor and enter insert mode" },
+    { "vi-substitute-line", V_SUBSTLINE,
+      "Vi replace entire line" },
+    { "vi-word-back", V_WORDBACK,
+      "Vi move to the previous word" },
+    { "vi-word-fwd", V_WORDFWD,
+      "Vi move to the next word" },
+    { "vi-undo", V_UNDO,
+      "Vi undo last change" },
+    { "vi-zero", V_ZERO,
+      "Vi goto the beginning of line" },
+    { "which-command", F_WHICH,
+      "Perform which of current command" },
+    { "yank", F_YANK_KILL,
+      "Paste cut buffer at cursor position" },
+    { NULL, 0,
+      NULL }
 };
 
 #ifdef DEBUG_EDIT
 void
 CheckMaps()
-{				/* check the size of the key maps */
+{		/* check the size of the key maps */
     int     c1 = (256 * sizeof(KEYCMD));
 
     if ((sizeof(CcKeyMap)) != c1)
