@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.02/RCS/ed.init.c,v 3.27 1992/04/03 22:15:14 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.04/RCS/ed.init.c,v 3.36 1993/06/25 21:17:12 christos Exp $ */
 /*
  * ed.init.c: Editor initializations
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: ed.init.c,v 3.27 1992/04/03 22:15:14 christos Exp $")
+RCSID("$Id: ed.init.c,v 3.36 1993/06/25 21:17:12 christos Exp $")
 
 #include "ed.h"
 #include "ed.term.h"
@@ -232,9 +232,9 @@ ed_Setup(rst)
     extty.d_t.c_lflag &= ~ttylist[EX_IO][M_LINED].t_clrmask;
     extty.d_t.c_lflag |=  ttylist[EX_IO][M_LINED].t_setmask;
 
-# ifdef IRIX3_3
+# if defined(IRIX3_3) && SYSVREL < 4
     extty.d_t.c_line = NTTYDISC;
-# endif /* IRIX3_3 */
+# endif /* IRIX3_3 && SYSVREL < 4 */
 
 #else	/* GSTTY */		/* V7, Berkeley style tty */
 
@@ -291,7 +291,7 @@ ed_Setup(rst)
 void
 ed_Init()
 {
-    ResetInLine();		/* reset the input pointers */
+    ResetInLine(1);		/* reset the input pointers */
     GettingInput = 0;		/* just in case */
     LastKill = KillBuf;		/* no kill buffer */
 
@@ -325,9 +325,9 @@ ed_Init()
     edtty.d_t.c_lflag |=  ttylist[ED_IO][M_LINED].t_setmask;
 
 
-# ifdef IRIX3_3
+# if defined(IRIX3_3) && SYSVREL < 4
     edtty.d_t.c_line = NTTYDISC;
-# endif /* IRIX3_3 */
+# endif /* IRIX3_3 && SYSVREL < 4 */
 
 #else /* GSTTY */
 
@@ -545,7 +545,7 @@ Rawmode()
 #ifdef DEBUG_TTY
 	xprintf("Rawmode: tty_setty: %s\n", strerror(errno));
 #endif /* DEBUG_TTY */
-	return -1;
+	return(-1);
     }
     Tty_raw_mode = 1;
     flush();			/* flush any buffered output */
@@ -568,7 +568,30 @@ Cookedmode()
 #ifdef BSDSIGS
     orig_intr = (sigret_t (*)()) signal(SIGINT, SIG_IGN);
 #else
+# ifdef SIG_HOLD
+    /*
+     * sigset doesn't return the previous handler if the signal is held,
+     * it will return SIG_HOLD instead. So instead of restoring the
+     * the signal we would end up installing a blocked SIGINT with a
+     * SIG_IGN signal handler. This is what happened when Cookedmode
+     * was called from sched_run, disabling interrupt for the rest
+     * of your session.
+     *
+     * This is what we do:
+     * - if the signal is blocked, keep it that way
+     * - else set it to SIG_IGN
+     *
+     * Casper Dik (casper@fwi.uva.nl)
+     */
+    orig_intr = (sigret_t (*)()) sigset(SIGINT, SIG_HOLD);
+    if (orig_intr != SIG_HOLD)
+	(void) sigset(SIGINT, SIG_IGN); /* returns SIG_HOLD */
+# else /* !SIG_HOLD */
+    /*
+     * No SIG_HOLD; probably no reliable signals as well.
+     */
     orig_intr = (sigret_t (*)()) sigset(SIGINT, SIG_IGN);
+# endif /* SIG_HOLD */
 #endif /* BSDSIGS */
     if (tty_setty(SHTTY, &extty) == -1) {
 #ifdef DEBUG_TTY
@@ -586,7 +609,8 @@ Cookedmode()
 }
 
 void
-ResetInLine()
+ResetInLine(macro)
+    int macro;
 {
     Cursor = InputBuf;		/* reset cursor */
     LastChar = InputBuf;
@@ -602,14 +626,23 @@ ResetInLine()
     LastKill = KillBuf;		/* no kill buffer */
 #endif 
     LastCmd = F_UNASSIGNED;	/* previous command executed */
-    MacroLvl = -1;		/* no currently active macros */
+    if (macro)
+	MacroLvl = -1;		/* no currently active macros */
 }
 
 static Char *Input_Line = NULL;
 int
 Load_input_line()
 {
-    long    chrs = 0;
+#ifdef SUNOS4
+    long chrs = 0;
+#else /* !SUNOS4 */
+    /* 
+     * *Everyone* else has an int, but SunOS wants long!
+     * This breaks where int != long (alpha)
+     */
+    int chrs = 0;
+#endif /* SUNOS4 */
 
     if (Input_Line)
 	xfree((ptr_t) Input_Line);
@@ -618,8 +651,8 @@ Load_input_line()
     if (Tty_raw_mode)
 	return 0;
 
-#ifdef FIONREAD
-    (void) ioctl(SHIN, FIONREAD, &chrs);
+#if defined(FIONREAD) && !defined(OREO)
+    (void) ioctl(SHIN, FIONREAD, (ioctl_t) &chrs);
     if (chrs > 0) {
 	char    buf[BUFSIZE];
 
@@ -630,7 +663,7 @@ Load_input_line()
 	    PushMacro(Input_Line);
 	}
     }
-#endif  /* FIONREAD */
+#endif  /* FIONREAD && !OREO */
     return chrs > 0;
 }
 
