@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.02/RCS/ed.inputl.c,v 3.21 1992/04/10 16:45:27 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.04/RCS/ed.inputl.c,v 3.35 1993/06/25 21:17:12 christos Exp $ */
 /*
  * ed.inputl.c: Input line handling.
  */
@@ -36,15 +36,13 @@
  */
 #include "sh.h"
 
-RCSID("$Id: ed.inputl.c,v 3.21 1992/04/10 16:45:27 christos Exp $")
+RCSID("$Id: ed.inputl.c,v 3.35 1993/06/25 21:17:12 christos Exp $")
 
 #include "ed.h"
 #include "ed.defns.h"		/* for the function names */
 #include "tw.h"			/* for twenex stuff */
 
 #define OKCMD (INBUFSIZE+INBUFSIZE)
-extern CCRETVAL e_up_hist();
-extern CCRETVAL e_expand_history();
 
 /* ed.inputl -- routines to get a single line from the input. */
 
@@ -60,6 +58,8 @@ static	int	SpellLine	__P((int));
 static	void	RunCommand	__P((Char *));
 static void 	doeval1		__P((Char **));
 
+static bool rotate = 0;
+
 /* CCRETVAL */
 int
 Inputl()
@@ -78,18 +78,37 @@ Inputl()
     Char   *SaveChar, *CorrChar;
     Char    Origin[INBUFSIZE], Change[INBUFSIZE];
     int     matchval;		/* from tenematch() */
+    COMMAND fn;
+    int curlen = 0;
+    int newlen;
+    int idx;
 
     if (!MapsAreInited)		/* double extra just in case */
 	ed_InitMaps();
 
     ClearDisp();		/* reset the display stuff */
-    ResetInLine();		/* reset the input pointers */
+    ResetInLine(0);		/* reset the input pointers */
     if (GettingInput)
 	MacroLvl = -1;		/* editor was interrupted during input */
 
-#ifdef FIONREAD
+    if (imode) {
+	if (!Strcmp(*(imode->vec), STRinsert))
+	    inputmode = MODE_INSERT;
+	else if (!Strcmp(*(imode->vec), STRoverwrite))
+	    inputmode = MODE_REPLACE;
+    }
+
+#if defined(FIONREAD) && !defined(OREO)
     if (!Tty_raw_mode && MacroLvl < 0) {
-	long    chrs = 0;
+# ifdef SUNOS4
+	long chrs = 0;
+# else /* !SUNOS4 */
+	/* 
+	 * *Everyone* else has an int, but SunOS wants long!
+	 * This breaks where int != long (alpha)
+	 */
+	int chrs = 0;
+# endif /* SUNOS4 */
 
 	(void) ioctl(SHIN, FIONREAD, (ioctl_t) & chrs);
 	if (chrs == 0) {
@@ -97,7 +116,7 @@ Inputl()
 		return 0;
 	}
     }
-#endif
+#endif /* FIONREAD && !OREO */
 
     GettingInput = 1;
     NeedsRedraw = 0;
@@ -163,12 +182,13 @@ Inputl()
 	    /*FALLTHROUGH*/
 	case CC_ARGHACK:	/* Suggested by Rich Salz */
 	    /* <rsalz@pineapple.bbn.com> */
+	    curchoice = -1;
+	    curlen = LastChar - InputBuf;
 	    break;		/* keep going... */
 
 	case CC_EOF:		/* end of file typed */
-#ifdef notdef
-	    PromptBuf[0] = '\0';
-#endif
+	    curchoice = -1;
+	    curlen = LastChar - InputBuf;
 	    num = 0;
 	    break;
 
@@ -181,12 +201,11 @@ Inputl()
 	    HistWhich = Hist_num;
 	    Hist_num = 0;	/* for the history commands */
 	    num = LastChar - InputBuf;	/* number characters read */
-#ifdef notdef
-	    ResetInLine();	/* reset the input pointers */
-#endif
 	    break;
 
 	case CC_NEWLINE:	/* normal end of line */
+	    curlen = 0;
+	    curchoice = -1;
 	    if (crct && (!Strcmp(*(crct->vec), STRcmd) ||
 			 !Strcmp(*(crct->vec), STRall))) {
 		copyn(Origin, InputBuf, INBUFSIZE);
@@ -213,6 +232,16 @@ Inputl()
 			    *LastChar-- = '\0';
 			    Cursor = LastChar;
 			    printprompt(3, NULL);
+			    ClearLines();
+			    ClearDisp();
+			    Refresh();
+			    break;
+			}
+			else if (ch == 'a') {
+			    xprintf("abort\n");
+			    *LastChar = '\0';
+			    Cursor = LastChar;
+			    printprompt(0, NULL);
 			    Refresh();
 			    break;
 			}
@@ -227,17 +256,7 @@ Inputl()
 	    /*
 	     * For continuation lines, we set the prompt to prompt 2
 	     */
-	    if (imode) {
-		if (!Strcmp(*(imode->vec), STRinsert))
-		    inputmode = MODE_INSERT;
-		else if (!Strcmp(*(imode->vec), STRoverwrite))
-		    inputmode = MODE_REPLACE;
-	    }
 	    printprompt(1, NULL);
-#ifdef notdef
-	    ResetInLine();	/* reset the input pointers */
-	    PromptBuf[0] = '\0';
-#endif
 	    break;
 
 	case CC_CORRECT:
@@ -251,6 +270,8 @@ Inputl()
 	    Refresh();
 	    Argument = 1;
 	    DoingArg = 0;
+	    curchoice = -1;
+	    curlen = LastChar - InputBuf;
 	    break;
 
 	case CC_CORRECT_L:
@@ -264,10 +285,54 @@ Inputl()
 	    Refresh();
 	    Argument = 1;
 	    DoingArg = 0;
+	    curchoice = -1;
+	    curlen = LastChar - InputBuf;
 	    break;
 
 
 	case CC_COMPLETE:
+	case CC_COMPLETE_ALL:
+	case CC_COMPLETE_FWD:
+	case CC_COMPLETE_BACK:
+	    expnum = Cursor - InputBuf;
+	    switch (retval) {
+	    case CC_COMPLETE:
+		fn = RECOGNIZE;
+		curlen = LastChar - InputBuf;
+		curchoice = -1;
+		rotate = 0;
+		break;
+	    case CC_COMPLETE_ALL:
+		fn = RECOGNIZE_ALL;
+		curlen = LastChar - InputBuf;
+		curchoice = -1;
+		rotate = 0;
+		break;
+	    case CC_COMPLETE_FWD:
+		fn = RECOGNIZE_SCROLL;
+		curchoice++;
+		rotate = 1;
+		break;
+	    case CC_COMPLETE_BACK:
+		fn = RECOGNIZE_SCROLL;
+		curchoice--;
+		rotate = 1;
+		break;
+	    default:
+		abort();
+	    }
+	    if (InputBuf[curlen] && rotate) {
+		newlen = LastChar - InputBuf;
+		for (idx = (Cursor - InputBuf); 
+		     idx <= newlen; idx++)
+			InputBuf[idx - newlen + curlen] =
+			InputBuf[idx];
+		LastChar = InputBuf + curlen;
+		Cursor = Cursor - newlen + curlen;
+	    }
+	    curlen = LastChar - InputBuf;
+
+
 	    if (adrof(STRautoexpand))
 		(void) e_expand_history(0);
 	    /*
@@ -276,7 +341,7 @@ Inputl()
 	     * completion, independently of autolisting.
 	     */
 	    expnum = Cursor - InputBuf;
-	    switch (matchval = tenematch(InputBuf, Cursor-InputBuf, RECOGNIZE)){
+	    switch (matchval = tenematch(InputBuf, Cursor-InputBuf, fn)){
 	    case 1:
 		if (non_unique_match && matchbeep &&
 		    (Strcmp(*(matchbeep->vec), STRnotunique) == 0))
@@ -294,6 +359,7 @@ Inputl()
 		break;
 	    default:
 		if (matchval < 0) {	/* Error from tenematch */
+		    curchoice = -1;
 		    Beep();
 		    break;
 		}
@@ -313,7 +379,8 @@ Inputl()
 		if (autol && (Strcmp(*(autol->vec), STRambiguous) != 0 || 
 				     expnum == Cursor - InputBuf)) {
 		    PastBottom();
-		    (void) tenematch(InputBuf, Cursor-InputBuf, LIST);
+		    fn = (retval == CC_COMPLETE_ALL) ? LIST_ALL : LIST;
+		    (void) tenematch(InputBuf, Cursor-InputBuf, fn);
 		}
 		break;
 	    }
@@ -329,20 +396,38 @@ Inputl()
 	    break;
 
 	case CC_LIST_CHOICES:
+	case CC_LIST_ALL:
+	    if (InputBuf[curlen] && rotate) {
+		newlen = LastChar - InputBuf;
+		for (idx = (Cursor - InputBuf); 
+		     idx <= newlen; idx++)
+			InputBuf[idx - newlen + curlen] =
+			InputBuf[idx];
+		LastChar = InputBuf + curlen;
+		Cursor = Cursor - newlen + curlen;
+	    }
+	    curlen = LastChar - InputBuf;
+	    if (curchoice >= 0)
+		curchoice--;
+
+	    fn = (retval == CC_LIST_ALL) ? LIST_ALL : LIST;
 	    /* should catch ^C here... */
-	    if (tenematch(InputBuf, Cursor - InputBuf, LIST) < 0)
+	    if (tenematch(InputBuf, Cursor - InputBuf, fn) < 0)
 		Beep();
 	    Refresh();
 	    Argument = 1;
 	    DoingArg = 0;
 	    break;
 
+
 	case CC_LIST_GLOB:
 	    if (tenematch(InputBuf, Cursor - InputBuf, GLOB) < 0)
 		Beep();
 	    Refresh();
 	    Argument = 1;
 	    DoingArg = 0;
+	    curchoice = -1;
+	    curlen = LastChar - InputBuf;
 	    break;
 
 	case CC_EXPAND_GLOB:
@@ -356,6 +441,8 @@ Inputl()
 	    Refresh();
 	    Argument = 1;
 	    DoingArg = 0;
+	    curchoice = -1;
+	    curlen = LastChar - InputBuf;
 	    break;
 
 	case CC_NORMALIZE_PATH:
@@ -369,6 +456,8 @@ Inputl()
 	    Refresh();
 	    Argument = 1;
 	    DoingArg = 0;
+	    curchoice = -1;
+	    curlen = LastChar - InputBuf;
 	    break;
 
 	case CC_EXPAND_VARS:
@@ -382,6 +471,8 @@ Inputl()
 	    Refresh();
 	    Argument = 1;
 	    DoingArg = 0;
+	    curchoice = -1;
+	    curlen = LastChar - InputBuf;
 	    break;
 
 	case CC_HELPME:
@@ -391,6 +482,8 @@ Inputl()
 	    Refresh();
 	    Argument = 1;
 	    DoingArg = 0;
+	    curchoice = -1;
+	    curlen = LastChar - InputBuf;
 	    break;
 
 	case CC_FATAL:		/* fatal error, reset to known state */
@@ -399,10 +492,12 @@ Inputl()
 #endif				/* DEBUG_EDIT */
 	    /* put (real) cursor in a known place */
 	    ClearDisp();	/* reset the display stuff */
-	    ResetInLine();	/* reset the input pointers */
+	    ResetInLine(1);	/* reset the input pointers */
 	    Refresh();		/* print the prompt again */
 	    Argument = 1;
 	    DoingArg = 0;
+	    curchoice = -1;
+	    curlen = LastChar - InputBuf;
 	    break;
 
 	case CC_ERROR:
@@ -411,6 +506,8 @@ Inputl()
 	    Argument = 1;
 	    Beep();
 	    flush();
+	    curchoice = -1;
+	    curlen = LastChar - InputBuf;
 	    break;
 	}
     }
@@ -447,7 +544,7 @@ doeval1(v)
     Char   *oevalp;
     int     my_reenter;
     Char  **savegv;
-    jmp_buf osetexit;
+    jmp_buf_t osetexit;
 
     oevalvec = evalvec;
     oevalp = evalp;
@@ -508,11 +605,17 @@ RunCommand(str)
     cmd[0] = str;
     cmd[1] = NULL;
 
+    (void) Cookedmode();
+    GettingInput = 0;
+
     doeval1(cmd);
     
+    (void) Rawmode();
+    GettingInput = 1;
+
     ClearLines();
     ClearDisp();
-    NeedsRedraw = 1;
+    NeedsRedraw = 0;
     Refresh();
 }
 
@@ -531,7 +634,7 @@ GetNextCommand(cmdnum, ch)
 #ifdef	KANJI
 	if (!adrof(STRnokanji) && (*ch & META)) {
 	    MetaNext = 0;
-	    cmd = CcViMap[' '];
+	    cmd = F_INSERT;
 	    break;
 	}
 	else
@@ -570,12 +673,7 @@ GetNextChar(cp)
     register Char *cp;
 {
     register int num_read;
-#if defined(EWOULDBLOCK) || (defined(POSIX) && defined(EAGAIN))
-# if defined(FIONBIO) || (defined(F_SETFL) && defined(O_NDELAY))
-#  define TRY_AGAIN
     int     tried = 0;
-# endif /* FIONBIO || (F_SETFL && O_NDELAY) */
-#endif /* EWOULDBLOCK || (POSIX && EAGAIN) */
     unsigned char tcp;
 
     for (;;) {
@@ -597,44 +695,16 @@ GetNextChar(cp)
     if (Rawmode() < 0)		/* make sure the tty is set up correctly */
 	return 0;		/* oops: SHIN was closed */
 
-    while ((num_read = read(SHIN, (char *) &tcp, 1)) == -1)
-	switch (errno) {
-	    /*
-	     * Someone might have set our file descriptor to non blocking From
-	     * Gray Watson (gray%antr.uucp@med.pitt.edu), Thanks!!!
-	     */
-#ifdef EWOULDBLOCK
-	case EWOULDBLOCK:
-#endif /* EWOULDBLOCK */
-#if defined(POSIX) && defined(EAGAIN)
-# if defined(EWOULDBLOCK) && EAGAIN != EWOULDBLOCK
-	case EAGAIN:
-# endif /* EWOULDBLOCK && EAGAIN != EWOULDBLOCK */
-#endif /* POSIX && EAGAIN */
-#ifdef TRY_AGAIN
-	    if (!tried) {
-# if defined(F_SETFL) && defined(O_NDELAY)
-		(void) fcntl(SHIN, F_SETFL,
-			     fcntl(SHIN, F_GETFL, 0) & ~O_NDELAY);
-# endif /* F_SETFL && O_NDELAY */
-# ifdef FIONBIO
-		(void) ioctl(SHIN, FIONBIO, (ioctl_t) & tried);
-# endif /* FIONBIO */
-		tried = 1;
-		break;
-	    }
-	    *cp = tcp;
-	    return (num_read);
-#endif /* TRY_AGAIN */
-	case EINTR:
-	    break;
-	default:
-#ifdef DEBUG_EDIT
-	    xprintf("GetNextChar(): errno == %d\n", errno);
-#endif /* DEBUG_EDIT */
-	    *cp = tcp;
-	    return num_read;
+    while ((num_read = read(SHIN, (char *) &tcp, 1)) == -1) {
+	if (errno == EINTR)
+	    continue;
+	if (!tried && fixio(SHIN, errno) != -1)
+	    tried = 1;
+	else {
+	    *cp = '\0';
+	    return -1;
 	}
+    }
     *cp = tcp;
     return num_read;
 }
@@ -698,3 +768,4 @@ SpellLine(cmdonly)
     Cursor = OldCursor;
     return matchval;
 }
+
