@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.02/RCS/ed.screen.c,v 3.22 1992/05/11 14:23:58 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.04/RCS/ed.screen.c,v 3.30 1993/07/03 23:47:53 christos Exp $ */
 /*
  * ed.screen.c: Editor/termcap-curses interface
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: ed.screen.c,v 3.22 1992/05/11 14:23:58 christos Exp $")
+RCSID("$Id: ed.screen.c,v 3.30 1993/07/03 23:47:53 christos Exp $")
 
 #include "ed.h"
 #include "tc.h"
@@ -47,7 +47,7 @@ RCSID("$Id: ed.screen.c,v 3.22 1992/05/11 14:23:58 christos Exp $")
  */
 extern char *tgoto();
 extern char *tgetstr();
-extern char *tputs();
+extern int tputs();
 extern int tgetent();
 extern int tgetflag();
 extern int tgetnum();
@@ -315,7 +315,7 @@ TCalloc(t, cap)
 		continue;
 	    termbuf[tlen++] = '\0';
 	}
-    copy(termcap_alloc, termbuf, TC_BUFSIZE);
+    (void) memmove((ptr_t) termcap_alloc, (ptr_t) termbuf, TC_BUFSIZE);
     tloc = tlen;
     if (tloc + 3 >= TC_BUFSIZE) {
 	stderror(ERR_NAME | ERR_TCNOSTR);
@@ -334,6 +334,7 @@ TellTC(what)
 {
     struct termcapstr *t;
 
+    USE(what);
     xprintf("\n\tTcsh thinks your terminal has the\n");
     xprintf("\tfollowing characteristics:\n\n");
     xprintf("\tIt has %d columns and %d lines\n",
@@ -437,18 +438,18 @@ SetTC(what, how)
 		stderror(ERR_SETTCUS, tv->name);
 		return;
 	    }
-	    T_Tabs = Val(T_pt);
-	    T_HasMeta = Val(T_km);
-	    T_Margin = Val(T_am) ? MARGIN_AUTO : 0;
-	    T_Margin |= Val(T_xn) ? MARGIN_MAGIC : 0;
+	    T_Tabs = (Char) Val(T_pt);
+	    T_HasMeta = (Char) Val(T_km);
+	    T_Margin = (Char) Val(T_am) ? MARGIN_AUTO : 0;
+	    T_Margin |= (Char) Val(T_xn) ? MARGIN_MAGIC : 0;
 	    if (tv == &tval[T_am] || tv == &tval[T_xn]) 
 		ChangeSize(Val(T_li), Val(T_co));
 	    return;
 	}
 	else {
 	    tv->val = atoi(how);
-	    T_Cols = Val(T_co);
-	    T_Lines = Val(T_li);
+	    T_Cols = (Char) Val(T_co);
+	    T_Lines = (Char) Val(T_li);
 	    if (tv == &tval[T_co] || tv == &tval[T_li])
 		ChangeSize(Val(T_li), Val(T_co));
 	    return;
@@ -563,6 +564,10 @@ EchoTC(v)
     if (t->name == NULL)
 	scap = tgetstr(cv, &area);
     if (!scap || scap[0] == '\0') {
+	if (tgetflag(cv)) {
+	    xprintf("yes\n");
+	    return;
+	}
 	if (silent)
 	    return;
 	else
@@ -667,29 +672,132 @@ EchoTC(v)
 
 bool    GotTermCaps = 0;
 
+static struct {
+    Char   *name;
+    int     key;
+    XmapVal fun;
+    int	    type;
+} arrow[] = {
+#define A_K_DN	0
+    { STRdown,	T_kd },
+#define A_K_UP	1
+    { STRup,	T_ku },
+#define A_K_LT	2
+    { STRleft,	T_kl },
+#define A_K_RT	3
+    { STRright,	T_kr }
+};
+
+
+void
+ResetArrowKeys()
+{
+    arrow[A_K_DN].fun.cmd = F_DOWN_HIST;
+    arrow[A_K_DN].type    = XK_CMD;
+
+    arrow[A_K_UP].fun.cmd = F_UP_HIST;
+    arrow[A_K_UP].type    = XK_CMD;
+
+    arrow[A_K_LT].fun.cmd = F_CHARBACK;
+    arrow[A_K_LT].type    = XK_CMD;
+
+    arrow[A_K_RT].fun.cmd = F_CHARFWD;
+    arrow[A_K_RT].type    = XK_CMD;
+
+}
+
+void
+DefaultArrowKeys() 
+{
+    static Char strA[] = {033, '[', 'A', '\0'};
+    static Char strB[] = {033, '[', 'B', '\0'};
+    static Char strC[] = {033, '[', 'C', '\0'};
+    static Char strD[] = {033, '[', 'D', '\0'};
+    static Char stOA[] = {033, 'O', 'A', '\0'};
+    static Char stOB[] = {033, 'O', 'B', '\0'};
+    static Char stOC[] = {033, 'O', 'C', '\0'};
+    static Char stOD[] = {033, 'O', 'D', '\0'};
+
+    AddXkey(strA, &arrow[A_K_UP].fun, arrow[A_K_UP].type);
+    AddXkey(strB, &arrow[A_K_DN].fun, arrow[A_K_DN].type);
+    AddXkey(strC, &arrow[A_K_RT].fun, arrow[A_K_RT].type);
+    AddXkey(strD, &arrow[A_K_LT].fun, arrow[A_K_LT].type);
+    AddXkey(stOA, &arrow[A_K_UP].fun, arrow[A_K_UP].type);
+    AddXkey(stOB, &arrow[A_K_DN].fun, arrow[A_K_DN].type);
+    AddXkey(stOC, &arrow[A_K_RT].fun, arrow[A_K_RT].type);
+    AddXkey(stOD, &arrow[A_K_LT].fun, arrow[A_K_LT].type);
+    if (VImode) {
+	AddXkey(&strA[1], &arrow[A_K_UP].fun, arrow[A_K_UP].type);
+	AddXkey(&strB[1], &arrow[A_K_DN].fun, arrow[A_K_DN].type);
+	AddXkey(&strC[1], &arrow[A_K_RT].fun, arrow[A_K_RT].type);
+	AddXkey(&strD[1], &arrow[A_K_LT].fun, arrow[A_K_LT].type);
+	AddXkey(&stOA[1], &arrow[A_K_UP].fun, arrow[A_K_UP].type);
+	AddXkey(&stOB[1], &arrow[A_K_DN].fun, arrow[A_K_DN].type);
+	AddXkey(&stOC[1], &arrow[A_K_RT].fun, arrow[A_K_RT].type);
+	AddXkey(&stOD[1], &arrow[A_K_LT].fun, arrow[A_K_LT].type);
+    }
+}
+
+
+int
+SetArrowKeys(name, fun, type)
+    Char *name;
+    XmapVal *fun;
+    int type;
+{
+    int i;
+    for (i = 0; i < 4; i++)
+	if (Strcmp(name, arrow[i].name) == 0) {
+	    arrow[i].fun  = *fun;
+	    arrow[i].type = type;
+	    return 0;
+	}
+    return -1;
+}
+
+
+int
+ClearArrowKeys(name)
+    Char *name;
+{
+    int i;
+    for (i = 0; i < 4; i++)
+	if (Strcmp(name, arrow[i].name) == 0) {
+	    arrow[i].type = XK_NOD;
+	    return 0;
+	}
+    return -1;
+}
+
+void
+PrintArrowKeys(name)
+    Char *name;
+{
+    int i;
+
+    for (i = 0; i < 4; i++)
+	if (name == STRNULL || Strcmp(name, arrow[i].name) == 0)
+	    if (arrow[i].type != XK_NOD)
+		(void) printOne(arrow[i].name, &arrow[i].fun, arrow[i].type);
+}
+
+
 void
 BindArrowKeys()
 {
     KEYCMD *map, *dmap;
     int     i, j;
     char   *p;
-    static struct {
-	int     key, fun;
-    }       ar[] =
-    {
-	{ T_kd, F_DOWN_HIST },
-	{ T_ku, F_UP_HIST   },
-	{ T_kl, F_CHARBACK  },
-	{ T_kr, F_CHARFWD   }
-    };
 
     if (!GotTermCaps)
 	return;
     map = VImode ? CcAltMap : CcKeyMap;
     dmap = VImode ? CcViCmdMap : CcEmacsMap;
 
+    DefaultArrowKeys();
+
     for (i = 0; i < 4; i++) {
-	p = tstr[ar[i].key].str;
+	p = tstr[arrow[i].key].str;
 	if (p && *p) {
 	    j = (unsigned char) *p;
 	    /*
@@ -700,13 +808,20 @@ BindArrowKeys()
 	     *    has re-assigned the leading character to be F_XKEY
 	     * 2. They are single arrow keys pointing to an unassigned key.
 	     */
-	    if (p[1] && (dmap[j] == map[j] || map[j] == F_XKEY)) {
-		AddXkey(str2short(p), XmapCmd(ar[i].fun), XK_CMD);
-		map[j] = F_XKEY;
-	    }
-	    else if (map[j] == F_UNASSIGNED) {
+	    if (arrow[i].type == XK_NOD)
 		ClearXkey(map, str2short(p));
-		map[j] = ar[i].fun;
+	    else {
+		if (p[1] && (dmap[j] == map[j] || map[j] == F_XKEY)) {
+		    AddXkey(str2short(p), &arrow[i].fun, arrow[i].type);
+		    map[j] = F_XKEY;
+		}
+		else if (map[j] == F_UNASSIGNED) {
+		    ClearXkey(map, str2short(p));
+		    if (arrow[i].type == XK_CMD)
+			map[j] = arrow[i].fun.cmd;
+		    else
+			AddXkey(str2short(p), &arrow[i].fun, arrow[i].type);
+		}
 	    }
 	}
     }
@@ -878,11 +993,10 @@ mc_again:
 			for (i = (CursorH & 0370); i < (where & 0370); i += 8)
 			    (void) putraw('\t');	/* then tab over */
 			CursorH = where & 0370;
-			if (CursorH < where && where == (TermH - 1)) {
-			    /* optimize: we can tab to the last column */
-			    (void) putraw('\t');
-			    CursorH = where;
-			}
+			/* Note: considering that we often want to go to
+			   TermH - 1 for the wrapping, it would be nice to
+			   optimize this case by tabbing to the last column
+			   - but this doesn't work for all terminals! */
 		    }
 		}
 		/* it's usually cheaper to just write the chars, so we do. */
@@ -956,7 +1070,7 @@ so_write(cp, n)
 		if ((c = Display[CursorV][CursorH]) != '\0')
 		    so_write(&c, 1);
 		else
-		    putraw(' ');
+		    (void) putraw(' ');
 		CursorH = 1;
 	    }
 	}
@@ -1072,7 +1186,7 @@ ClearEOL(num)			/* clear to end of line.  There are num */
 {
     register int i;
 
-    if (num == 0)
+    if (num <= 0)
 	return;
 
     if (T_CanCEOL && GoodStr(T_ce))
@@ -1167,7 +1281,7 @@ GetTermCaps()
 	ptr = "dumb";
 #endif /* apollo */
 
-    if (!ptr || !ptr[0])
+    if (!ptr || !ptr[0] || !strcmp(ptr, "wm"))
 	ptr = "dumb";
 
     setzero(bp, TC_BUFSIZE);
@@ -1206,13 +1320,13 @@ GetTermCaps()
     if (Val(T_li) < 1)
 	Val(T_li) = 24;
 
-    T_Cols = Val(T_co);
-    T_Lines = Val(T_li);
+    T_Cols = (Char) Val(T_co);
+    T_Lines = (Char) Val(T_li);
     if (T_Tabs)
-	T_Tabs = Val(T_pt);
-    T_HasMeta = Val(T_km);
-    T_Margin = Val(T_am) ? MARGIN_AUTO : 0;
-    T_Margin |= Val(T_xn) ? MARGIN_MAGIC : 0;
+	T_Tabs = (Char) Val(T_pt);
+    T_HasMeta = (Char) Val(T_km);
+    T_Margin = (Char) Val(T_am) ? MARGIN_AUTO : 0;
+    T_Margin |= (Char) Val(T_xn) ? MARGIN_MAGIC : 0;
     T_CanCEOL = GoodStr(T_ce);
     T_CanDel = GoodStr(T_dc) || GoodStr(T_DC);
     T_CanIns = GoodStr(T_im) || GoodStr(T_ic) || GoodStr(T_IC);
@@ -1331,12 +1445,12 @@ ChangeSize(lins, cols)
 
 	if (getenv("COLUMNS")) {
 	    Itoa(Val(T_co), buf);
-	    Setenv(STRCOLUMNS, buf);
+	    tsetenv(STRCOLUMNS, buf);
 	}
 
 	if (getenv("LINES")) {
 	    Itoa(Val(T_li), buf);
-	    Setenv(STRLINES, buf);
+	    tsetenv(STRLINES, buf);
 	}
 
 	if ((tptr = getenv("TERMCAP")) != NULL) {
@@ -1357,7 +1471,7 @@ ChangeSize(lins, cols)
 	    }
 	    else {
 		i = ptr - termcap + Strlen(buf);
-		(void) Strncpy(backup, termcap, i);
+		(void) Strncpy(backup, termcap, (size_t) i);
 		backup[i] = '\0';
 		Itoa(Val(T_co), buf);
 		(void) Strcat(backup + i, buf);
@@ -1375,14 +1489,14 @@ ChangeSize(lins, cols)
 	    }
 	    else {
 		i = ptr - backup + Strlen(buf);
-		(void) Strncpy(termcap, backup, i);
+		(void) Strncpy(termcap, backup, (size_t) i);
 		termcap[i] = '\0';
 		Itoa(Val(T_li), buf);
 		(void) Strcat(termcap, buf);
 		ptr = Strchr(ptr, ':');
 		(void) Strcat(termcap, ptr);
 	    }
-	    Setenv(STRTERMCAP, termcap);
+	    tsetenv(STRTERMCAP, termcap);
 	}
     }
 #endif /* KNOWsize */
