@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.02/RCS/ed.xmap.c,v 3.6 1992/01/27 04:20:47 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.04/RCS/ed.xmap.c,v 3.10 1993/06/25 21:17:12 christos Exp $ */
 /*
  * ed.xmap.c: This module contains the procedures for maintaining
  *	      the extended-key map.
@@ -92,7 +92,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: ed.xmap.c,v 3.6 1992/01/27 04:20:47 christos Exp $")
+RCSID("$Id: ed.xmap.c,v 3.10 1993/06/25 21:17:12 christos Exp $")
 
 #include "ed.h"
 #include "ed.defns.h"
@@ -123,13 +123,12 @@ static Char printbuf[MAXXKEY];	/* buffer for printing */
 /* Some declarations of procedures */
 static	int       TraverseMap	__P((XmapNode *, Char *, XmapVal *));
 static	int       TryNode	__P((XmapNode *, Char *, XmapVal *, int));
-static	XmapNode *GetFreeNode	__P((int));
+static	XmapNode *GetFreeNode	__P((Char *));
 static	void	  PutFreeNode	__P((XmapNode *));
 static	int	  TryDeleteNode	__P((XmapNode **, Char *));
 static	int	  Lookup	__P((Char *, XmapNode *, int));
 static	int	  Enumerate	__P((XmapNode *, int));
-static	int	  printOne	__P((Char *, XmapVal *, int));
-static	int	  unparsech	__P((int, int));
+static	int	  unparsech	__P((int, Char *));
 
 
 XmapVal *
@@ -155,38 +154,12 @@ XmapStr(str)
  *	initializes Xmap with arrow keys
  */
 void
-ResetXmap(vi)
-    int     vi;
+ResetXmap()
 {
-    static Char strA[] = {033, '[', 'A', '\0'};
-    static Char strB[] = {033, '[', 'B', '\0'};
-    static Char strC[] = {033, '[', 'C', '\0'};
-    static Char strD[] = {033, '[', 'D', '\0'};
-    static Char stOA[] = {033, 'O', 'A', '\0'};
-    static Char stOB[] = {033, 'O', 'B', '\0'};
-    static Char stOC[] = {033, 'O', 'C', '\0'};
-    static Char stOD[] = {033, 'O', 'D', '\0'};
-
     PutFreeNode(Xmap);
     Xmap = NULL;
-    AddXkey(strA, XmapCmd(F_UP_HIST),   XK_CMD);
-    AddXkey(strB, XmapCmd(F_DOWN_HIST), XK_CMD);
-    AddXkey(strC, XmapCmd(F_CHARFWD),   XK_CMD);
-    AddXkey(strD, XmapCmd(F_CHARBACK),  XK_CMD);
-    AddXkey(stOA, XmapCmd(F_UP_HIST),   XK_CMD);
-    AddXkey(stOB, XmapCmd(F_DOWN_HIST), XK_CMD);
-    AddXkey(stOC, XmapCmd(F_CHARFWD),   XK_CMD);
-    AddXkey(stOD, XmapCmd(F_CHARBACK),  XK_CMD);
-    if (vi) {
-	AddXkey(&strA[1], XmapCmd(F_UP_HIST),   XK_CMD);
-	AddXkey(&strB[1], XmapCmd(F_DOWN_HIST), XK_CMD);
-	AddXkey(&strC[1], XmapCmd(F_CHARFWD),   XK_CMD);
-	AddXkey(&strD[1], XmapCmd(F_CHARBACK),  XK_CMD);
-	AddXkey(&stOA[1], XmapCmd(F_UP_HIST),   XK_CMD);
-	AddXkey(&stOB[1], XmapCmd(F_DOWN_HIST), XK_CMD);
-	AddXkey(&stOC[1], XmapCmd(F_CHARFWD),   XK_CMD);
-	AddXkey(&stOD[1], XmapCmd(F_CHARBACK),  XK_CMD);
-    }
+
+    DefaultArrowKeys();
     return;
 }
 
@@ -264,7 +237,7 @@ AddXkey(Xkey, val, ntype)
 
     if (Xmap == NULL)
 	/* tree is initially empty.  Set up new node to match Xkey[0] */
-	Xmap = GetFreeNode(Xkey[0]);	/* it is properly initialized */
+	Xmap = GetFreeNode(Xkey);	/* it is properly initialized */
 
     /* Now recurse through Xmap */
     (void) TryNode(Xmap, Xkey, val, ntype);	
@@ -288,7 +261,7 @@ TryNode(ptr, string, val, ntype)
 	    if (xm->sibling->ch == *string)
 		break;
 	if (xm->sibling == NULL)
-	    xm->sibling = GetFreeNode(*string);	/* setup new node */
+	    xm->sibling = GetFreeNode(string);	/* setup new node */
 	ptr = xm->sibling;
     }
 
@@ -298,14 +271,27 @@ TryNode(ptr, string, val, ntype)
 	    PutFreeNode(ptr->next);	/* lose longer Xkeys with this prefix */
 	    ptr->next = NULL;
 	}
+
+	switch (ptr->type) {
+	case XK_STR:
+	case XK_EXE:
+	    if (ptr->val.str != NULL)
+		xfree((ptr_t) ptr->val.str);
+	    break;
+	case XK_NOD:
+	case XK_CMD:
+	    break;
+	default:
+	    abort();
+	    break;
+	}
+
 	switch (ptr->type = ntype) {
 	case XK_CMD:
 	    ptr->val = *val;
 	    break;
 	case XK_STR:
 	case XK_EXE:
-	    if (ptr->val.str)
-		xfree((ptr_t) ptr->val.str);
 	    ptr->val.str = Strsave(val->str);
 	    break;
 	default:
@@ -316,7 +302,7 @@ TryNode(ptr, string, val, ntype)
     else {
 	/* still more chars to go */
 	if (ptr->next == NULL)
-	    ptr->next = GetFreeNode(*string);	/* setup new node */
+	    ptr->next = GetFreeNode(string);	/* setup new node */
 	(void) TryNode(ptr->next, string, val, ntype);
     }
     return (0);
@@ -438,19 +424,19 @@ PutFreeNode(ptr)
  */
 static XmapNode *
 GetFreeNode(ch)
-    int    ch;
+    Char *ch;
 {
     XmapNode *ptr;
 
     ptr = (XmapNode *) xmalloc((size_t) sizeof(XmapNode));
-    ptr->ch = ch;
+    ptr->ch = *ch;
     ptr->type = XK_NOD;
     ptr->val.str = NULL;
     ptr->next = NULL;
     ptr->sibling = NULL;
     return (ptr);
 }
-
+ 
 
 /* PrintXKey():
  *	Print the binding associated with Xkey key.
@@ -467,7 +453,7 @@ PrintXkey(key)
     printbuf[0] =  '"';
     if (Lookup(key, Xmap, 1) <= -1)
 	/* key is not bound */
-	xprintf("Unbound extended key \"%s\"\n", short2str(key));
+	xprintf("Unbound extended key \"%S\"\n", key);
     return;
 }
 
@@ -495,7 +481,7 @@ Lookup(string, ptr, cnt)
 	/* If match put this char into printbuf.  Recurse */
 	if (ptr->ch == *string) {
 	    /* match found */
-	    ncnt = unparsech(cnt, ptr->ch);
+	    ncnt = unparsech(cnt, &ptr->ch);
 	    if (ptr->next != NULL)
 		/* not yet at leaf */
 		return (Lookup(string + 1, ptr->next, ncnt + 1));
@@ -532,7 +518,7 @@ Enumerate(ptr, cnt)
 	printbuf[++cnt] = '"';
 	printbuf[++cnt] = '\0';
 	xprintf("Some extended keys too long for internal print buffer");
-	xprintf(" \"%s...\"\n", short2str(printbuf));
+	xprintf(" \"%S...\"\n", printbuf);
 	return (0);
     }
 
@@ -543,7 +529,7 @@ Enumerate(ptr, cnt)
 	return (-1);
     }
 
-    ncnt = unparsech(cnt, ptr->ch);	/* put this char at end of string */
+    ncnt = unparsech(cnt, &ptr->ch); /* put this char at end of string */
     if (ptr->next == NULL) {
 	/* print this Xkey and function */
 	printbuf[ncnt + 1] = '"';
@@ -564,7 +550,7 @@ Enumerate(ptr, cnt)
  *	Print the specified key and its associated
  *	function specified by val
  */
-static int
+int
 printOne(key, val, ntype)
     Char    *key;
     XmapVal *val;
@@ -572,63 +558,64 @@ printOne(key, val, ntype)
 {
     struct KeyFuncs *fp;
     unsigned char unparsbuf[200];
-    static char *fmt = "%-15s->  %s\n";
+    static char *fmt = "%-15S->  %s\n";
 
     if (val != NULL)
 	switch (ntype) {
 	case XK_STR:
 	case XK_EXE:
-	    xprintf(fmt, short2str(key), 
+	    xprintf(fmt, key, 
 		    unparsestring(val->str, unparsbuf, 
 				  ntype == XK_STR ? STRQQ : STRBB));
 	    break;
 	case XK_CMD:
 	    for (fp = FuncNames; fp->name; fp++)
 		if (val->cmd == fp->func)
-		    xprintf(fmt, short2str(key), fp->name);
+		    xprintf(fmt, key, fp->name);
 		break;
 	default:
 	    abort();
 	    break;
 	}
     else
-	xprintf(fmt, short2str(key), "no input");
+	xprintf(fmt, key, "no input");
     return (0);
 }
 
 static int
 unparsech(cnt, ch)
-    int     cnt, ch;
+    int     cnt;
+    Char   *ch;
 {
-    if (ch == 0) {
+    if (*ch == 0) {
 	printbuf[cnt++] = '^';
 	printbuf[cnt] = '@';
 	return cnt;
     }
 
-    if (Iscntrl(ch)) {
+    if (Iscntrl(*ch)) {
 	printbuf[cnt++] = '^';
-	if (ch == '\177')
+	if (*ch == '\177')
 	    printbuf[cnt] = '?';
 	else
-	    printbuf[cnt] = ch | 0100;
+	    printbuf[cnt] = *ch | 0100;
     }
-    else if (ch == '^') {
+    else if (*ch == '^') {
 	printbuf[cnt++] = '\\';
 	printbuf[cnt] = '^';
     }
-    else if (ch == '\\') {
+    else if (*ch == '\\') {
 	printbuf[cnt++] = '\\';
 	printbuf[cnt] = '\\';
     }
-    else if (ch == ' ' || (Isprint(ch) && !Isspace(ch))) {
-	printbuf[cnt] = ch;
+    else if (*ch == ' ' || (Isprint(*ch) && !Isspace(*ch))) {
+	printbuf[cnt] = *ch;
     }
     else {
 	printbuf[cnt++] = '\\';
-	printbuf[cnt++] = ((ch >> 6) & 7) + '0';
-	printbuf[cnt++] = ((ch >> 3) & 7) + '0';
-	printbuf[cnt] = (ch & 7) + '0';
+	printbuf[cnt++] = (*ch >> 6) & (7 + '0');
+	printbuf[cnt++] = (*ch >> 3) & (7 + '0');
+	printbuf[cnt] = *ch & (7 + '0');
     }
     return cnt;
 }
