@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.02/RCS/sh.c,v 3.29 1992/05/09 04:03:53 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.04/RCS/sh.c,v 3.53 1993/07/03 23:47:53 christos Exp $ */
 /*
  * sh.c: Main shell routines
  */
@@ -41,9 +41,9 @@
 char    copyright[] =
 "@(#) Copyright (c) 1991 The Regents of the University of California.\n\
  All rights reserved.\n";
-#endif				/* not lint */
+#endif /* not lint */
 
-RCSID("$Id: sh.c,v 3.29 1992/05/09 04:03:53 christos Exp $")
+RCSID("$Id: sh.c,v 3.53 1993/07/03 23:47:53 christos Exp $")
 
 #include "tc.h"
 #include "ed.h"
@@ -82,45 +82,49 @@ extern bool NoNLSRebind;
  * ported to Apple Unix (TM) (OREO)  26 -- 29 Jun 1987
  */
 
-jmp_buf reslab;
+jmp_buf_t reslab;
+
+sigret_t (*parintr) () = 0;	/* Parents interrupt catch */
+sigret_t (*parterm) () = 0;	/* Parents terminate catch */
 
 #ifdef TESLA
 int do_logout;
-#endif				/* TESLA */
+#endif /* TESLA */
 
-static Char   *dumphist[] = {STRhistory, STRmh, 0, 0};
-static Char   *loadhist[] = {STRsource, STRmh, STRtildothist, 0};
 
-#ifdef CSHDIRS
-static Char   *loaddirs[] = {STRsource, STRdirfile, 0};
-static bool    dflag = 0;
-#endif
-
-#if defined(convex) || defined(__convex__)
 bool    use_fork = 0;		/* use fork() instead of vfork()? */
-#endif
+
+/*
+ * Magic pointer values. Used to specify other invalid conditions aside
+ * from null.
+ */
+static Char	INVCHAR;
+Char    *INVPTR = &INVCHAR;
+Char    **INVPPTR = &INVPTR;
 
 static int     nofile = 0;
 static bool    reenter = 0;
 static bool    nverbose = 0;
 static bool    nexececho = 0;
 static bool    quitit = 0;
+static bool    rdirs = 0;
 bool    fast = 0;
 static bool    batch = 0;
 static bool    mflag = 0;
 static bool    prompt = 1;
-static bool    enterhist = 0;
+static int     enterhist = 0;
 bool    tellwhat = 0;
 time_t  t_period;
+Char  *ffile = NULL;
+bool	dolzero = 0;
 static time_t  chktim;		/* Time mail last checked */
-static int     gid;		/* Invokers gid */
 
 extern char **environ;
 
 static	int		  srccat	__P((Char *, Char *));
-static	int		  srcfile	__P((char *, bool, bool, Char **));
+static	int		  srcfile	__P((char *, bool, int, Char **));
 static	sigret_t	  phup		__P((int));
-static	void		  srcunit	__P((int, bool, bool, Char **));
+static	void		  srcunit	__P((int, bool, int, Char **));
 static	void		  mailchk	__P((void));
 static	Char	 	**defaultpath	__P((void));
 
@@ -141,6 +145,12 @@ main(argc, argv)
     sigvec_t osv;
 #endif /* BSDSIGS */
 
+#ifdef MALLOC_TRACE
+     mal_setstatsfile(fdopen(dup2(open("/tmp/tcsh.trace", 
+				       O_WRONLY|O_CREAT, 0666), 25), "w"));
+     mal_trace(1);
+#endif /* MALLOC_TRACE */
+
 #if !(defined(BSDTIMES) || defined(_SEQUENT_)) && defined(POSIX)
 # ifdef _SC_CLK_TCK
     clk_tck = (clock_t) sysconf(_SC_CLK_TCK);
@@ -179,11 +189,12 @@ main(argc, argv)
 #ifdef _PATH_BSHELL
     STR_BSHELL = SAVE(_PATH_BSHELL);
 #endif
-#ifdef _PATH_CSHELL
-    STR_SHELLPATH = SAVE(_PATH_CSHELL);
-#endif
 #ifdef _PATH_TCSHELL
     STR_SHELLPATH = SAVE(_PATH_TCSHELL);
+#else
+# ifdef _PATH_CSHELL
+    STR_SHELLPATH = SAVE(_PATH_CSHELL);
+# endif
 #endif
     STR_environ = blk2short(environ);
     environ = short2blk(STR_environ);	/* So that we can free it */
@@ -194,11 +205,18 @@ main(argc, argv)
     word_chars = STR_WORD_CHARS;
     bslash_quote = 0;		/* PWP: do tcsh-style backslash quoting? */
 
+    /* Default history size to 100 */
+    set(STRhistory, SAVE("100"), VAR_READWRITE);
+
     tempv = argv;
-    if (eq(str2short(tempv[0]), STRaout))	/* A.out's are quittable */
+    ffile = SAVE(tempv[0]);
+    dolzero = 0;
+    if (eq(ffile, STRaout))	/* A.out's are quittable */
 	quitit = 1;
     uid = getuid();
     gid = getgid();
+    euid = geteuid();
+    egid = getegid();
 #ifdef OREO
     /*
      * We are a login shell if: 1. we were invoked as -<something> with
@@ -216,6 +234,15 @@ main(argc, argv)
 				   tempv[1][0] == '-' && tempv[1][1] == 'l' &&
 						tempv[1][2] == '\0');
 #endif
+
+#ifdef _VMS_POSIX
+    /* No better way to find if we are a login shell */
+    if (!loginsh) {
+	loginsh = (argc == 1 && getppid() == 1);
+	**tempv = '-';	/* Avoid giving VMS an acidic stomach */
+    }
+#endif /* _VMS_POSIX */
+
     if (loginsh && **tempv != '-') {
 	/*
 	 * Mangle the argv space
@@ -230,16 +257,28 @@ main(argc, argv)
 	*++tcp = '-';
 	argc--;
     }
-    if (loginsh)
+    if (loginsh) {
 	(void) time(&chktim);
+	set(STRloginsh, Strsave(STRNULL), VAR_READWRITE);
+    }
 
     AsciiOnly = 1;
     NoNLSRebind = getenv("NOREBIND") != NULL;
 #ifdef NLS
+# ifdef SETLOCALEBUG
+    dont_free = 1;
+# endif /* SETLOCALEBUG */
     (void) setlocale(LC_ALL, "");
 # ifdef LC_COLLATE
     (void) setlocale(LC_COLLATE, "");
 # endif
+# ifdef SETLOCALEBUG
+    dont_free = 0;
+# endif /* SETLOCALEBUG */
+# ifdef STRCOLLBUG
+    fix_strcoll_bug();
+# endif /* STRCOLLBUG */
+
     {
 	int     k;
 
@@ -252,13 +291,16 @@ main(argc, argv)
 #endif				/* NLS */
     if (MapsAreInited && !NLSMapsAreInited)
 	ed_InitNLSMaps();
+    ResetArrowKeys();
 
     /*
      * Initialize for periodic command intervals. Also, initialize the dummy
      * tty list for login-watch.
      */
     (void) time(&t_period);
+#ifndef HAVENOUTMP
     initwatch();
+#endif /* !HAVENOUTMP */
 
 #if defined(alliant)
     /*
@@ -282,7 +324,7 @@ main(argc, argv)
 	 (void) close(SHIN);
 	 SHIN = open(ttyn, O_RDWR);
 	 shpgrp = getpid();
-	 (void) ioctl (SHIN, TIOCSPGRP, (ptr_t) &shpgrp);
+	 (void) ioctl (SHIN, TIOCSPGRP, (ioctl_t) &shpgrp);
 	 (void) setpgid(0, shpgrp);
      }
 #endif /* alliant */
@@ -304,12 +346,12 @@ main(argc, argv)
 	 * utmp, so we keep it too.
 	 */
 	if (strncmp(ttyn, "/dev/", 5) == 0)
-	    set(STRtty, cp = SAVE(ttyn + 5));
+	    set(STRtty, cp = SAVE(ttyn + 5), VAR_READWRITE);
 	else
-	    set(STRtty, cp = SAVE(ttyn));
+	    set(STRtty, cp = SAVE(ttyn), VAR_READWRITE);
     }
     else
-	set(STRtty, cp = SAVE(""));
+	set(STRtty, cp = SAVE(""), VAR_READWRITE);
     /*
      * Initialize the shell variables. ARGV and PROMPT are initialized later.
      * STATUS is also munged in several places. CHILD is munged when
@@ -352,32 +394,33 @@ main(argc, argv)
 		  (cp[3] >= 'p' && cp[3] <= 'u'))) {
 		if (getenv("DISPLAY") == NULL) {
 		    /* NOT on X window shells */
-		    set(STRautologout, Strsave(STRdefautologout));
+		    set(STRautologout, Strsave(STRdefautologout), 
+			VAR_READWRITE);
 		}
 	    }
 	}
     }
-#endif				/* AUTOLOGOUT */
+#endif /* AUTOLOGOUT */
 
     (void) sigset(SIGALRM, alrmcatch);
 
-    set(STRstatus, Strsave(STR0));
+    set(STRstatus, Strsave(STR0), VAR_READWRITE);
     fix_version();		/* publish the shell version */
 
     /*
      * Publish the selected echo style
      */
 #if ECHO_STYLE == NONE_ECHO
-    set(STRecho_style, Strsave(STRnone));
+    set(STRecho_style, Strsave(STRnone), VAR_READWRITE);
 #endif /* ECHO_STYLE == NONE_ECHO */
 #if ECHO_STYLE == BSD_ECHO
-    set(STRecho_style, Strsave(STRbsd));
+    set(STRecho_style, Strsave(STRbsd), VAR_READWRITE);
 #endif /* ECHO_STYLE == BSD_ECHO */
 #if ECHO_STYLE == SYSV_ECHO
-    set(STRecho_style, Strsave(STRsysv));
+    set(STRecho_style, Strsave(STRsysv), VAR_READWRITE);
 #endif /* ECHO_STYLE == SYSV_ECHO */
 #if ECHO_STYLE == BOTH_ECHO
-    set(STRecho_style, Strsave(STRboth));
+    set(STRecho_style, Strsave(STRboth), VAR_READWRITE);
 #endif /* ECHO_STYLE == BOTH_ECHO */
 
     /*
@@ -386,13 +429,13 @@ main(argc, argv)
     shlvl(1);
 
     if ((tcp = getenv("HOME")) != NULL)
-	cp = SAVE(tcp);
+	cp = quote(SAVE(tcp));
     else
 	cp = NULL;
     if (cp == NULL)
 	fast = 1;		/* No home -> can't read scripts */
     else
-	set(STRhome, cp);
+	set(STRhome, cp, VAR_READWRITE);
     dinit(cp);			/* dinit thinks that HOME == cwd in a login
 				 * shell */
     /*
@@ -413,26 +456,25 @@ main(argc, argv)
 #endif /* apollo */
 
 	Itoa(uid, buff);
-	set(STRuid, Strsave(buff));
+	set(STRuid, Strsave(buff), VAR_READWRITE);
 
 	Itoa(gid, buff);
-	set(STRgid, Strsave(buff));
+	set(STRgid, Strsave(buff), VAR_READWRITE);
 
 	cln = getenv("LOGNAME");
 	cus = getenv("USER");
 	if (cus != NULL)
-	    set(STRuser, SAVE(cus));
+	    set(STRuser, quote(SAVE(cus)), VAR_READWRITE);
 	else if (cln != NULL)
-	    set(STRuser, SAVE(cln));
+	    set(STRuser, quote(SAVE(cln)), VAR_READWRITE);
 	else if ((pw = getpwuid(uid)) == NULL)
-	    set(STRuser, SAVE("unknown"));
+	    set(STRuser, SAVE("unknown"), VAR_READWRITE);
 	else
-	    set(STRuser, SAVE(pw->pw_name));
+	    set(STRuser, SAVE(pw->pw_name), VAR_READWRITE);
 	if (cln == NULL)
-	    Setenv(STRLOGNAME, value(STRuser));
+	    tsetenv(STRLOGNAME, value(STRuser));
 	if (cus == NULL)
-	    Setenv(STRUSER, value(STRuser));
-	    
+	    tsetenv(STRKUSER, value(STRuser));
     }
 
     /*
@@ -444,22 +486,22 @@ main(argc, argv)
 
 	if (gethostname(cbuff, sizeof(cbuff)) >= 0) {
 	    cbuff[sizeof(cbuff) - 1] = '\0';	/* just in case */
-	    Setenv(STRHOST, str2short(cbuff));
+	    tsetenv(STRHOST, str2short(cbuff));
 	}
 	else
-	    Setenv(STRHOST, str2short("unknown"));
+	    tsetenv(STRHOST, str2short("unknown"));
     }
 
-
     /*
      * HOSTTYPE, too. Just set it again.
      */
-    Setenv(STRHOSTTYPE, gethosttype());
+    tsetenv(STRHOSTTYPE, str2short(gethosttype()));
+ 
 #ifdef apollo
     if ((tcp = getenv("SYSTYPE")) == NULL)
 	tcp = "bsd4.3";
-    Setenv(STRSYSTYPE, str2short(tcp));
-#endif				/* apollo */
+    tsetenv(STRSYSTYPE, quote(str2short(tcp)));
+#endif /* apollo */
 
     /*
      * set editing on by default, unless running under Emacs as an inferior
@@ -472,10 +514,11 @@ main(argc, argv)
      * xterm, so we are going to want to edit. Unfortunately emacs
      * does not restore all the tty modes, so xterm is not very well
      * set up. But this is not the shell's fault.
+     * Also don't edit if $TERM == wm, for when we're running under an ATK app.
      */
     if ((tcp = getenv("TERM")) != NULL) {
-	set(STRterm, SAVE(tcp));
-	editing = (strcmp(tcp, "emacs") != 0);
+	set(STRterm, quote(SAVE(tcp)), VAR_READWRITE);
+	editing = (strcmp(tcp, "emacs") != 0 && strcmp(tcp, "wm") != 0);
     }
     else 
 	editing = ((tcp = getenv("EMACS")) == NULL || strcmp(tcp, "t") != 0);
@@ -485,24 +528,48 @@ main(argc, argv)
      * need a value.  Making it 'emacs' might be confusing. 
      */
     if (editing)
-	set(STRedit, Strsave(STRNULL));
+	set(STRedit, Strsave(STRNULL), VAR_READWRITE);
 
 
     /*
      * still more mutability: make the complete routine automatically add the
      * suffix of file names...
      */
-    set(STRaddsuffix, Strsave(STRNULL));
+    set(STRaddsuffix, Strsave(STRNULL), VAR_READWRITE);
 
     /*
      * Re-initialize path if set in environment
      */
     if ((tcp = getenv("PATH")) == NULL)
-	set1(STRpath, defaultpath(), &shvhed);
+	set1(STRpath, defaultpath(), &shvhed, VAR_READWRITE);
     else
-	importpath(SAVE(tcp));
+	/* Importpath() allocates memory for the path, and the
+	 * returned pointer from SAVE() was discarded, so
+	 * this was a memory leak.. (sg)
+	 *
+	 * importpath(SAVE(tcp));
+	 */
+	importpath(str2short(tcp));
 
-    set(STRshell, Strsave(STR_SHELLPATH));
+
+    {
+	/* If the SHELL environment variable ends with "tcsh", set
+	 * STRshell to the same path.  This is to facilitate using
+	 * the executable in environments where the compiled-in
+	 * default isn't appropriate (sg).
+	 */
+
+	int sh_len;
+
+	if ((tcp = getenv("SHELL")) != (char *)0 &&
+	    (sh_len = strlen(tcp)) >= 5 &&
+	    strcmp(tcp + (sh_len - 5), "/tcsh") == 0)
+	{
+	    set(STRshell, quote(SAVE(tcp)), VAR_READWRITE);
+	}
+	else
+	    set(STRshell, Strsave(STR_SHELLPATH), VAR_READWRITE);
+    }
 
     doldol = putn((int) getpid());	/* For $$ */
     shtemp = Strspl(STRtmpsh, doldol);	/* For << */
@@ -514,6 +581,11 @@ main(argc, argv)
      * only if we are the login shell.
      */
 #ifdef BSDSIGS
+    /* 
+     * PURIFY-2 claims that osv does not get 
+     * initialized after the sigvec call
+     */
+    setzero((char*) &osv, sizeof(osv));
     /* parents interruptibility */
     (void) mysigvec(SIGINT, NULL, &osv);
     parintr = (sigret_t(*) ()) osv.sv_handler;
@@ -522,24 +594,25 @@ main(argc, argv)
 #else				/* BSDSIGS */
     parintr = signal(SIGINT, SIG_IGN);	/* parents interruptibility */
     (void) sigset(SIGINT, parintr);	/* ... restore */
-    parterm = signal(SIGTERM, SIG_IGN);	/* parents terminability */
-    (void) sigset(SIGTERM, parterm);	/* ... restore */
-#endif				/* BSDSIGS */
 
-    if (loginsh) {
-	(void) signal(SIGHUP, phup);	/* exit processing on HUP */
-#ifdef SIGXCPU
-	(void) signal(SIGXCPU, phup);	/* ...and on XCPU */
-#endif				/* SIGXCPU */
-#ifdef SIGXFSZ
-	(void) signal(SIGXFSZ, phup);	/* ...and on XFSZ */
-#endif				/* SIGXFSZ */
-    }
+# ifdef COHERENT
+    if (loginsh) /* it seems that SIGTERM is always set to SIG_IGN by */
+                 /* init/getty so it should be set to SIG_DFL - there may be */
+                 /* a better fix for this. */
+         parterm = SIG_DFL;
+    else
+# else /* !COHERENT */
+    parterm = signal(SIGTERM, SIG_IGN);	/* parents terminability */
+# endif /* COHERENT */
+    (void) sigset(SIGTERM, parterm);	/* ... restore */
+
+#endif /* BSDSIGS */
+
 
 #ifdef TCF
     /* Enable process migration on ourselves and our progeny */
     (void) signal(SIGMIGRATE, SIG_DFL);
-#endif				/* TCF */
+#endif /* TCF */
 
     /*
      * Process the arguments.
@@ -547,9 +620,9 @@ main(argc, argv)
      * Note that processing of -v/-x is actually delayed till after script
      * processing.
      * 
-     * We set the first character of our name to be '-' if we are a shell running
-     * interruptible commands.  Many programs which examine ps'es use this to
-     * filter such shells out.
+     * We set the first character of our name to be '-' if we are a shell 
+     * running interruptible commands.  Many programs which examine ps'es 
+     * use this to filter such shells out.
      */
     argc--, tempv++;
     while (argc > 0 && (tcp = tempv[0])[0] == '-' &&
@@ -589,7 +662,6 @@ main(argc, argv)
 		 * ensure that you don't make * nonportable csh scripts.
 		 */
 		{
-		    register Char *cp;
 		    register int count;
 
 		    cp = arginp + Strlen(arginp);
@@ -604,31 +676,27 @@ main(argc, argv)
 		prompt = 0;
 		nofile = 1;
 		break;
+	    case 'd':		/* -d	Load directory stack from file */
+		rdirs = 1;
+		break;
 
 #ifdef apollo
 	    case 'D':		/* -D	Define environment variable */
 		{
-		    register Char *cp, *dp;
+		    register Char *dp;
 
 		    cp = str2short(tcp);
 		    if (dp = Strchr(cp, '=')) {
 			*dp++ = '\0';
-			Setenv(cp, dp);
+			tsetenv(cp, dp);
 		    }
 		    else
-			Setenv(cp, STRNULL);
+			tsetenv(cp, STRNULL);
 		}
 		*tcp = '\0'; 	/* done with this argument */
 		break;
 #endif /* apollo */
 
-#ifdef CSHDIRS
-	    case 'd':		/* -d   Force load of ~/.cshdirs */
-		dflag++;
-		break;
-#endif
-
-
 	    case 'e':		/* -e	Exit on any error */
 		exiterr = 1;
 		break;
@@ -680,7 +748,6 @@ main(argc, argv)
 		setNS(STRecho);	/* NOW! */
 		break;
 
-#if defined(__convex__) || defined(convex)
 	    case 'F':		/* Undocumented flag */
 		/*
 		 * This will cause children to be created using fork instead of
@@ -688,7 +755,7 @@ main(argc, argv)
 		 */
 		use_fork = 1;
 		break;
-#endif
+
 	    default:		/* Unknown command option */
 		exiterr = 1;
 		stderror(ERR_TCSHUSAGE, tcp-1);
@@ -713,6 +780,9 @@ main(argc, argv)
 	    /* ... doesn't return */
 	    stderror(ERR_SYSTEM, tempv[0], strerror(errno));
 	}
+	if (ffile != NULL)
+	    xfree((ptr_t) ffile);
+	dolzero = 1;
 	ffile = SAVE(tempv[0]);
 	/* 
 	 * Replace FSHIN. Handle /dev/std{in,out,err} specially
@@ -734,12 +804,12 @@ main(argc, argv)
 		stderror(ERR_SYSTEM, tempv[0], strerror(errno));
 		break;
 	    }
-#ifdef FIOCLEX
-	(void) ioctl(SHIN, FIOCLEX, NULL);
-#endif
+	(void) close_on_exec(SHIN, 1);
 	prompt = 0;
 	 /* argc not used any more */ tempv++;
     }
+
+
     /*
      * Consider input a tty if it really is or we are interactive. but not for
      * editing (christos)
@@ -751,7 +821,7 @@ main(argc, argv)
     }
     intty |= intact;
     if (intty || (intact && isatty(SHOUT))) {
-	if (!batch && (uid != geteuid() || gid != getegid())) {
+	if (!batch && (uid != euid || gid != egid)) {
 	    errno = EACCES;
 	    child = 1;		/* So this ... */
 	    /* ... doesn't return */
@@ -773,16 +843,17 @@ main(argc, argv)
     /*
      * Save the remaining arguments in argv.
      */
-    setq(STRargv, blk2short(tempv), &shvhed);
+    setq(STRargv, blk2short(tempv), &shvhed, VAR_READWRITE);
 
     /*
      * Set up the prompt.
      */
     if (prompt) {
-	set(STRprompt, Strsave(uid == 0 ? STRsymhash : STRsymarrow));
+	set(STRprompt, Strsave(uid == 0 ? STRsymhash : STRsymarrow),
+	    VAR_READWRITE);
 	/* that's a meta-questionmark */
-	set(STRprompt2, Strsave(STRmquestion));
-	set(STRprompt3, Strsave(STRCORRECT));
+	set(STRprompt2, Strsave(STRmquestion), VAR_READWRITE);
+	set(STRprompt3, Strsave(STRKCORRECT), VAR_READWRITE);
     }
 
     /*
@@ -792,12 +863,39 @@ main(argc, argv)
     shpgrp = mygetpgrp();
     opgrp = tpgrp = -1;
     if (setintr) {
+	sigret_t (*osig)();
 	**argv = '-';
 	if (!quitit)		/* Wary! */
 	    (void) signal(SIGQUIT, SIG_IGN);
 	(void) sigset(SIGINT, pintr);
 	(void) sighold(SIGINT);
 	(void) signal(SIGTERM, SIG_IGN);
+
+	/* 
+	 * No reason I can see not to save history on all these events..
+	 * Most usual occurrence is in a window system, where we're not a login
+	 * shell, but might as well be... (sg)
+	 * But there might be races when lots of shells exit together...
+	 * [this is also incompatible].
+	 * We have to be mre careful here. If the parent wants to 
+	 * ignore the signals then we leave them untouched...
+	 * We also only setup the handlers for shells that are trully
+	 * interactive.
+	 */
+	osig = signal(SIGHUP, phup);	/* exit processing on HUP */
+	if (osig == SIG_IGN)
+	    (void) signal(SIGHUP, osig);
+#ifdef SIGXCPU
+	osig = signal(SIGXCPU, phup);	/* exit processing on XCPU */
+	if (osig == SIG_IGN)
+	    (void) signal(SIGXCPU, osig);
+#endif
+#ifdef SIGXFSZ
+	osig = signal(SIGXFSZ, phup);	/* exit processing on XFSZ */
+	if (osig == SIG_IGN)
+	    (void) signal(SIGXFSZ, osig);
+#endif
+
 	if (quitit == 0 && arginp == 0) {
 #ifdef SIGTSTP
 	    (void) signal(SIGTSTP, SIG_IGN);
@@ -887,11 +985,7 @@ main(argc, argv)
 		    else
 			(void) setpgid(0, shpgrp);
 #endif
-#ifdef FIOCLEX
-		    (void) ioctl(dcopy(f, FSHTTY), FIOCLEX, NULL);
-#else				/* FIOCLEX */
-		    (void) dcopy(f, FSHTTY);
-#endif				/* FIOCLEX */
+		    (void) close_on_exec(dcopy(f, FSHTTY), 1);
 		}
 		else
 		    tpgrp = -1;
@@ -928,13 +1022,13 @@ main(argc, argv)
     {
 	struct sigaction act;
         act.sa_handler=pchild;
-	sigemptyset(&(act.sa_mask)); /* Don't block any extra sigs when the
-				      * handler is called
-				      */
+	(void) sigemptyset(&(act.sa_mask));	/* Don't block any extra sigs
+						 * when the handler is called
+						 */
         act.sa_flags=0;	           /* want behaviour of sigset() without
                                     * SA_NOCLDSTOP
 				    */
-        sigaction(SIGCHLD,&act,(struct sigaction *)NULL);
+        (void) sigaction(SIGCHLD,&act,(struct sigaction *)NULL);
     }
 #else /* SYSVREL <= 3 */
     (void) sigset(SIGCHLD, pchild);	/* while signals not ready */
@@ -960,7 +1054,7 @@ main(argc, argv)
 #ifdef BSDSIGS
 	    sigmask_t omask = sigblock(sigmask(SIGINT));
 #else
-	    sighold(SIGINT);
+	    (void) sighold(SIGINT);
 #endif
 	    setintr = 0;
 	    parintr = SIG_IGN;	/* onintr in /etc/ files has no effect */
@@ -991,37 +1085,17 @@ main(argc, argv)
 
 	if (!fast && !arginp && !onelflg && !havhash)
 	    dohash(NULL,NULL);
+
 	/*
 	 * Source history before .login so that it is available in .login
 	 */
-	if ((cp = value(STRhistfile)) != STRNULL)
-	    loadhist[2] = cp;
-	dosource(loadhist, NULL);
+	loadhist(NULL, 0);
 #ifndef LOGINFIRST
 	if (loginsh)
 	    (void) srccat(value(STRhome), STRsldotlogin);
 #endif
-#ifdef CSHDIRS
-	/*
-	 * if dflag then source ~/.cshdirs, but if fast ALWAYS skip the dirs
-	 * restoring. (dflag used to get non-login shells to source the save
-	 * dirs file). Of course, ~/.cshdirs must exist. -strike
-	 */
-	{
-	    extern int bequiet;	/* make dirs shut up */
-	    Char    cshd[BUFSIZE];
-	    struct stat st;
-
-	    (void) Strcpy(cshd, value(STRhome));
-	    (void) Strcat(cshd, STRsldtdirs);
-	    if (!stat(short2str(cshd), &st) &&
-		(dflag || loginsh) && !fast) {
-		bequiet = 1;
-		dosource(loaddirs, NULL);
-		bequiet = 0;
-	    }
-	}
-#endif
+	if (!fast && (loginsh || rdirs))
+	    loaddirs(NULL);
     }
     /* Initing AFTER .cshrc is the Right Way */
     if (intty && !arginp) {	/* PWP setup stuff */
@@ -1064,13 +1138,8 @@ main(argc, argv)
 	    xprintf("exit\n");
 	}
     }
-#ifdef CSHDIRS
-    /*
-     * save the directory stack -strike
-     */
-    recdirs();
-#endif
-    rechist();
+    recdirs(NULL);
+    rechist(NULL);
     exitstat();
     return (0);
 }
@@ -1084,7 +1153,7 @@ untty()
 	(void) tcsetpgrp(FSHTTY, opgrp);
 	(void) resetdisc(FSHTTY);
     }
-#endif				/* BSDJOBS */
+#endif /* BSDJOBS */
 }
 
 void
@@ -1097,7 +1166,7 @@ importpath(cp)
     int     c;
 
     for (dp = cp; *dp; dp++)
-	if (*dp == ':')
+	if (*dp == PATHSEP)
 	    i++;
     /*
      * i+2 where i is the number of colons in the path. There are i+1
@@ -1108,12 +1177,12 @@ importpath(cp)
     i = 0;
     if (*dp)
 	for (;;) {
-	    if ((c = *dp) == ':' || c == 0) {
+	    if ((c = *dp) == PATHSEP || c == 0) {
 		*dp = 0;
 		pv[i++] = Strsave(*cp ? cp : STRdot);
 		if (c) {
 		    cp = dp + 1;
-		    *dp = ':';
+		    *dp = PATHSEP;
 		}
 		else
 		    break;
@@ -1121,7 +1190,7 @@ importpath(cp)
 	    dp++;
 	}
     pv[i] = 0;
-    set1(STRpath, pv, &shvhed);
+    set1(STRpath, pv, &shvhed, VAR_READWRITE);
 }
 
 /*
@@ -1148,7 +1217,8 @@ srccat(cp, dp)
 static int
 srcfile(f, onlyown, flag, av)
     char   *f;
-    bool    onlyown, flag;
+    bool    onlyown;
+    int flag;
     Char **av;
 {
     register int unit;
@@ -1157,9 +1227,7 @@ srcfile(f, onlyown, flag, av)
 	return 0;
     unit = dmove(unit, -1);
 
-#ifdef FIOCLEX
-    (void) ioctl(unit, FIOCLEX, NULL);
-#endif
+    (void) close_on_exec(unit, 1);
     srcunit(unit, onlyown, flag, av);
     return 1;
 }
@@ -1173,7 +1241,8 @@ static  Char **goargv = NULL;
 static void
 srcunit(unit, onlyown, hflg, av)
     register int unit;
-    bool    onlyown, hflg;
+    bool    onlyown;
+    int hflg;
     Char **av;
 {
     /*
@@ -1197,14 +1266,14 @@ srcunit(unit, onlyown, hflg, av)
     Char   *oevalp = evalp, **oevalvec = evalvec;
     int     oonelflg = onelflg;
     bool    oenterhist = enterhist;
-    char    OHIST = HIST;
+    Char    OHIST = HIST;
     bool    otell = cantell;
     Char **oargv;
     struct Bin saveB;
 #ifdef BSDSIGS
     volatile sigmask_t omask = (sigmask_t) 0;
 #endif
-    jmp_buf oldexit;
+    jmp_buf_t oldexit;
 
     /* The (few) real local variables */
     int     my_reenter;
@@ -1217,9 +1286,7 @@ srcunit(unit, onlyown, hflg, av)
     if (onlyown) {
 	struct stat stb;
 
-	if (fstat(unit, &stb) < 0
-	/* || (stb.st_uid != uid && stb.st_gid != gid) */
-	    ) {
+	if (fstat(unit, &stb) < 0) {
 	    (void) close(unit);
 	    return;
 	}
@@ -1252,7 +1319,12 @@ srcunit(unit, onlyown, hflg, av)
      * compiler-dependant code here) PWP: THANKS LOTS !!!
      */
     /* Setup the new values of the state stuff saved above */
-    copy((char *) &(saveB), (char *) &B, sizeof(B));
+
+#ifdef NO_STRUCT_ASSIGNMENT
+    (void) memmove((ptr_t) &(saveB), (ptr_t) &B, sizeof(B));
+#else
+    saveB = B;
+#endif
     fbuf = NULL;
     fseekp = feobp = fblocks = 0;
     oSHIN = SHIN, SHIN = unit, arginp = 0, onelflg = 0;
@@ -1274,7 +1346,7 @@ srcunit(unit, onlyown, hflg, av)
 	struct varent *vp;
 	if ((vp = adrof(STRargv)) != NULL)
 	    goargv = saveblk(vp->vec);
-	setq(STRargv, saveblk(av), &shvhed);
+	setq(STRargv, saveblk(av), &shvhed, VAR_READWRITE);
     }
 
     /*
@@ -1310,15 +1382,20 @@ srcunit(unit, onlyown, hflg, av)
     if (oSHIN >= 0) {
 	register int i;
 
-	/* We made it to the new state... free up its storage */
-	/* This code could get run twice but xfree doesn't care */
-	for (i = 0; i < fblocks; i++)
-	    xfree((ptr_t) fbuf[i]);
-	xfree((ptr_t) fbuf);
+	register Char** nfbuf = fbuf;
+	register int nfblocks = fblocks;
+	fblocks = 0;
+	fbuf = NULL;
+	for (i = 0; i < nfblocks; i++)
+	    xfree((ptr_t) nfbuf[i]);
+	xfree((ptr_t) nfbuf);
 
 	/* Reset input arena */
-	copy((char *) &B, (char *) &(saveB), sizeof(B));
-
+#ifdef NO_STRUCT_ASSIGNMENT
+	(void) memmove((ptr_t) &B, (ptr_t) &(saveB), sizeof(B));
+#else
+	B = saveB;
+#endif
 	(void) close(SHIN), SHIN = oSHIN;
 	arginp = oarginp, onelflg = oonelflg;
 	evalp = oevalp, evalvec = oevalvec;
@@ -1327,7 +1404,7 @@ srcunit(unit, onlyown, hflg, av)
 	    HIST = OHIST;
 	if (av != NULL && *av != NULL && **av != '\0') {
 	    if (goargv)
-		setq(STRargv, goargv, &shvhed);
+		setq(STRargv, goargv, &shvhed, VAR_READWRITE);
 	    else
 		unsetv(STRargv);
 	}
@@ -1345,49 +1422,6 @@ srcunit(unit, onlyown, hflg, av)
     insource = oinsource;
 }
 
-void
-rechist()
-{
-    Char    buf[BUFSIZE], hbuf[BUFSIZE], *hfile;
-    int     fp, ftmp, oldidfds;
-    struct  varent *shist;
-
-    if (!fast) {
-	/*
-	 * If $savehist is just set, we use the value of $history
-	 * else we use the value in $savehist
-	 */
-	if ((shist = adrof(STRsavehist)) != NULL) {
-	    if (shist->vec[0][0] != '\0')
-		(void) Strcpy(hbuf, shist->vec[0]);
-	    else if ((shist = adrof(STRhistory)) != 0 && 
-		     shist->vec[0][0] != '\0')
-		(void) Strcpy(hbuf, shist->vec[0]);
-	    else
-		return;
-	}
-	else
-	    return;
-
-	if ((hfile = value(STRhistfile)) == STRNULL) {
-	    hfile = Strcpy(buf, value(STRhome));
-	    (void) Strcat(buf, STRsldthist);
-	}
-
-	fp = creat(short2str(hfile), 0600);
-	if (fp == -1) 
-	    return;
-	oldidfds = didfds;
-	didfds = 0;
-	ftmp = SHOUT;
-	SHOUT = fp;
-	dumphist[2] = hbuf;
-	dohist(dumphist, NULL);
-	(void) close(fp);
-	SHOUT = ftmp;
-	didfds = oldidfds;
-    }
-}
 
 /*ARGSUSED*/
 void
@@ -1395,11 +1429,9 @@ goodbye(v, c)
     Char **v;
     struct command *c;
 {
-    rechist();
-
-#ifdef CSHDIRS
-    recdirs();
-#endif
+    USE(c);
+    rechist(NULL);
+    recdirs(NULL);
 
     if (loginsh) {
 	(void) signal(SIGQUIT, SIG_IGN);
@@ -1407,7 +1439,7 @@ goodbye(v, c)
 	(void) signal(SIGTERM, SIG_IGN);
 	setintr = 0;		/* No interrupts after "logout" */
 	if (!(adrof(STRlogout)))
-	    set(STRlogout, STRnormal);
+	    set(STRlogout, STRnormal, VAR_READWRITE);
 #ifdef _PATH_DOTLOGOUT
 	(void) srcfile(_PATH_DOTLOGOUT, 0, 0, NULL);
 #endif
@@ -1415,7 +1447,7 @@ goodbye(v, c)
 	    (void) srccat(value(STRhome), STRsldtlogout);
 #ifdef TESLA
 	do_logout = 1;
-#endif				/* TESLA */
+#endif /* TESLA */
     }
     exitstat();
 }
@@ -1423,8 +1455,6 @@ goodbye(v, c)
 void
 exitstat()
 {
-    register Char *cp;
-    register int i;
 #ifdef PROF
     monitor(0);
 #endif
@@ -1435,18 +1465,7 @@ exitstat()
      */
     child = 1;
 
-    /* 
-     * PWP: do this step-by-step because we might get a bus error if
-     * status isn't set, so we call getn(NULL).
-     */
-    cp = value(STRstatus);
-
-    if (!cp)
-	i = 13;
-    else
-	i = getn(cp);
-
-    xexit(i);
+    xexit(getn(value(STRstatus)));
 }
 
 /*
@@ -1460,13 +1479,42 @@ int snum;
     if (snum)
 	(void) sigset(snum, SIG_IGN);
 #endif /* UNRELSIGS */
-    rechist();
-#ifdef CSHDIRS
+    rechist(NULL);
+    recdirs(NULL);
+
+#ifdef POSIXJOBS 
     /*
-     * save the directory stack on HUP - strike
+     * We kill the last foreground process group. It then becomes
+     * responsible to propagate the SIGHUP to its progeny. 
      */
-    recdirs();
-#endif
+    {
+	struct process *pp, *np;
+
+	for (pp = proclist.p_next; pp; pp = pp->p_next) {
+	    np = pp;
+	    /* 
+	     * Find if this job is in the foreground. It could be that
+	     * the process leader has exited and the foreground flag
+	     * is cleared for it.
+	     */
+	    do
+		/*
+		 * If a process is in the foreground; we try to kill
+		 * it's process group. If we succeed, then the 
+		 * whole job is gone. Otherwise we keep going...
+		 * But avoid sending HUP to the shell again.
+		 */
+		if ((np->p_flags & PFOREGND) != 0 && np->p_jobid != shpgrp &&
+		    killpg(np->p_jobid, SIGHUP) != -1) {
+		    /* In case the job was suspended... */
+		    (void) killpg(np->p_jobid, SIGCONT);
+		    break;
+		}
+	    while ((np = np->p_friends) != pp);
+	}
+    }
+#endif /* POSIXJOBS */
+
     xexit(snum);
 #ifndef SIGVOID
     return (snum);
@@ -1549,7 +1597,9 @@ pintr1(wantnl)
     (void) sigrelse(SIGCHLD);
 #endif
     draino();
+#ifndef _VMS_POSIX
     (void) endpwent();
+#endif /*atp vmsposix */
 
     /*
      * If we have an active "onintr" then we search for the label. Note that if
@@ -1566,9 +1616,21 @@ pintr1(wantnl)
 	reset();
     }
     else if (intty && wantnl) {
-	/* xputchar('\n'); *//* Some like this, others don't */
-	(void) putraw('\r');
-	(void) putraw('\n');
+	if (editing) {
+	    /* 
+	     * If we are editing a multi-line input command, and move to
+	     * the beginning of the line, we don't want to trash it when
+	     * we hit ^C
+	     */
+	    PastBottom();
+	    ClearLines();
+	    ClearDisp();
+	}
+	else {
+	    /* xputchar('\n'); *//* Some like this, others don't */
+	    (void) putraw('\r');
+	    (void) putraw('\n');
+	}
     }
     stderror(ERR_SILENT);
 }
@@ -1593,7 +1655,7 @@ process(catch)
     bool    catch;
 {
     extern char Expand;
-    jmp_buf osetexit;
+    jmp_buf_t osetexit;
     /* PWP: This might get nuked my longjmp so don't make it a register var */
     struct command *t = savet;
 
@@ -1602,6 +1664,18 @@ process(catch)
     for (;;) {
 
 	pendjob();
+
+	/* This was leaking memory badly, particularly when sourcing
+	 * files, etc.. For whatever reason we were arriving here with
+	 * allocated pointers still active, and the code was simply
+	 * overwriting them.  I can't say I fully understand the
+	 * control flow here, but according to Purify this particular
+	 * leak has been plugged, and I haven't noticed any ill
+	 * effects.. (sg)
+	 */
+	if (paraml.next && paraml.next != &paraml)
+	    freelex(&paraml);
+
 	paraml.next = paraml.prev = &paraml;
 	paraml.word = STRNULL;
 	(void) setexit();
@@ -1653,13 +1727,16 @@ process(catch)
 	if (neednote)
 	    pnote();
 	if (intty && prompt && evalvec == 0) {
+	    just_signaled = 0;
 	    mailchk();
 	    /*
 	     * Watch for logins/logouts. Next is scheduled commands stored
 	     * previously using "sched." Then execute periodic commands.
 	     * Following that, the prompt precmd is run.
 	     */
+#ifndef HAVENOUTMP
 	    watch_login();
+#endif /* !HAVENOUTMP */
 	    sched_run();
 	    period_cmd();
 	    precmd();
@@ -1682,8 +1759,8 @@ process(catch)
 	 * Echo not only on VERBOSE, but also with history expansion. If there
 	 * is a lexical error then we forego history echo.
 	 */
-	if ((lex(&paraml) && !seterr && intty && !tellwhat && !Expand) ||
-	    adrof(STRverbose)) {
+	if ((lex(&paraml) && !seterr && intty && !tellwhat && !Expand && 
+	     !whyles) || adrof(STRverbose)) {
 	    haderr = 1;
 	    prlex(&paraml);
 	    haderr = 0;
@@ -1707,8 +1784,8 @@ process(catch)
 	 * PWP: entry of items in the history list while in a while loop is done
 	 * elsewhere...
 	 */
-	if (enterhist || (catch && intty && !whyles && !tellwhat))
-	    savehist(&paraml);
+	if (enterhist || (catch && intty && !whyles && !tellwhat && !arun))
+	    savehist(&paraml, enterhist > 1);
 
 	if (Expand && seterr)
 	    Expand = 0;
@@ -1784,18 +1861,26 @@ dosource(t, c)
 {
     register Char *f;
     bool    hflg = 0;
+    extern int bequiet;
     char    buf[BUFSIZE];
 
+    USE(c);
     t++;
     if (*t && eq(*t, STRmh)) {
 	if (*++t == NULL)
 	    stderror(ERR_NAME | ERR_HFLAG);
 	hflg++;
     }
+    else if (*t && eq(*t, STRmm)) {
+    	if (*++t == NULL)
+	    stderror(ERR_NAME | ERR_MFLAG);
+	hflg = 2;
+    }
+
     f = globone(*t++, G_ERROR);
     (void) strcpy(buf, short2str(f));
     xfree((ptr_t) f);
-    if ((!srcfile(buf, 0, hflg, t)) && (!hflg))
+    if ((!srcfile(buf, 0, hflg, t)) && (!hflg) && (!bequiet))
 	stderror(ERR_SYSTEM, buf, strerror(errno));
 }
 
@@ -1808,6 +1893,16 @@ dosource(t, c)
  * knows, since the login program insists on saying
  * "You have mail."
  */
+
+/*
+ * The AMS version.
+ * This version checks if the file is a directory, and if so,
+ * tells you the number of files in it, otherwise do the old thang.
+ * The magic "+1" in the time calculation is to compensate for
+ * an AFS bug where directory mtimes are set to 1 second in
+ * the future.
+ */
+
 static void
 mailchk()
 {
@@ -1830,22 +1925,54 @@ mailchk()
     if (chktim + intvl > t)
 	return;
     for (; *vp; vp++) {
-	if (stat(short2str(*vp), &stb) < 0)
+	char *filename = short2str(*vp);
+
+	if (stat(filename, &stb) < 0)
 	    continue;
 #if defined(BSDTIMES) || defined(_SEQUENT_)
 	new = stb.st_mtime > time0.tv_sec;
 #else
 	new = stb.st_mtime > time0;
 #endif
-	if (stb.st_size == 0 || stb.st_atime > stb.st_mtime ||
-	    (stb.st_atime <= chktim && stb.st_mtime <= chktim) ||
-	    (loginsh && !new))
-	    continue;
-	if (cnt == 1)
-	    xprintf("You have %smail.\n", new ? "new " : "");
-	else
-	    xprintf("%s in %s.\n", new ? "New mail" : "Mail",
-		    short2str(*vp));
+	if ((stb.st_mode & S_IFMT) == S_IFDIR) {
+	    DIR *mailbox;
+	    int mailcount = 0;
+
+	    if (stb.st_mtime <= chktim + 1 || (loginsh && !new))
+		continue;
+
+	    if (!(mailbox = opendir(filename)))
+		continue;
+
+	    if (!readdir(mailbox) || !readdir(mailbox))
+		continue;
+
+	    while (readdir(mailbox))
+		mailcount++;
+
+	    if (mailcount == 0)
+		continue;
+
+	    if (cnt == 1) {
+		xprintf("You have %d piece%s of mail.\n", mailcount,
+			  mailcount == 1 ? "" : "s");
+	    }
+	    else {
+		xprintf("You have %d piece%s of mail in %s.\n",
+			  mailcount, mailcount == 1 ? "" : "s", filename);
+	    }
+	}
+	else {
+	    if (stb.st_size == 0 || stb.st_atime > stb.st_mtime ||
+		(stb.st_atime <= chktim && stb.st_mtime <= chktim) ||
+		(loginsh && !new))
+		continue;
+	    if (cnt == 1)
+		xprintf("You have %smail.\n", new ? "new " : "");
+	    else
+		xprintf("You have %smail in %s.\n", new ? "new " : "",
+			filename);
+	}
     }
     chktim = t;
 }
@@ -1866,7 +1993,7 @@ gethdir(home)
      * Is it us?
      */
     if (*home == '\0') {
-	if ((h = value(STRhome)) != NULL) {
+	if ((h = value(STRhome)) != STRNULL) {
 	    (void) Strcpy(home, h);
 	    return 0;
 	}
@@ -1894,20 +2021,15 @@ initdesc()
 {
 
     didfds = 0;			/* 0, 1, 2 aren't set up */
-#ifdef FIOCLEX
-    (void) ioctl(SHIN = dcopy(0, FSHIN), FIOCLEX, NULL);
-    (void) ioctl(SHOUT = dcopy(1, FSHOUT), FIOCLEX, NULL);
-    (void) ioctl(SHDIAG = dcopy(2, FSHDIAG), FIOCLEX, NULL);
-    (void) ioctl(OLDSTD = dcopy(SHIN, FOLDSTD), FIOCLEX, NULL);
-#else
+    (void) close_on_exec(SHIN = dcopy(0, FSHIN), 1);
+    (void) close_on_exec(SHOUT = dcopy(1, FSHOUT), 1);
+    (void) close_on_exec(SHDIAG = dcopy(2, FSHDIAG), 1);
+    (void) close_on_exec(OLDSTD = dcopy(SHIN, FOLDSTD), 1);
+#ifndef CLOSE_ON_EXEC
     didcch = 0;			/* Havent closed for child */
-    SHIN = dcopy(0, FSHIN);
-    SHOUT = dcopy(1, FSHOUT);
-    isoutatty = isatty(SHOUT);
-    SHDIAG = dcopy(2, FSHDIAG);
+#endif /* CLOSE_ON_EXEC */
     isdiagatty = isatty(SHDIAG);
-    OLDSTD = dcopy(SHIN, FOLDSTD);
-#endif
+    isoutatty = isatty(SHOUT);
     closem();
 }
 
@@ -1933,9 +2055,9 @@ xexit(i)
 	(void) ioctl(FSHTTY, TIOCCDTR, NULL);
 	(void) sleep(1);
 	(void) ioctl(FSHTTY, TIOCSDTR, NULL);
-#endif				/* TIOCCDTR */
+#endif /* TIOCCDTR */
     }
-#endif				/* TESLA */
+#endif /* TESLA */
 
     untty();
     _exit(i);
