@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.02/RCS/sh.decls.h,v 3.11 1992/05/02 23:39:58 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.04/RCS/sh.decls.h,v 3.19 1993/06/25 21:17:12 christos Exp $ */
 /*
  * sh.decls.h	 External declarations from sh*.c
  */
@@ -49,7 +49,6 @@ extern	void		  initdesc	__P((void));
 extern	sigret_t	  pintr		__P((int));
 extern	void		  pintr1	__P((bool));
 extern	void		  process	__P((bool));
-extern	void		  rechist	__P((void));
 extern	void		  untty		__P((void));
 #ifdef PROF
 extern	void		  done		__P((int));
@@ -63,17 +62,17 @@ extern	void		  xexit		__P((int));
 extern	void		  dinit		__P((Char *));
 extern	void		  dodirs	__P((Char **, struct command *));
 extern	Char		 *dcanon	__P((Char *, Char *));
-extern	void		  dtildepr	__P((Char *, Char *));
+extern	void		  dtildepr	__P((Char *));
 extern	void		  dtilde	__P((void));
 extern	void		  dochngd	__P((Char **, struct command *));
 extern	Char		 *dnormalize	__P((Char *, int));
 extern	void		  dopushd	__P((Char **, struct command *));
 extern	void		  dopopd	__P((Char **, struct command *));
 extern	void		  dfree		__P((struct directory *));
+extern	void		  dsetstack	__P((void));
 extern	int		  getstakd	__P((Char *, int));
-#ifdef	CSHDIRS
-extern	void		  recdirs	__P((void));
-#endif
+extern	void		  recdirs	__P((Char *));
+extern	void		  loaddirs	__P((Char *));
 
 /*
  * sh.dol.c
@@ -120,7 +119,7 @@ extern	int		  tenex		__P((Char *, int));
 /*
  * sh.func.c
  */
-extern	void		  Setenv	__P((Char *, Char *));
+extern	void		  tsetenv	__P((Char *, Char *));
 extern	void		  Unsetenv	__P((Char *));
 extern	void		  doalias	__P((Char **, struct command *));
 extern	void		  dobreak	__P((Char **, struct command *));
@@ -142,6 +141,7 @@ extern	void		  donewgrp	__P((Char **, struct command *));
 #endif
 extern	void		  donohup	__P((Char **, struct command *));
 extern	void		  doonintr	__P((Char **, struct command *));
+extern	void		  doprintenv	__P((Char **, struct command *));
 extern	void		  dorepeat	__P((Char **, struct command *));
 extern	void		  dosetenv	__P((Char **, struct command *));
 extern	void		  dosuspend	__P((Char **, struct command *));
@@ -160,6 +160,7 @@ extern	void		  prvars	__P((void));
 extern	int		  srchx		__P((Char *));
 extern	void		  unalias	__P((Char **, struct command *));
 extern	void		  wfree		__P((void));
+extern	void		  dobuiltins	__P((Char **, struct command *));
 
 /*
  * sh.glob.c
@@ -183,8 +184,11 @@ extern	int		  sortscmp	__P((Char **, Char **));
  * sh.hist.c
  */
 extern	void	 	  dohist	__P((Char **, struct command *));
-extern struct Hist 	 *enthist	__P((int, struct wordent *, bool));
-extern	void	 	  savehist	__P((struct wordent *));
+extern  struct Hist 	 *enthist	__P((int, struct wordent *, bool, bool));
+extern	void	 	  savehist	__P((struct wordent *, bool));
+extern	void		  fmthist	__P((int, ptr_t, char *));
+extern	void		  rechist	__P((Char *));
+extern	void		  loadhist	__P((Char *, bool));
 
 
 /*
@@ -214,13 +218,10 @@ extern	void		  blkfree	__P((Char **));
 extern	int		  blklen	__P((Char **));
 extern	void		  blkpr		__P((Char **));
 extern	Char		**blkspl	__P((Char **, Char **));
-#ifndef copy
-extern  void		  copy		__P((char *, char *, int));
-#endif
 extern	void		  closem	__P((void));
-#ifndef FIOCLEX
+#ifndef CLOSE_ON_EXEC
 extern  void 		  closech	__P((void));
-#endif
+#endif /* !CLOSE_ON_EXEC */
 extern	Char		**copyblk	__P((Char **));
 extern	int		  dcopy		__P((int, int));
 extern	int		  dmove		__P((int, int));
@@ -232,15 +233,16 @@ extern	int		  prefix	__P((Char *, Char *));
 extern	Char		**saveblk	__P((Char **));
 extern	void		  setzero	__P((char *, int));
 extern	Char		 *strip		__P((Char *));
+extern	Char		 *quote		__P((Char *));
 extern	char		 *strsave	__P((const char *));
-extern	char		 *strspl	__P((char *, char *));
+extern	void		  udvar		__P((Char *));
 #ifndef POSIX
 extern  char   	  	 *strstr	__P((const char *, const char *));
-#endif
-extern	void		  udvar		__P((Char *));
+#endif /* !POSIX */
 #ifndef SHORT_STRINGS
+extern	char		 *strspl	__P((char *, char *));
 extern	char		 *strend	__P((char *));
-#endif
+#endif /* SHORT_STRINGS */
 
 /*
  * sh.parse.c
@@ -264,9 +266,9 @@ extern	void		  pcsecs	__P((clock_t));
 extern	void		  pcsecs	__P((time_t));
 # endif /* !POSIX */
 #endif /* BSDTIMES */
-#ifdef RLIMIT_CPU
+#ifdef BSDLIMIT
 extern	void		  psecs		__P((long));
-#endif /* RLIMIT_CPU */
+#endif /* BSDLIMIT */
 extern	int		  putpure	__P((int));
 extern	int		  putraw	__P((int));
 extern	void		  xputchar	__P((int));
@@ -296,6 +298,7 @@ extern	void		  prestjob	__P((void));
 extern	void		  psavejob	__P((void));
 extern	void		  pstart	__P((struct process *, int));
 extern	void		  pwait		__P((void));
+extern  struct process   *pfind		__P((Char *));
 
 /*
  * sh.sem.c
@@ -313,15 +316,17 @@ extern	void		  dolet		__P((Char **, struct command *));
 extern	Char		 *putn		__P((int));
 extern	int		  getn		__P((Char *));
 extern	Char		 *value1	__P((Char *, struct varent *));
-extern	void		  set		__P((Char *, Char *));
-extern	void		  set1		__P((Char *, Char **, struct varent *));
-extern	void		  setq		__P((Char *, Char **, struct varent *));
+extern	void		  set		__P((Char *, Char *, int));
+extern	void		  set1		__P((Char *, Char **, struct varent *,
+					     int));
+extern	void		  setq		__P((Char *, Char **, struct varent *,
+					     int));
 extern	void		  unset		__P((Char **, struct command *));
 extern	void		  unset1	__P((Char *[], struct varent *));
 extern	void		  unsetv	__P((Char *));
 extern	void		  setNS		__P((Char *));
 extern	void		  shift		__P((Char **, struct command *));
-extern	void		  plist		__P((struct varent *));
+extern	void		  plist		__P((struct varent *, int));
 
 /*
  * sh.time.c
@@ -331,11 +336,14 @@ extern	void		  dotime	__P((Char **, struct command *));
 #ifdef BSDTIMES
 extern	void		  prusage	__P((struct rusage *, struct rusage *, 
 					     timeval_t *, timeval_t *));
+extern	void		  ruadd		__P((struct rusage *, struct rusage *));
 #else /* BSDTIMES */
 # ifdef _SEQUENT_
 extern	void		  prusage	__P((struct process_stats *,
 					     struct process_stats *, 
 					     timeval_t *, timeval_t *));
+extern	void		  ruadd		__P((struct process_stats *,
+					     struct process_stats *));
 # else /* !_SEQUENT_ */
 #  ifdef POSIX
 extern	void		  prusage	__P((struct tms *, struct tms *, 
@@ -348,7 +356,6 @@ extern	void		  prusage	__P((struct tms *, struct tms *,
 #endif /* BSDTIMES */
 extern	void		  settimes	__P((void));
 #if defined(BSDTIMES) || defined(_SEQUENT_)
-extern	void		  ruadd		__P((struct rusage *, struct rusage *));
 extern	void		  tvsub		__P((struct timeval *, 
 					     struct timeval *, 
 					     struct timeval *));
