@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.02/RCS/sh.dol.c,v 3.15 1992/05/15 21:54:34 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.04/RCS/sh.dol.c,v 3.24 1993/06/25 21:17:12 christos Exp $ */
 /*
  * sh.dol.c: Variable substitutions
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.dol.c,v 3.15 1992/05/15 21:54:34 christos Exp $")
+RCSID("$Id: sh.dol.c,v 3.24 1993/06/25 21:17:12 christos Exp $")
 
 /*
  * C shell
@@ -58,7 +58,7 @@ static Char *Dcp, **Dvp;	/* Input vector for Dreadc */
 
 #define	unDgetC(c)	Dpeekc = c
 
-#define QUOTES		(_Q|_Q1|_ESC)	/* \ ' " ` */
+#define QUOTES		(_QF|_QB|_ESC)	/* \ ' " ` */
 
 /*
  * The following variables give the information about the current
@@ -193,7 +193,7 @@ Dpack(wbuf, wp)
 	    Gcat(STRNULL, wbuf);
 	    return (NULL);
 	}
-	if (cmap(c, _SP | _NL | _Q | _Q1)) {	/* sp \t\n'"` */
+	if (cmap(c, _SP | _NL | _QF | _QB)) {	/* sp \t\n'"` */
 	    unDgetC(c);
 	    if (cmap(c, QUOTES))
 		return (wp);
@@ -297,7 +297,18 @@ Dword()
 	    if ((wp = Dpack(wbuf, wp)) == NULL)
 		return (1);
 	    else {
+#ifdef masscomp
+    /*
+     * Avoid a nasty message from the RTU 4.1A & RTU 5.0 compiler concerning
+     * the "overuse of registers". According to the compiler release notes,
+     * incorrect code may be produced unless the offending expression is
+     * rewritten. Therefore, we can't just ignore it, DAS DEC-90.
+     */
+		i = MAXWLEN;
+		i -= (wp - wbuf);
+#else /* !masscomp */
 		i = MAXWLEN - (wp - wbuf);
+#endif /* masscomp */
 		done = 0;
 	    }
 	    break;
@@ -320,7 +331,18 @@ Dword()
 	    if ((wp = Dpack(wbuf, wp)) == NULL)
 		return (1);
 	    else {
+#ifdef masscomp
+    /*
+     * Avoid a nasty message from the RTU 4.1A & RTU 5.0 compiler concerning
+     * the "overuse of registers". According to the compiler release notes,
+     * incorrect code may be produced unless the offending expression is
+     * rewritten. Therefore, we can't just ignore it, DAS DEC-90.
+     */
+		i = MAXWLEN;
+		i -= (wp - wbuf);
+#else /* !masscomp */
 		i = MAXWLEN - (wp - wbuf);
+#endif /* masscomp */
 		done = 0;
 	    }
 	}
@@ -380,8 +402,9 @@ quotspec:
     return (c);
 }
 
-static Char *nulvec[] = {0};
-static struct varent nulargv = {nulvec, STRargv, 0};
+static Char *nulvec[] = { NULL };
+static struct varent nulargv = {nulvec, STRargv, VAR_READWRITE, 
+				{ NULL, NULL, NULL }, 0 };
 
 static void
 dolerror(s)
@@ -403,7 +426,7 @@ Dgetdol()
     Char    name[4 * MAXVARLEN + 1];
     int     c, sc;
     int     subscr = 0, lwb = 1, upb = 0;
-    bool    dimen = 0, bitset = 0;
+    bool    dimen = 0, bitset = 0, length = 0;
     char    tnp;
     Char    wbuf[BUFSIZE];
     static Char *dolbang = NULL;
@@ -420,10 +443,12 @@ Dgetdol()
 	dimen++, c = DgetC(0);	/* $# takes dimension */
     else if (c == '?')
 	bitset++, c = DgetC(0);	/* $? tests existence */
+    else if (c == '%')
+	length++, c = DgetC(0); /* $% returns length in chars */
     switch (c) {
 
     case '!':
-	if (dimen || bitset)
+	if (dimen || bitset || length)
 	    stderror(ERR_SYNTAX);
 	if (backpid != 0) {
 	    if (dolbang) 
@@ -433,16 +458,28 @@ Dgetdol()
 	goto eatbrac;
 
     case '$':
-	if (dimen || bitset)
+	if (dimen || bitset || length)
 	    stderror(ERR_SYNTAX);
 	setDolp(doldol);
 	goto eatbrac;
 
-    case '<' | QUOTE:
+#ifdef COHERENT
+    /* Coherent compiler doesn't allow case-labels that are not 
+       constant-expressions */
+#ifdef SHORT_STRINGS
+    case 0100074:
+#else /* !SHORT_STRINGS */
+    case 0274:
+#endif
+#else /* !COHERENT */
+    case '<'|QUOTE:
+#endif
 	if (bitset)
 	    stderror(ERR_NOTALLOWED, "$?<");
 	if (dimen)
-	    stderror(ERR_NOTALLOWED, "$?#");
+	    stderror(ERR_NOTALLOWED, "$#<");
+	if (length)
+	    stderror(ERR_NOTALLOWED, "$%<");
 	for (np = wbuf; read(OLDSTD, &tnp, 1) == 1; np++) {
 	    *np = (unsigned char) tnp;
 	    if (np >= &wbuf[BUFSIZE - 1])
@@ -467,18 +504,27 @@ Dgetdol()
 	setDolp(wbuf);
 	goto eatbrac;
 
-    case DEOF:
-    case '\n':
-	stderror(ERR_SYNTAX);
-	/* NOTREACHED */
-	break;
-
     case '*':
 	(void) Strcpy(name, STRargv);
 	vp = adrof(STRargv);
 	subscr = -1;		/* Prevent eating [...] */
 	break;
 
+    case DEOF:
+    case '\n':
+	np = dimen ? STRargv : (bitset ? STRstatus : NULL);
+	if (np) {
+	    bitset = 0;
+	    (void) Strcpy(name, np);
+	    vp = adrof(np);
+	    subscr = -1;		/* Prevent eating [...] */
+	    unDredc(c);
+	    break;
+	}
+	else
+	    stderror(ERR_SYNTAX);
+	/*NOTREACHED*/
+
     default:
 	np = name;
 	if (Isdigit(c)) {
@@ -496,7 +542,7 @@ Dgetdol()
 	    }
 	    if (subscr == 0) {
 		if (bitset) {
-		    dolp = ffile ? STR1 : STR0;
+		    dolp = dolzero ? STR1 : STR0;
 		    goto eatbrac;
 		}
 		if (ffile == 0)
@@ -506,7 +552,9 @@ Dgetdol()
 		goto eatbrac;
 	    }
 	    if (bitset)
-		stderror(ERR_DOLQUEST);
+		stderror(ERR_NOTALLOWED, "$?<num>");
+	    if (length)
+		stderror(ERR_NOTALLOWED, "$%<num>");
 	    vp = adrof(STRargv);
 	    if (vp == 0) {
 		vp = &nulargv;
@@ -514,10 +562,21 @@ Dgetdol()
 	    }
 	    break;
 	}
-	if (!alnum(c))
-	    stderror(ERR_VARALNUM);
+	if (!alnum(c)) {
+	    np = dimen ? STRargv : (bitset ? STRstatus : NULL);
+	    if (np) {
+		bitset = 0;
+		(void) Strcpy(name, np);
+		vp = adrof(np);
+		subscr = -1;		/* Prevent eating [...] */
+		unDredc(c);
+		break;
+	    }
+	    else
+		stderror(ERR_VARALNUM);
+	}
 	for (;;) {
-	    *np++ = c;
+	    *np++ = (Char) c;
 	    c = DgetC(0);
 	    if (!alnum(c))
 		break;
@@ -554,7 +613,7 @@ Dgetdol()
 		stderror(ERR_INCBR);
 	    if (np >= &name[sizeof(name) / sizeof(Char) - 2])
 		stderror(ERR_VARTOOLONG);
-	    *np++ = c;
+	    *np++ = (Char) c;
 	}
 	*np = 0, np = name;
 	if (dolp || dolcnt)	/* $ exp must end before ] */
@@ -620,6 +679,15 @@ Dgetdol()
 	addla(cp);
 	xfree((ptr_t) cp);
     }
+    else if (length) {
+	int i;
+	Char   *cp;
+	for (i = lwb - 1, length = 0; i < upb; i++)
+	    length += Strlen(vp->vec[i]);
+	cp = putn(length);
+	addla(cp);
+	xfree((ptr_t) cp);
+    }
     else {
 eatmod:
 	fixDolMod();
@@ -665,7 +733,7 @@ fixDolMod()
 		int delimcnt = 2;
 		int delim = DgetC(0);
 		dolmod[dolnmod++] = (Char) c;
-		dolmod[dolnmod++] = delim;
+		dolmod[dolnmod++] = (Char) delim;
 		
 		if (!delim || letter(delim)
 		    || Isdigit(delim) || any(" \t\n", delim)) {
@@ -754,7 +822,7 @@ setDolp(cp)
 		    np = (Char *) xmalloc((size_t)
 					  ((Strlen(cp) + 1 - lhlen + rhlen) *
 					  sizeof(Char)));
-		    (void) Strncpy(np, cp, dp - cp);
+		    (void) Strncpy(np, cp, (size_t) (dp - cp));
 		    (void) Strcpy(np + (dp - cp), rhsub);
 		    (void) Strcpy(np + (dp - cp) + rhlen, dp + lhlen);
 
@@ -770,11 +838,13 @@ setDolp(cp)
 	    /*
 	     * restore dolmod for additional words
 	     */
-	    dolmod[i] = rhsub[-1] = delim;
+	    dolmod[i] = rhsub[-1] = (Char) delim;
 	    if (didmod)
 		dolmcnt--;
+#ifdef notdef
 	    else
 		break;
+#endif
         } else {
 	    int didmod = 0;
 
@@ -798,8 +868,10 @@ setDolp(cp)
 	    dp = cp;
 	    if (didmod)
 		dolmcnt--;
+#ifdef notdef
 	    else
 		break;
+#endif
 	}
     }
 #endif /* COMPAT */
@@ -904,7 +976,7 @@ heredoc(term)
 	    c = readc(1);	/* 1 -> Want EOF returns */
 	    if (c < 0 || c == '\n')
 		break;
-	    if (c &= TRIM) {
+	    if ((c &= TRIM) != 0) {
 		*lbp++ = (Char) c;
 		if (--lcnt < 0) {
 		    setname("<<");
