@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.02/RCS/sh.file.c,v 3.4 1992/05/09 04:03:53 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.04/RCS/sh.file.c,v 3.8 1993/06/25 21:17:12 christos Exp $ */
 /*
  * sh.file.c: File completion for csh. This file is not used in tcsh.
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.file.c,v 3.4 1992/05/09 04:03:53 christos Exp $")
+RCSID("$Id: sh.file.c,v 3.8 1993/06/25 21:17:12 christos Exp $")
 
 #ifdef FILEC
 
@@ -74,7 +74,7 @@ static	void	 retype			__P((void));
 static	void	 beep			__P((void));
 static	void 	 print_recognized_stuff	__P((Char *));
 static	void	 extract_dir_and_name	__P((Char *, Char *, Char *));
-static	Char	*getentry		__P((DIR *, int));
+static	Char	*getitem		__P((DIR *, int));
 static	void	 free_items		__P((Char **));
 static	int	 tsearch		__P((Char *, COMMAND, int));
 static	int	 recognize		__P((Char *, Char *, int, int));
@@ -181,7 +181,7 @@ back_to_col_1()
 # ifdef BSDSIGS
     sigmask_t omask = sigblock(sigmask(SIGINT));
 # else
-    sighold(SIGINT);
+    (void) sighold(SIGINT);
 # endif /* BSDSIGS */
 
 #ifdef TERMIO
@@ -242,7 +242,7 @@ pushback(string)
 #ifdef BSDSIGS
     sigmask_t omask = sigblock(sigmask(SIGINT));
 #else
-    sighold(SIGINT);
+    (void) sighold(SIGINT);
 #endif /* BSDSIGS */
 
 #ifdef TERMIO
@@ -376,7 +376,7 @@ print_by_column(dir, items, count)
 	    if (i < count) {
 		register int w;
 
-		xprintf("%s", short2str(items[i]));
+		xprintf("%S", items[i]);
 		xputchar(dir ? filetype(dir, items[i]) : ' ');
 		if (c < columns - 1) {	/* last column? */
 		    w = Strlen(items[i]) + 1;
@@ -468,25 +468,25 @@ print_recognized_stuff(recognized_part)
     Char   *recognized_part;
 {
     /* An optimized erasing of that silly ^[ */
-    putraw('\b');
-    putraw('\b');
+    (void) putraw('\b');
+    (void) putraw('\b');
     switch (Strlen(recognized_part)) {
 
     case 0:			/* erase two Characters: ^[ */
-	putraw(' ');
-	putraw(' ');
-	putraw('\b');
-	putraw('\b');
+	(void) putraw(' ');
+	(void) putraw(' ');
+	(void) putraw('\b');
+	(void) putraw('\b');
 	break;
 
     case 1:			/* overstrike the ^, erase the [ */
-	xprintf("%s", short2str(recognized_part));
-	putraw(' ');
-	putraw('\b');
+	xprintf("%S", recognized_part);
+	(void) putraw(' ');
+	(void) putraw('\b');
 	break;
 
     default:			/* overstrike both Characters ^[ */
-	xprintf("%s", short2str(recognized_part));
+	xprintf("%S", recognized_part);
 	break;
     }
     flush();
@@ -512,9 +512,13 @@ extract_dir_and_name(path, dir, name)
 	copyn(dir, path, p - path);
     }
 }
-
+/* atp vmsposix - I need to remove all the setpwent 
+ *		  getpwent endpwent stuff. VMS_POSIX has getpwnam getpwuid
+ *		  and getlogin. This needs fixing. (There is no access to 
+ *		  pw->passwd in VMS - a secure system benefit :-| )
+ */
 static Char *
-getentry(dir_fd, looking_for_lognames)
+getitem(dir_fd, looking_for_lognames)
     DIR    *dir_fd;
     int     looking_for_lognames;
 {
@@ -522,9 +526,13 @@ getentry(dir_fd, looking_for_lognames)
     register struct dirent *dirp;
 
     if (looking_for_lognames) {
+#ifdef _VMS_POSIX
+	    return (NULL);
+#else
 	if ((pw = getpwent()) == NULL)
 	    return (NULL);
 	return (str2short(pw->pw_name));
+#endif /* atp vmsposix */
     }
     if (dirp = readdir(dir_fd))
 	return (str2short(dirp->d_name));
@@ -575,7 +583,7 @@ tsearch(word, command, max_word_length)
     register name_length, looking_for_lognames;
     Char    tilded_dir[MAXPATHLEN + 1], dir[MAXPATHLEN + 1];
     Char    name[MAXNAMLEN + 1], extended_name[MAXNAMLEN + 1];
-    Char   *entry;
+    Char   *item;
 
 #define MAXITEMS 1024
 
@@ -584,7 +592,9 @@ tsearch(word, command, max_word_length)
 
     looking_for_lognames = (*word == '~') && (Strchr(word, '/') == NULL);
     if (looking_for_lognames) {
+#ifndef _VMS_POSIX
 	(void) setpwent();
+#endif /*atp vmsposix */
 	copyn(name, &word[1], MAXNAMLEN);	/* name sans ~ */
 	dir_fd = NULL;
     }
@@ -599,11 +609,11 @@ tsearch(word, command, max_word_length)
 
 again:				/* search for matches */
     name_length = Strlen(name);
-    for (numitems = 0; entry = getentry(dir_fd, looking_for_lognames);) {
-	if (!is_prefix(name, entry))
+    for (numitems = 0; item = getitem(dir_fd, looking_for_lognames);) {
+	if (!is_prefix(name, item))
 	    continue;
 	/* Don't match . files on null prefix match */
-	if (name_length == 0 && entry[0] == '.' &&
+	if (name_length == 0 && item[0] == '.' &&
 	    !looking_for_lognames)
 	    continue;
 	if (command == LIST) {
@@ -613,18 +623,28 @@ again:				/* search for matches */
 			"names in password file" : "files");
 		break;
 	    }
+	    /*
+	     * From Beto Appleton (beto@aixwiz.austin.ibm.com)
+	     *	typing "./control-d" will cause the csh to core-dump.
+	     *	the problem can be reproduce as following:
+	     *	 1. set ignoreeof
+	     *	 2. set filec
+	     *	 3. create a directory with 1050 files
+	     *	 4. typing "./control-d" will cause the csh to core-dump
+	     * Solution: Add + 1 to MAXITEMS
+	     */
 	    if (items == NULL)
-		items = (Char **) xcalloc(sizeof(items[0]), MAXITEMS);
-	    items[numitems] = (Char *) xmalloc((size_t) (Strlen(entry) + 1) *
+		items = (Char **) xcalloc(sizeof(items[0]), MAXITEMS + 1);
+	    items[numitems] = (Char *) xmalloc((size_t) (Strlen(item) + 1) *
 					       sizeof(Char));
-	    copyn(items[numitems], entry, MAXNAMLEN);
+	    copyn(items[numitems], item, MAXNAMLEN);
 	    numitems++;
 	}
 	else {			/* RECOGNIZE command */
-	    if (ignoring && ignored(entry))
+	    if (ignoring && ignored(item))
 		nignored++;
 	    else if (recognize(extended_name,
-			       entry, name_length, ++numitems))
+			       item, name_length, ++numitems))
 		break;
 	}
     }
@@ -632,14 +652,18 @@ again:				/* search for matches */
 	ignoring = FALSE;
 	nignored = 0;
 	if (looking_for_lognames)
+#ifndef _VMS_POSIX
 	    (void) setpwent();
+#endif /* atp vmsposix */
 	else
 	    rewinddir(dir_fd);
 	goto again;
     }
 
     if (looking_for_lognames)
+#ifndef _VMS_POSIX
 	(void) endpwent();
+#endif /*atp vmsposix */
     else
 	(void) closedir(dir_fd);
     if (numitems == 0)
@@ -655,7 +679,7 @@ again:				/* search for matches */
 	return (numitems);
     }
     else {			/* LIST */
-	qsort((ptr_t) items, numitems, sizeof(items[0]), sortscmp);
+	qsort((ptr_t) items, (size_t) numitems, sizeof(items[0]), sortscmp);
 	print_by_column(looking_for_lognames ? NULL : tilded_dir,
 			items, numitems);
 	if (items != NULL)
@@ -667,24 +691,24 @@ again:				/* search for matches */
 /*
  * Object: extend what user typed up to an ambiguity.
  * Algorithm:
- * On first match, copy full entry (assume it'll be the only match)
+ * On first match, copy full item (assume it'll be the only match)
  * On subsequent matches, shorten extended_name to the first
- * Character mismatch between extended_name and entry.
+ * Character mismatch between extended_name and item.
  * If we shorten it back to the prefix length, stop searching.
  */
 static int
-recognize(extended_name, entry, name_length, numitems)
-    Char   *extended_name, *entry;
+recognize(extended_name, item, name_length, numitems)
+    Char   *extended_name, *item;
     int     name_length, numitems;
 {
     if (numitems == 1)		/* 1st match */
-	copyn(extended_name, entry, MAXNAMLEN);
+	copyn(extended_name, item, MAXNAMLEN);
     else {			/* 2nd & subsequent matches */
 	register Char *x, *ent;
 	register int len = 0;
 
 	x = extended_name;
-	for (ent = entry; *x && *x == *ent++; x++, len++);
+	for (ent = item; *x && *x == *ent++; x++, len++);
 	*x = '\0';		/* Shorten at 1st Char diff */
 	if (len == name_length)	/* Ambiguous to prefix? */
 	    return (-1);	/* So stop now and save time */
@@ -800,8 +824,8 @@ tenex(inputline, inputline_size)
 }
 
 static int
-ignored(entry)
-    register Char *entry;
+ignored(item)
+    register Char *item;
 {
     struct varent *vp;
     register Char **cp;
@@ -809,7 +833,7 @@ ignored(entry)
     if ((vp = adrof(STRfignore)) == NULL || (cp = vp->vec) == NULL)
 	return (FALSE);
     for (; *cp != NULL; cp++)
-	if (is_suffix(entry, *cp))
+	if (is_suffix(item, *cp))
 	    return (TRUE);
     return (FALSE);
 }
