@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.02/RCS/sh.func.c,v 3.33 1992/05/15 23:49:22 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.04/RCS/sh.func.c,v 3.49 1993/07/03 23:47:53 christos Exp $ */
 /*
  * sh.func.c: csh builtin functions
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.func.c,v 3.33 1992/05/15 23:49:22 christos Exp $")
+RCSID("$Id: sh.func.c,v 3.49 1993/07/03 23:47:53 christos Exp $")
 
 #include "ed.h"
 #include "tw.h"
@@ -76,7 +76,7 @@ isbfunc(t)
     static struct biltins foregnd = {"%job", dofg1, 0, 0};
     static struct biltins backgnd = {"%job &", dobg1, 0, 0};
 
-    if (lastchr(cp) == ':') {
+    if (*cp != ':' && lastchr(cp) == ':') {
 	label.bname = short2str(cp);
 	return (&label);
     }
@@ -145,6 +145,7 @@ doonintr(v, c)
     register Char *cp;
     register Char *vv = v[1];
 
+    USE(c);
     if (parintr == SIG_IGN)
 	return;
     if (setintr && intty)
@@ -154,13 +155,17 @@ doonintr(v, c)
     xfree((ptr_t) cp);
     if (vv == 0) {
 #ifdef BSDSIGS
-	if (setintr)
+	if (setintr) {
 	    (void) sigblock(sigmask(SIGINT));
-	else
+	    (void) signal(SIGINT, pintr);
+	}
+	else 
 	    (void) signal(SIGINT, SIG_DFL);
 #else /* !BSDSIGS */
-	if (setintr)
+	if (setintr) {
 	    (void) sighold(SIGINT);
+	    (void) sigset(SIGINT, pintr);
+	}
 	else
 	    (void) sigset(SIGINT, SIG_DFL);
 #endif /* BSDSIGS */
@@ -190,6 +195,8 @@ donohup(v, c)
     Char **v;
     struct command *c;
 {
+    USE(c);
+    USE(v);
     if (intty)
 	stderror(ERR_NAME | ERR_TERMINAL);
     if (setintr == 0) {
@@ -206,13 +213,14 @@ dozip(v, c)
     Char **v;
     struct command *c;
 {
-    ;
+    USE(c);
+    USE(v);
 }
 
 void
 prvars()
 {
-    plist(&shvhed);
+    plist(&shvhed, VAR_ALL);
 }
 
 /*ARGSUSED*/
@@ -224,10 +232,11 @@ doalias(v, c)
     register struct varent *vp;
     register Char *p;
 
+    USE(c);
     v++;
     p = *v++;
     if (p == 0)
-	plist(&aliases);
+	plist(&aliases, VAR_ALL);
     else if (*v == 0) {
 	vp = adrof1(strip(p), &aliases);
 	if (vp)
@@ -238,7 +247,7 @@ doalias(v, c)
 	    setname(short2str(p));
 	    stderror(ERR_NAME | ERR_DANGER);
 	}
-	set1(strip(p), saveblk(v), &aliases);
+	set1(strip(p), saveblk(v), &aliases, VAR_READWRITE);
 	tw_cmd_free();
     }
 }
@@ -249,6 +258,7 @@ unalias(v, c)
     Char  **v;
     struct command *c;
 {
+    USE(c);
     unset1(v, &aliases);
     tw_cmd_free();
 }
@@ -259,6 +269,8 @@ dologout(v, c)
     Char **v;
     struct command *c;
 {
+    USE(c);
+    USE(v);
     islogin();
     goodbye(NULL, NULL);
 }
@@ -269,8 +281,9 @@ dologin(v, c)
     Char  **v;
     struct command *c;
 {
+    USE(c);
     islogin();
-    rechist();
+    rechist(NULL);
     (void) signal(SIGTERM, parterm);
     (void) execl(_PATH_LOGIN, "login", short2str(v[1]), NULL);
     untty();
@@ -285,11 +298,18 @@ donewgrp(v, c)
     Char  **v;
     struct command *c;
 {
+    char **p;
     if (chkstop == 0 && setintr)
 	panystop(0);
     (void) signal(SIGTERM, parterm);
-    (void) execl(_PATH_BIN_NEWGRP, "newgrp", short2str(v[1]), NULL);
-    (void) execl(_PATH_USRBIN_NEWGRP, "newgrp", short2str(v[1]), NULL);
+    p = short2blk(&v[1]);
+    /*
+     * From Beto Appleton (beto@aixwiz.austin.ibm.com)
+     * Newgrp can take 2 arguments...
+     */
+    (void) execv(_PATH_BIN_NEWGRP, "newgrp", p, NULL);
+    (void) execv(_PATH_USRBIN_NEWGRP, "newgrp", p, NULL);
+    blkfree((Char **) p);
     untty();
     xexit(1);
 }
@@ -327,7 +347,7 @@ doif(v, kp)
 	 * following code.
 	 */
 	if (!i)
-	    search(T_IF, 0, NULL);
+	    search(TC_IF, 0, NULL);
 	return;
     }
     /*
@@ -365,7 +385,9 @@ doelse (v, c)
     Char **v;
     struct command *c;
 {
-    search(T_ELSE, 0, NULL);
+    USE(c);
+    USE(v);
+    search(TC_ELSE, 0, NULL);
 }
 
 /*ARGSUSED*/
@@ -376,6 +398,7 @@ dogoto(v, c)
 {
     Char   *lp;
 
+    USE(c);
     gotolab(lp = globone(v[1], G_ERROR));
     xfree((ptr_t) lp);
 }
@@ -389,16 +412,16 @@ gotolab(lab)
      * While we still can, locate any unknown ends of existing loops. This
      * obscure code is the WORST result of the fact that we don't really parse.
      */
-    zlast = T_GOTO;
+    zlast = TC_GOTO;
     for (wp = whyles; wp; wp = wp->w_next)
 	if (wp->w_end.type == F_SEEK && wp->w_end.f_seek == 0) {
-	    search(T_BREAK, 0, NULL);
+	    search(TC_BREAK, 0, NULL);
 	    btell(&wp->w_end);
 	}
 	else {
 	    bseek(&wp->w_end);
 	}
-    search(T_GOTO, 0, lab);
+    search(TC_GOTO, 0, lab);
     /*
      * Eliminate loops which were exited.
      */
@@ -413,6 +436,7 @@ doswitch(v, c)
 {
     register Char *cp, *lp;
 
+    USE(c);
     v++;
     if (!*v || *(*v++) != '(')
 	stderror(ERR_SYNTAX);
@@ -421,7 +445,7 @@ doswitch(v, c)
 	v--;
     if (*v)
 	stderror(ERR_SYNTAX);
-    search(T_SWITCH, 0, lp = globone(cp, G_ERROR));
+    search(TC_SWITCH, 0, lp = globone(cp, G_ERROR));
     xfree((ptr_t) lp);
 }
 
@@ -431,6 +455,8 @@ dobreak(v, c)
     Char **v;
     struct command *c;
 {
+    USE(v);
+    USE(c);
     if (whyles)
 	toend();
     else
@@ -443,6 +469,7 @@ doexit(v, c)
     Char  **v;
     struct command *c;
 {
+    USE(c);
     if (chkstop == 0 && (intty || intact) && evalvec == 0)
 	panystop(0);
     /*
@@ -450,7 +477,7 @@ doexit(v, c)
      */
     v++;
     if (*v) {
-	set(STRstatus, putn(expr(&v)));
+	set(STRstatus, putn(expr(&v)), VAR_READWRITE);
 	if (*v)
 	    stderror(ERR_NAME | ERR_EXPRESSION);
     }
@@ -468,6 +495,7 @@ doforeach(v, c)
     register Char *cp, *sp;
     register struct whyle *nwp;
 
+    USE(c);
     v++;
     sp = cp = strip(*v);
     if (!letter(*sp))
@@ -483,9 +511,15 @@ doforeach(v, c)
 	stderror(ERR_NAME | ERR_NOPAREN);
     v++;
     gflag = 0, tglob(v);
-    v = globall(v);
-    if (v == 0)
-	stderror(ERR_NAME | ERR_NOMATCH);
+    if (gflag) {
+	v = globall(v);
+	if (v == 0)
+	    stderror(ERR_NAME | ERR_NOMATCH);
+    }
+    else {
+	v = gargv = saveblk(v);
+	trim(v);
+    }
     nwp = (struct whyle *) xcalloc(1, sizeof *nwp);
     nwp->w_fe = nwp->w_fe0 = v;
     gargv = 0;
@@ -497,7 +531,7 @@ doforeach(v, c)
     /*
      * Pre-read the loop so as to be more comprehensible to a terminal user.
      */
-    zlast = T_FOREACH;
+    zlast = TC_FOREACH;
     if (intty)
 	preread();
     doagain();
@@ -510,9 +544,11 @@ dowhile(v, c)
     struct command *c;
 {
     register int status;
-    register bool again = whyles != 0 && SEEKEQ(&whyles->w_start, &lineloc) &&
-    whyles->w_fename == 0;
+    register bool again = whyles != 0 && 
+			  SEEKEQ(&whyles->w_start, &lineloc) &&
+			  whyles->w_fename == 0;
 
+    USE(c);
     v++;
     /*
      * Implement prereading here also, taking care not to evaluate the
@@ -533,7 +569,7 @@ dowhile(v, c)
 	nwp->w_end.f_seek = 0;
 	nwp->w_next = whyles;
 	whyles = nwp;
-	zlast = T_WHILE;
+	zlast = TC_WHILE;
 	if (intty) {
 	    /*
 	     * The tty preread
@@ -558,7 +594,7 @@ preread()
 #else /* !BSDSIGS */
 	(void) sigrelse (SIGINT);
 #endif /* BSDSIGS */
-    search(T_BREAK, 0, NULL);		/* read the expression in */
+    search(TC_BREAK, 0, NULL);		/* read the expression in */
     if (setintr)
 #ifdef BSDSIGS
 	(void) sigblock(sigmask(SIGINT));
@@ -574,6 +610,8 @@ doend(v, c)
     Char **v;
     struct command *c;
 {
+    USE(v);
+    USE(c);
     if (!whyles)
 	stderror(ERR_NAME | ERR_NOTWHILE);
     btell(&whyles->w_end);
@@ -586,6 +624,8 @@ docontin(v, c)
     Char **v;
     struct command *c;
 {
+    USE(v);
+    USE(c);
     if (!whyles)
 	stderror(ERR_NAME | ERR_NOTWHILE);
     doagain();
@@ -608,7 +648,7 @@ doagain()
 	dobreak(NULL, NULL);
 	return;
     }
-    set(whyles->w_fename, Strsave(*whyles->w_fe++));
+    set(whyles->w_fename, quote(Strsave(*whyles->w_fe++)), VAR_READWRITE);
     bseek(&whyles->w_start);
 }
 
@@ -657,7 +697,9 @@ doswbrk(v, c)
     Char **v;
     struct command *c;
 {
-    search(T_BRKSW, 0, NULL);
+    USE(v);
+    USE(c);
+    search(TC_BRKSW, 0, NULL);
 }
 
 int
@@ -710,9 +752,9 @@ search(type, level, goal)
     register Char *aword = wordbuf;
     register Char *cp;
 
-    Stype = type;
+    Stype = (Char) type;
     Sgoal = goal;
-    if (type == T_GOTO) {
+    if (type == TC_GOTO) {
 	struct Ain a;
 	a.type = F_SEEK;
 	a.f_seek = 0;
@@ -720,69 +762,69 @@ search(type, level, goal)
     }
     do {
 	if (intty && fseekp == feobp && aret == F_SEEK)
-	    printprompt(1, isrchx(type == T_BREAK ? zlast : type));
+	    printprompt(1, isrchx(type == TC_BREAK ? zlast : type));
 	/* xprintf("? "), flush(); */
 	aword[0] = 0;
 	(void) getword(aword);
 	switch (srchx(aword)) {
 
-	case T_ELSE:
-	    if (level == 0 && type == T_IF)
+	case TC_ELSE:
+	    if (level == 0 && type == TC_IF)
 		return;
 	    break;
 
-	case T_IF:
+	case TC_IF:
 	    while (getword(aword))
 		continue;
-	    if ((type == T_IF || type == T_ELSE) &&
+	    if ((type == TC_IF || type == TC_ELSE) &&
 		eq(aword, STRthen))
 		level++;
 	    break;
 
-	case T_ENDIF:
-	    if (type == T_IF || type == T_ELSE)
+	case TC_ENDIF:
+	    if (type == TC_IF || type == TC_ELSE)
 		level--;
 	    break;
 
-	case T_FOREACH:
-	case T_WHILE:
-	    if (type == T_BREAK)
+	case TC_FOREACH:
+	case TC_WHILE:
+	    if (type == TC_BREAK)
 		level++;
 	    break;
 
-	case T_END:
-	    if (type == T_BREAK)
+	case TC_END:
+	    if (type == TC_BREAK)
 		level--;
 	    break;
 
-	case T_SWITCH:
-	    if (type == T_SWITCH || type == T_BRKSW)
+	case TC_SWITCH:
+	    if (type == TC_SWITCH || type == TC_BRKSW)
 		level++;
 	    break;
 
-	case T_ENDSW:
-	    if (type == T_SWITCH || type == T_BRKSW)
+	case TC_ENDSW:
+	    if (type == TC_SWITCH || type == TC_BRKSW)
 		level--;
 	    break;
 
-	case T_LABEL:
-	    if (type == T_GOTO && getword(aword) && eq(aword, goal))
+	case TC_LABEL:
+	    if (type == TC_GOTO && getword(aword) && eq(aword, goal))
 		level = -1;
 	    break;
 
 	default:
-	    if (type != T_GOTO && (type != T_SWITCH || level != 0))
+	    if (type != TC_GOTO && (type != TC_SWITCH || level != 0))
 		break;
 	    if (lastchr(aword) != ':')
 		break;
 	    aword[Strlen(aword) - 1] = 0;
-	    if ((type == T_GOTO && eq(aword, goal)) ||
-		(type == T_SWITCH && eq(aword, STRdefault)))
+	    if ((type == TC_GOTO && eq(aword, goal)) ||
+		(type == TC_SWITCH && eq(aword, STRdefault)))
 		level = -1;
 	    break;
 
-	case T_CASE:
-	    if (type != T_SWITCH || level != 0)
+	case TC_CASE:
+	    if (type != TC_SWITCH || level != 0)
 		break;
 	    (void) getword(aword);
 	    if (lastchr(aword) == ':')
@@ -793,8 +835,8 @@ search(type, level, goal)
 	    xfree((ptr_t) cp);
 	    break;
 
-	case T_DEFAULT:
-	    if (type == T_SWITCH && level == 0)
+	case TC_DEFAULT:
+	    if (type == TC_SWITCH && level == 0)
 		level = -1;
 	    break;
 	}
@@ -863,24 +905,24 @@ getword(wp)
 past:
     switch (Stype) {
 
-    case T_IF:
+    case TC_IF:
 	stderror(ERR_NAME | ERR_NOTFOUND, "then/endif");
 	break;
 
-    case T_ELSE:
+    case TC_ELSE:
 	stderror(ERR_NAME | ERR_NOTFOUND, "endif");
 	break;
 
-    case T_BRKSW:
-    case T_SWITCH:
+    case TC_BRKSW:
+    case TC_SWITCH:
 	stderror(ERR_NAME | ERR_NOTFOUND, "endsw");
 	break;
 
-    case T_BREAK:
+    case TC_BREAK:
 	stderror(ERR_NAME | ERR_NOTFOUND, "end");
 	break;
 
-    case T_GOTO:
+    case TC_GOTO:
 	setname(short2str(Sgoal));
 	stderror(ERR_NAME | ERR_NOTFOUND, "label");
 	break;
@@ -923,7 +965,7 @@ static void
 toend()
 {
     if (whyles->w_end.type == F_SEEK && whyles->w_end.f_seek == 0) {
-	search(T_BREAK, 0, NULL);
+	search(TC_BREAK, 0, NULL);
 	btell(&whyles->w_end);
 	whyles->w_end.f_seek--;
     }
@@ -996,6 +1038,7 @@ doecho(v, c)
     Char  **v;
     struct command *c;
 {
+    USE(c);
     xecho(' ', v);
 }
 
@@ -1005,6 +1048,7 @@ doglob(v, c)
     Char  **v;
     struct command *c;
 {
+    USE(c);
     xecho(0, v);
     flush();
 }
@@ -1130,6 +1174,56 @@ done:
 	blkfree(gargv), gargv = 0;
 }
 
+/* check whether an environment variable should invoke 'set_locale()' */
+static bool islocale_var(var)
+register Char *var;
+{
+    static Char *locale_vars[] = {
+	STRLANG,	STRLC_CTYPE,	STRLC_NUMERIC,	STRLC_TIME,
+	STRLC_COLLATE,	STRLC_MESSAGES,	STRLC_MONETARY, 0
+    };
+    register Char **v;
+
+    for (v = locale_vars; *v; ++v)
+	if (eq(var, *v))
+	    return 1;
+    return 0;
+}
+
+/*ARGSUSED*/
+void
+doprintenv(v, c)
+    register Char **v;
+    struct command *c;
+{
+    Char   *e;
+    extern bool output_raw;
+    extern bool xlate_cr;
+
+    USE(c);
+    if (setintr)
+#ifdef BSDSIGS
+	(void) sigsetmask(sigblock((sigmask_t) 0) & ~sigmask(SIGINT));
+#else /* !BSDSIGS */
+	(void) sigrelse (SIGINT);
+#endif /* BSDSIGS */
+
+    v++;
+    if (*v == 0) {
+	register Char **ep;
+
+	xlate_cr = 1;
+	for (ep = STR_environ; *ep; ep++)
+	    xprintf("%S\n", *ep);
+	xlate_cr = 0;
+    }
+    else if ((e = tgetenv(*v)) != NULL) {
+	output_raw = 1;
+	xprintf("%S\n", e);
+	output_raw = 0;
+    }
+}
+
 /* from "Karl Berry." <karl%mote.umb.edu@relay.cs.net> -- for NeXT things
    (and anything else with a modern compiler) */
 
@@ -1141,39 +1235,51 @@ dosetenv(v, c)
 {
     Char   *vp, *lp;
 
-    v++;
-    if ((vp = *v++) == 0) {
-	register Char **ep;
-
-	if (setintr)
-#ifdef BSDSIGS
-	    (void) sigsetmask(sigblock((sigmask_t) 0) & ~sigmask(SIGINT));
-#else /* !BSDSIGS */
-	    (void) sigrelse (SIGINT);
-#endif /* BSDSIGS */
-	for (ep = STR_environ; *ep; ep++)
-	    xprintf("%s\n", short2str(*ep));
+    USE(c);
+    if (*++v == 0) {
+	doprintenv(--v, 0);
 	return;
     }
+
+    vp = *v++;
+
     if ((lp = *v++) == 0)
 	lp = STRNULL;
-    Setenv(vp, lp = globone(lp, G_APPEND));
-    if (eq(vp, STRPATH)) {
+
+    tsetenv(vp, lp = globone(lp, G_APPEND));
+    if (eq(vp, STRKPATH)) {
 	importpath(lp);
 	dohash(NULL, NULL);
+	xfree((ptr_t) lp);
+	return;
     }
+
 #ifdef apollo
-    else if (eq(vp, STRSYSTYPE))
+    if (eq(vp, STRSYSTYPE)) {
 	dohash(NULL, NULL);
+	xfree((ptr_t) lp);
+	return;
+    }
 #endif /* apollo */
-    else if (eq(vp, STRLANG) || eq(vp, STRLC_CTYPE)) {
+
+    if (islocale_var(vp)) {
 #ifdef NLS
 	int     k;
 
+# ifdef SETLOCALEBUG
+	dont_free = 1;
+# endif /* SETLOCALEBUG */
 	(void) setlocale(LC_ALL, "");
 # ifdef LC_COLLATE
 	(void) setlocale(LC_COLLATE, "");
 # endif
+# ifdef SETLOCALEBUG
+	dont_free = 0;
+# endif /* SETLOCALEBUG */
+# ifdef STRCOLLBUG
+	fix_strcoll_bug();
+# endif /* STRCOLLBUG */
+	tw_cmd_free();	/* since the collation sequence has changed */
 	for (k = 0200; k <= 0377 && !Isprint(k); k++)
 	    continue;
 	AsciiOnly = k > 0377;
@@ -1184,41 +1290,67 @@ dosetenv(v, c)
 	ed_Init();
 	if (MapsAreInited && !NLSMapsAreInited)
 	    ed_InitNLSMaps();
+	xfree((ptr_t) lp);
+	return;
     }
-    else if (eq(vp, STRNOREBIND)) {
+
+    if (eq(vp, STRNOREBIND)) {
 	NoNLSRebind = 1;
+	xfree((ptr_t) lp);
+	return;
     }
-    else if (eq(vp, STRTERM)) {
-	set(STRterm, Strsave(lp));
+
+    if (eq(vp, STRKTERM)) {
+	set(STRterm, quote(lp), VAR_READWRITE);	/* lp memory used here */
 	GotTermCaps = 0;
 	ed_Init();
+	return;
     }
-    else if (eq(vp, STRHOME)) {
-	Char *cp = Strsave(value(vp));	/* get the old value back */
 
+    if (eq(vp, STRKHOME)) {
 	/*
 	 * convert to canonical pathname (possibly resolving symlinks)
 	 */
-	cp = dcanon(cp, cp);
-
-	set(STRhome, Strsave(cp));	/* have to save the new val */
+	lp = dcanon(lp, lp);
+	set(STRhome, quote(lp), VAR_READWRITE);	/* cp memory used here */
 
 	/* fix directory stack for new tilde home */
 	dtilde();
-	xfree((ptr_t) cp);
+	return;
     }
-    else if (eq(vp, STRSHLVL)) 
-	set(STRshlvl, Strsave(lp));
-    else if (eq(vp, STRUSER))
-	set(STRuser, Strsave(lp));
+
+    if (eq(vp, STRKSHLVL)) {
+	/* lp memory used here */
+	set(STRshlvl, quote(lp), VAR_READWRITE);
+	return;
+    }
+
+    if (eq(vp, STRKUSER)) {
+	set(STRuser, quote(lp), VAR_READWRITE);	/* lp memory used here */
+	return;
+    }
+
 #ifdef SIG_WINDOW
-    else if ((eq(lp, STRNULL) &&
-	      (eq(vp, STRLINES) || eq(vp, STRCOLUMNS))) ||
-	     eq(vp, STRTERMCAP)) {
+    /*
+     * Load/Update $LINES $COLUMNS
+     */
+    if ((eq(lp, STRNULL) && (eq(vp, STRLINES) || eq(vp, STRCOLUMNS))) ||
+	eq(vp, STRTERMCAP)) {
+	xfree((ptr_t) lp);
 	check_window_size(1);
+	return;
+    }
+
+    /*
+     * Change the size to the one directed by $LINES and $COLUMNS
+     */
+    if (eq(vp, STRLINES) || eq(vp, STRCOLUMNS)) {
+	GotTermCaps = 0;
+	xfree((ptr_t) lp);
+	ed_Init();
+	return;
     }
 #endif /* SIG_WINDOW */
-    xfree((ptr_t) lp);
 }
 
 /*ARGSUSED*/
@@ -1231,6 +1363,7 @@ dounsetenv(v, c)
     int     i, maxi;
     static Char *name = NULL;
 
+    USE(c);
     if (name)
 	xfree((ptr_t) name);
     /*
@@ -1254,20 +1387,39 @@ dounsetenv(v, c)
 		if (!Gmatch(name, *v))
 		    continue;
 		maxi = 1;
+
+		/* Unset the name. This wasn't being done until
+		 * later but most of the stuff following won't
+		 * work (particularly the setlocale() and getenv()
+		 * stuff) as intended until the name is actually
+		 * removed. (sg)
+		 */
+		Unsetenv(name);
+
 		if (eq(name, STRNOREBIND))
 		    NoNLSRebind = 0;
 #ifdef apollo
 		else if (eq(name, STRSYSTYPE))
 		    dohash(NULL, NULL);
 #endif /* apollo */
-		else if (eq(name, STRLANG) || eq(name, STRLC_CTYPE)) {
+		else if (islocale_var(name)) {
 #ifdef NLS
 		    int     k;
 
+# ifdef SETLOCALEBUG
+		    dont_free = 1;
+# endif /* SETLOCALEBUG */
 		    (void) setlocale(LC_ALL, "");
 # ifdef LC_COLLATE
 		    (void) setlocale(LC_COLLATE, "");
 # endif
+# ifdef SETLOCALEBUG
+		    dont_free = 0;
+# endif /* SETLOCALEBUG */
+# ifdef STRCOLLBUG
+		    fix_strcoll_bug();
+# endif /* STRCOLLBUG */
+		    tw_cmd_free();/* since the collation sequence has changed */
 		    for (k = 0200; k <= 0377 && !Isprint(k); k++)
 			continue;
 		    AsciiOnly = k > 0377;
@@ -1282,16 +1434,15 @@ dounsetenv(v, c)
 
 		}
 		/*
-		 * Delete name, and start again cause the environment changes
+		 * start again cause the environment changes
 		 */
-		Unsetenv(name);
 		break;
 	    }
     xfree((ptr_t) name); name = NULL;
 }
 
 void
-Setenv(name, val)
+tsetenv(name, val)
     Char   *name, *val;
 {
 #ifdef SETENV_IN_LIB
@@ -1315,9 +1466,8 @@ Setenv(name, val)
     Char   *blk[2];
     Char  **oep = ep;
 
-
     for (; *ep; ep++) {
-	for (cp = name, dp = *ep; *cp && *cp == *dp; cp++, dp++)
+	for (cp = name, dp = *ep; *cp && (*cp & TRIM) == *dp; cp++, dp++)
 	    continue;
 	if (*cp != 0 || *dp != '=')
 	    continue;
@@ -1373,6 +1523,7 @@ doumask(v, c)
     register Char *cp = v[1];
     register int i;
 
+    USE(c);
     if (cp == 0) {
 	i = umask(0);
 	(void) umask(i);
@@ -1388,7 +1539,7 @@ doumask(v, c)
 }
 
 #ifndef HAVENOLIMIT
-# ifndef BSDTIMES
+# ifndef BSDLIMIT
    typedef long RLIM_TYPE;
 #  ifndef RLIM_INFINITY
 #   ifndef _MINIX
@@ -1404,75 +1555,111 @@ doumask(v, c)
 #  else /* aiws */
 #   define toset(a) ((a) + 1)
 #  endif /* aiws */
-# else /* BSDTIMES */
-   typedef int RLIM_TYPE;
-# endif /* BSDTIMES */
+# else /* BSDLIMIT */
+#  if defined(BSD4_4) && !defined(__386BSD__) && !defined(bsdi)
+    typedef quad_t RLIM_TYPE;
+#  else
+    typedef unsigned long RLIM_TYPE;
+#  endif /* BSD4_4 && !__386BSD__ && !defined(bsdi) */
+# endif /* BSDLIMIT */
 
+# if defined(hpux) && defined(BSDLIMIT)
+/* Yes hpux8.0 has limits but <sys/resource.h> does not make them public */
+/* Yes, we could have defined _KERNEL, and -I/etc/conf/h, but is that better? */
+#  ifndef RLIMIT_CPU
+#   define RLIMIT_CPU		0
+#   define RLIMIT_FSIZE		1
+#   define RLIMIT_DATA		2
+#   define RLIMIT_STACK		3
+#   define RLIMIT_CORE		4
+#   define RLIMIT_RSS		5
+#   define RLIMIT_NOFILE	6
+#  endif /* RLIMIT_CPU */
+#  ifndef RLIM_INFINITY
+#   define RLIM_INFINITY	0x7fffffff
+#  endif /* RLIM_INFINITY */
+# endif /* hpux && BSDLIMIT */
+
+# if SYSVREL > 3 && defined(BSDLIMIT)
+/* In order to use rusage, we included "/usr/ucbinclude/sys/resource.h" in */
+/* sh.h.  However, some SVR4 limits are defined in <sys/resource.h>.  Rather */
+/* than include both and get warnings, we define the extra SVR4 limits here. */
+#  ifndef RLIMIT_VMEM
+#   define RLIMIT_VMEM	6
+#  endif
+#  ifndef RLIMIT_AS
+#   define RLIMIT_AS	RLIMIT_VMEM
+#  endif
+# endif /* SYSVREL > 3 && BSDLIMIT */
 
 struct limits limits[] = 
 {
 # ifdef RLIMIT_CPU
-    RLIMIT_CPU, 	"cputime",	1,	"seconds",
+    { RLIMIT_CPU, 	"cputime",	1,	"seconds"	},
 # endif /* RLIMIT_CPU */
 
 # ifdef RLIMIT_FSIZE
 #  ifndef aiws
-    RLIMIT_FSIZE, 	"filesize",	1024,	"kbytes",
+    { RLIMIT_FSIZE, 	"filesize",	1024,	"kbytes"	},
 #  else
-    RLIMIT_FSIZE, 	"filesize",	512,	"blocks",
+    { RLIMIT_FSIZE, 	"filesize",	512,	"blocks"	},
 #  endif /* aiws */
 # endif /* RLIMIT_FSIZE */
 
 # ifdef RLIMIT_DATA
-    RLIMIT_DATA, 	"datasize",	1024,	"kbytes",
+    { RLIMIT_DATA, 	"datasize",	1024,	"kbytes"	},
 # endif /* RLIMIT_DATA */
 
 # ifdef RLIMIT_STACK
 #  ifndef aiws
-    RLIMIT_STACK, 	"stacksize",	1024,	"kbytes",
+    { RLIMIT_STACK, 	"stacksize",	1024,	"kbytes"	},
 #  else
-    RLIMIT_STACK, 	"stacksize",	1024 * 1024,	"kbytes",
+    { RLIMIT_STACK, 	"stacksize",	1024 * 1024,	"kbytes"},
 #  endif /* aiws */
 # endif /* RLIMIT_STACK */
 
 # ifdef RLIMIT_CORE
-    RLIMIT_CORE, 	"coredumpsize",	1024,	"kbytes",
+    { RLIMIT_CORE, 	"coredumpsize",	1024,	"kbytes"	},
 # endif /* RLIMIT_CORE */
 
 # ifdef RLIMIT_RSS
-    RLIMIT_RSS, 	"memoryuse",	1024,	"kbytes",
+    { RLIMIT_RSS, 	"memoryuse",	1024,	"kbytes"	},
 # endif /* RLIMIT_RSS */
 
+# ifdef RLIMIT_VMEM
+    { RLIMIT_VMEM, 	"vmemoryuse",	1024,	"kbytes"	},
+# endif /* RLIMIT_VMEM */
+
 # ifdef RLIMIT_NOFILE
-    RLIMIT_NOFILE, 	"descriptors", 1,	"",
+    { RLIMIT_NOFILE, 	"descriptors", 1,	""		},
 # endif /* RLIMIT_NOFILE */
 
 # ifdef RLIMIT_CONCUR
-    RLIMIT_CONCUR, 	"concurrency", 1,	"thread(s)",
+    { RLIMIT_CONCUR, 	"concurrency", 1,	"thread(s)"	},
 # endif /* RLIMIT_CONCUR */
 
 # ifdef RLIMIT_MEMLOCK
-    RLIMIT_MEMLOCK,	"memorylocked",	1024,	"kbytes",
+    { RLIMIT_MEMLOCK,	"memorylocked",	1024,	"kbytes"	},
 # endif /* RLIMIT_MEMLOCK */
 
 # ifdef RLIMIT_NPROC
-    RLIMIT_NPROC,	"maxproc",	1,	"",
+    { RLIMIT_NPROC,	"maxproc",	1,	""		},
 # endif /* RLIMIT_NPROC */
 
 # ifdef RLIMIT_OFILE
-    RLIMIT_OFILE,	"openfiles",	1,	"",
+    { RLIMIT_OFILE,	"openfiles",	1,	""		},
 # endif /* RLIMIT_OFILE */
 
-    -1, 		NULL, 		0, 	NULL
+    { -1, 		NULL, 		0, 	NULL		}
 };
 
-static struct limits *findlim();
-static RLIM_TYPE getval();
-static void limtail();
-static void plim();
-static int setlim();
+static struct limits *findlim	__P((Char *));
+static RLIM_TYPE getval		__P((struct limits *, Char **));
+static void limtail		__P((Char *, char*));
+static void plim		__P((struct limits *, int));
+static int setlim		__P((struct limits *, int, RLIM_TYPE));
 
-#if defined(convex) || defined(__convex__)
+#ifdef convex
 static  RLIM_TYPE
 restrict_limit(value)
     double  value;
@@ -1481,12 +1668,14 @@ restrict_limit(value)
      * is f too large to cope with? return the maximum or minimum int
      */
     if (value > (double) INT_MAX)
-	return (INT_MAX);
+	return (RLIM_TYPE) INT_MAX;
     else if (value < (double) INT_MIN)
-	return (INT_MIN);
+	return (RLIM_TYPE) INT_MIN;
     else
-	return ((int) value);
+	return (RLIM_TYPE) value;
 }
+#else /* !convex */
+# define restrict_limit(x)	((RLIM_TYPE) (x))
 #endif /* convex */
 
 
@@ -1518,8 +1707,9 @@ dolimit(v, c)
 {
     register struct limits *lp;
     register RLIM_TYPE limit;
-    char    hard = 0;
+    int    hard = 0;
 
+    USE(c);
     v++;
     if (*v && eq(*v, STRmh)) {
 	hard = 1;
@@ -1545,20 +1735,15 @@ getval(lp, v)
     register struct limits *lp;
     Char  **v;
 {
-# if defined(convex) || defined(__convex__)
-    RLIM_TYPE restrict_limit __P((double));
-# endif /* convex */
-
     register float f;
 #ifndef atof	/* This can be a macro on linux */
     extern double  atof __P((const char *));
 #endif /* atof */
-    static int lmin = 0x80000000, lmax = 0x7fffffff;
     Char   *cp = *v++;
 
     f = atof(short2str(cp));
 
-# if defined(convex) || defined(__convex__)
+# ifdef convex
     /*
      * is f too large to cope with. limit f to minint, maxint  - X-6768 by
      * strike
@@ -1572,11 +1757,7 @@ getval(lp, v)
 	cp++;
     if (*cp == 0) {
 	if (*v == 0)
-# if defined(convex) || defined(__convex__)
-	    return ((RLIM_TYPE) restrict_limit((f + 0.5) * lp->limdiv));
-# else /* convex */
-	    return ((RLIM_TYPE) ((f + 0.5) * lp->limdiv));
-# endif /* convex */
+	    return restrict_limit((f + 0.5) * lp->limdiv);
 	cp = *v;
     }
     switch (*cp) {
@@ -1584,12 +1765,7 @@ getval(lp, v)
     case ':':
 	if (lp->limconst != RLIMIT_CPU)
 	    goto badscal;
-#  if defined(convex) || defined(__convex__)
-	return ((RLIM_TYPE)
-		restrict_limit((f * 60.0 + atof(short2str(cp + 1)))));
-#  else /* convex */
-	return ((RLIM_TYPE) (f * 60.0 + atof(short2str(cp + 1))));
-#  endif /* convex */
+	return restrict_limit((f * 60.0 + atof(short2str(cp + 1))));
     case 'h':
 	if (lp->limconst != RLIMIT_CPU)
 	    goto badscal;
@@ -1646,14 +1822,12 @@ badscal:
 # endif /* RLIMIT_CPU */
 	stderror(ERR_NAME | ERR_SCALEF);
     }
-# if defined(convex) || defined(__convex__)
-    return ((RLIM_TYPE) restrict_limit((f + 0.5)));
+# ifdef convex
+    return restrict_limit((f + 0.5));
 # else
     f += 0.5;
-    if (f > (float) lmax)
-	return lmax;
-    else if (f < (float) lmin)
-	return lmin;
+    if (f > (float) RLIM_INFINITY)
+	return RLIM_INFINITY;
     else
 	return ((RLIM_TYPE) f);
 # endif /* convex */
@@ -1675,44 +1849,49 @@ limtail(cp, str)
 static void
 plim(lp, hard)
     register struct limits *lp;
-    Char    hard;
+    int hard;
 {
-# ifdef BSDTIMES
+# ifdef BSDLIMIT
     struct rlimit rlim;
-# endif /* BSDTIMES */
+# endif /* BSDLIMIT */
     RLIM_TYPE limit;
+    int     div = lp->limdiv;
 
     xprintf("%s \t", lp->limname);
 
-# ifndef BSDTIMES
+# ifndef BSDLIMIT
     limit = ulimit(lp->limconst, 0);
 #  ifdef aiws
     if (lp->limconst == RLIMIT_DATA)
 	limit -= 0x20000000;
 #  endif /* aiws */
-# else /* BSDTIMES */
+# else /* BSDLIMIT */
     (void) getrlimit(lp->limconst, &rlim);
     limit = hard ? rlim.rlim_max : rlim.rlim_cur;
-# endif /* BSDTIMES */
+# endif /* BSDLIMIT */
+
+# if !defined(BSDLIMIT) || defined(hpux)
+    /*
+     * Christos: filesize comes in 512 blocks. we divide by 2 to get 1024
+     * blocks. Note we cannot pre-multiply cause we might overflow (A/UX)
+     */
+    if (lp->limconst == RLIMIT_FSIZE) {
+	if (limit >= (RLIM_INFINITY / 512))
+	    limit = RLIM_INFINITY;
+	else
+	    div = (div == 1024 ? 2 : 1);
+    }
+# endif /* !BSDLIMIT || hpux */
 
     if (limit == RLIM_INFINITY)
 	xprintf("unlimited");
+    else
 # ifdef RLIMIT_CPU
-    else if (lp->limconst == RLIMIT_CPU)
+    if (lp->limconst == RLIMIT_CPU)
 	psecs((long) limit);
+    else
 # endif /* RLIMIT_CPU */
-    else
-# ifndef BSDTIMES
-    if (lp->limconst == RLIMIT_FSIZE)
-	/*
-	 * Christos: filesize comes in 512 blocks. we divide by 2 to get 1024
-	 * blocks. Note we cannot pre-multiply cause we might overflow (A/UX)
-	 */
-	xprintf("%ld %s", (long) (limit / (lp->limdiv == 1024 ? 2 : 1)), 
-	        lp->limscale);
-    else
-# endif /* BSDTIMES */
-	xprintf("%ld %s", (long) (limit / lp->limdiv), lp->limscale);
+	xprintf("%ld %s", (long) (limit / div), lp->limscale);
     xputchar('\n');
 }
 
@@ -1723,25 +1902,38 @@ dounlimit(v, c)
     struct command *c;
 {
     register struct limits *lp;
-    int     lerr = 0;
-    Char    hard = 0;
+    int    lerr = 0;
+    int    hard = 0;
+    int	   force = 0;
 
-    v++;
-    if (*v && eq(*v, STRmh)) {
-	hard = 1;
-	v++;
+    USE(c);
+    while (*++v && **v == '-') {
+	Char   *vp = *v;
+	while (*++vp)
+	    switch (*vp) {
+	    case 'f':
+		force = 1;
+		break;
+	    case 'h':
+		hard = 1;
+		break;
+	    default:
+		stderror(ERR_ULIMUS);
+		break;
+	    }
     }
+
     if (*v == 0) {
 	for (lp = limits; lp->limconst >= 0; lp++)
 	    if (setlim(lp, hard, (RLIM_TYPE) RLIM_INFINITY) < 0)
 		lerr++;
-	if (lerr)
+	if (!force && lerr)
 	    stderror(ERR_SILENT);
 	return;
     }
     while (*v) {
 	lp = findlim(*v++);
-	if (setlim(lp, hard, (RLIM_TYPE) RLIM_INFINITY) < 0)
+	if (setlim(lp, hard, (RLIM_TYPE) RLIM_INFINITY) < 0 && !force)
 	    stderror(ERR_SILENT);
     }
 }
@@ -1749,23 +1941,28 @@ dounlimit(v, c)
 static int
 setlim(lp, hard, limit)
     register struct limits *lp;
-    Char    hard;
+    int    hard;
     RLIM_TYPE limit;
 {
-# ifdef BSDTIMES
+# ifdef BSDLIMIT
     struct rlimit rlim;
 
     (void) getrlimit(lp->limconst, &rlim);
 
+#  ifdef hpux
+    /* Even though hpux has setrlimit(), it expects fsize in 512 byte blocks */
+    if (limit != RLIM_INFINITY && lp->limconst == RLIMIT_FSIZE)
+	limit /= 512;
+#  endif /* hpux */
     if (hard)
 	rlim.rlim_max = limit;
-    else if (limit == RLIM_INFINITY && geteuid() != 0)
+    else if (limit == RLIM_INFINITY && euid != 0)
 	rlim.rlim_cur = rlim.rlim_max;
     else
 	rlim.rlim_cur = limit;
 
     if (setrlimit(lp->limconst, &rlim) < 0) {
-# else /* BSDTIMES */
+# else /* BSDLIMIT */
     if (limit != RLIM_INFINITY && lp->limconst == RLIMIT_FSIZE)
 	limit /= 512;
 # ifdef aiws
@@ -1773,7 +1970,7 @@ setlim(lp, hard, limit)
 	limit += 0x20000000;
 # endif /* aiws */
     if (ulimit(toset(lp->limconst), limit) < 0) {
-# endif /* BSDTIMES */
+# endif /* BSDLIMIT */
 	xprintf("%s: %s: Can't %s%s limit\n", bname, lp->limname,
 		limit == RLIM_INFINITY ? "remove" : "set",
 		hard ? " hard" : "");
@@ -1793,8 +1990,11 @@ dosuspend(v, c)
 #ifdef BSDJOBS
     int     ctpgrp;
 
-    sigret_t(*old) ();
+    sigret_t (*old) ();
 #endif /* BSDJOBS */
+    
+    USE(c);
+    USE(v);
 
     if (loginsh)
 	stderror(ERR_SUSPLOG);
@@ -1850,21 +2050,22 @@ doeval(v, c)
     Char  **oevalvec;
     Char   *oevalp;
     int     odidfds;
-#ifndef FIOCLEX
+#ifndef CLOSE_ON_EXEC
     int     odidcch;
-#endif /* FIOCLEX */
-    jmp_buf osetexit;
+#endif /* CLOSE_ON_EXEC */
+    jmp_buf_t osetexit;
     int     my_reenter;
     Char  **savegv;
     int     saveIN, saveOUT, saveDIAG;
     int     oSHIN, oSHOUT, oSHDIAG;
 
+    USE(c);
     oevalvec = evalvec;
     oevalp = evalp;
     odidfds = didfds;
-#ifndef FIOCLEX
+#ifndef CLOSE_ON_EXEC
     odidcch = didcch;
-#endif /* FIOCLEX */
+#endif /* CLOSE_ON_EXEC */
     oSHIN = SHIN;
     oSHOUT = SHOUT;
     oSHDIAG = SHDIAG;
@@ -1908,9 +2109,9 @@ doeval(v, c)
 	SHIN = dcopy(0, -1);
 	SHOUT = dcopy(1, -1);
 	SHDIAG = dcopy(2, -1);
-#ifndef FIOCLEX
+#ifndef CLOSE_ON_EXEC
 	didcch = 0;
-#endif /* FIOCLEX */
+#endif /* CLOSE_ON_EXEC */
 	didfds = 0;
 	process(0);
     }
@@ -1918,9 +2119,9 @@ doeval(v, c)
     evalvec = oevalvec;
     evalp = oevalp;
     doneinp = 0;
-#ifndef FIOCLEX
+#ifndef CLOSE_ON_EXEC
     didcch = odidcch;
-#endif /* FIOCLEX */
+#endif /* CLOSE_ON_EXEC */
     didfds = odidfds;
     (void) close(SHIN);
     (void) close(SHOUT);
@@ -1937,3 +2138,57 @@ doeval(v, c)
     if (my_reenter)
 	stderror(ERR_SILENT);
 }
+
+/*************************************************************************/
+/* print list of builtin commands */
+
+/*ARGSUSED*/
+void
+dobuiltins(v, c)
+Char **v;
+struct command *c;
+{
+    /* would use print_by_column() in tw.parse.c but that assumes
+     * we have an array of Char * to pass.. (sg)
+     */
+    extern int Tty_raw_mode;
+    extern int TermH;		/* from the editor routines */
+    extern int lbuffed;		/* from sh.print.c */
+
+    register struct biltins *b;
+    register int row, col, columns, rows;
+    unsigned int w, maxwidth;
+
+    USE(c);
+    USE(v);
+    lbuffed = 0;		/* turn off line buffering */
+
+    /* find widest string */
+    for (maxwidth = 0, b = bfunc; b < &bfunc[nbfunc]; ++b)
+	maxwidth = max(maxwidth, strlen(b->bname));
+    ++maxwidth;					/* for space */
+
+    columns = (TermH + 1) / maxwidth;	/* PWP: terminal size change */
+    if (!columns)
+	columns = 1;
+    rows = (nbfunc + (columns - 1)) / columns;
+
+    for (b = bfunc, row = 0; row < rows; row++) {
+	for (col = 0; col < columns; col++) {
+	    if (b < &bfunc[nbfunc]) {
+		w = strlen(b->bname);
+		xprintf("%s", b->bname);
+		if (col < (columns - 1))	/* Not last column? */
+		    for (; w < maxwidth; w++)
+			xputchar(' ');
+		++b;
+	    }
+	}
+	if (Tty_raw_mode)
+	    xputchar('\r');
+	xputchar('\n');
+    }
+
+    lbuffed = 1;		/* turn back on line buffering */
+    flush();
+}
