@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.02/RCS/sh.glob.c,v 3.22 1992/05/02 23:39:58 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.04/RCS/sh.glob.c,v 3.33 1993/06/25 21:17:12 christos Exp $ */
 /*
  * sh.glob.c: Regular expression expansion
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.glob.c,v 3.22 1992/05/02 23:39:58 christos Exp $")
+RCSID("$Id: sh.glob.c,v 3.33 1993/06/25 21:17:12 christos Exp $")
 
 #include "tc.h"
 
@@ -53,6 +53,8 @@ static int pargsiz, gargsiz;
 #define	G_CSH	2		/* string contains ~`{ characters	*/
 
 #define	GLOBSPACE	100	/* Alloc increment			*/
+#define LONGBSIZE	10240	/* Backquote expansion buffer size	*/
+
 
 #define LBRC '{'
 #define RBRC '}'
@@ -61,9 +63,9 @@ static int pargsiz, gargsiz;
 #define EOS '\0'
 
 Char  **gargv = NULL;
-long    gargc = 0;
+int     gargc = 0;
 Char  **pargv = NULL;
-static long    pargc = 0;
+static int pargc = 0;
 
 /*
  * globbing is now done in two stages. In the first pass we expand
@@ -80,7 +82,7 @@ static	Char	**globexpand	__P((Char **));
 static	int	  globbrace	__P((Char *, Char *, Char ***));
 static  void	  expbrace	__P((Char ***, Char ***, int));
 static  int	  pmatch	__P((Char *, Char *, Char **));
-static	void	  pword		__P((void));
+static	void	  pword		__P((int));
 static	void	  psave		__P((int));
 static	void	  backeval	__P((Char *, bool));
 
@@ -88,7 +90,7 @@ static Char *
 globtilde(nv, s)
     Char  **nv, *s;
 {
-    Char    gbuf[MAXPATHLEN], *gstart, *b, *u, *e;
+    Char    gbuf[BUFSIZE], *gstart, *b, *u, *e;
 #ifdef apollo
     int slash;
 #endif
@@ -96,12 +98,14 @@ globtilde(nv, s)
     gstart = gbuf;
     *gstart++ = *s++;
     u = s;
-    for (b = gstart, e = &gbuf[MAXPATHLEN - 1]; 
+    for (b = gstart, e = &gbuf[BUFSIZE - 1]; 
 	 *s && *s != '/' && *s != ':' && b < e;
 	 *b++ = *s++)
 	continue;
     *b = EOS;
     if (gethdir(gstart)) {
+	if (adrof(STRnonomatch))
+	    return (--u);
 	blkfree(nv);
 	if (*gstart)
 	    stderror(ERR_UNKUSER, short2str(gstart));
@@ -129,7 +133,7 @@ globequal(new, old)
     Char *new, *old;
 {
     int     dig;
-    Char    *b = NULL, *d;
+    Char    *b, *d;
 
     /*
      * kfk - 17 Jan 1984 - stack hack allows user to get at arbitrary dir names
@@ -158,7 +162,7 @@ globequal(new, old)
 
     /* Copy the rest of the string */
     for (d = &new[Strlen(new)]; 
-	 d < &new[MAXPATHLEN - 1] && (*d++ = *b++) != '\0';)
+	 d < &new[BUFSIZE - 1] && (*d++ = *b++) != '\0';)
 	continue;
     *d = '\0';
 
@@ -172,7 +176,7 @@ globbrace(s, p, bl)
     int     i, len;
     Char   *pm, *pe, *lm, *pl;
     Char  **nv, **vl;
-    Char    gbuf[MAXPATHLEN];
+    Char    gbuf[BUFSIZE];
     int     size = GLOBSPACE;
 
     nv = vl = (Char **) xmalloc((size_t) (sizeof(Char *) * size));
@@ -292,10 +296,8 @@ expbrace(nvp, elp, size)
 		xfree((ptr_t) bl);
 		continue;
 	    }
-	    len = blklen(bl);
 	    if (&el[len] >= &nv[size]) {
 		int     l, e;
-
 		l = &el[len] - &nv[size];
 		size += GLOBSPACE > l ? GLOBSPACE : l;
 		l = vl - nv;
@@ -305,12 +307,24 @@ expbrace(nvp, elp, size)
 		vl = nv + l;
 		el = nv + e;
 	    }
+	    /* nv vl   el     bl
+	     * |  |    |      |
+	     * -.--..--	      x--
+	     *   |            len
+	     *   vp
+	     */
 	    vp = vl--;
 	    *vp = *bl;
 	    len--;
 	    for (bp = el; bp != vp; bp--)
 		bp[len] = *bp;
 	    el += len;
+	    /* nv vl    el bl
+	     * |  |     |  |
+	     * -.-x  ---    --
+	     *   |len
+	     *   vp
+	     */
 	    vp++;
 	    for (bp = bl + 1; *bp; *vp++ = *bp++)
 		continue;
@@ -374,8 +388,7 @@ globexpand(v)
      * Step 2: expand braces
      */
     el = vl;
-    vl = nv;
-    expbrace(&vl, &el, size);
+    expbrace(&nv, &el, size);
 
 
     /*
@@ -384,7 +397,7 @@ globexpand(v)
     vl = nv;
     for (s = *vl; s; s = *++vl)
 	switch (*s) {
-	    Char gp[MAXPATHLEN], *ns;
+	    Char gp[BUFSIZE], *ns;
 	case '~':
 	    *vl = globtilde(nv, s);
 	    break;
@@ -588,7 +601,7 @@ globall(v)
 
     if (!noglob && (gflg & G_GLOB)) {
 	vl = libglob(vo);
-	if ((gflg & G_CSH) && vl != vo)
+	if (vl != vo)
 	    blkfree(vo);
     }
     else
@@ -642,12 +655,21 @@ tglob(t)
 	else if (*p == '{' &&
 		 (p[1] == '\0' || (p[1] == '}' && p[2] == '\0')))
 	    continue;
-	while ( *(c = p++) ) {
-	    /*
-	     * eat everything inside the matching backquotes
-	     */
+	/*
+	 * The following line used to be *(c = p++), but hp broke their
+	 * optimizer in 9.01, so we break the assignment into two pieces
+	 * The careful reader here will note that *most* compiler workarounds
+	 * in tcsh are either for apollo/DomainOS or hpux. Is it a coincidence?
+	 */
+	while ( *(c = p) != '\0') {
+	    p++;
 	    if (*c == '`') {
 		gflag |= G_CSH;
+#ifdef notdef
+		/*
+		 * We do want to expand echo `echo '*'`, so we don't\
+		 * use this piece of code anymore.
+		 */
 		while (*p && *p != '`') 
 		    if (*p++ == '\\') {
 			if (*p)		/* Quoted chars */
@@ -659,6 +681,7 @@ tglob(t)
 		    p++;
 		else
 		    break;
+#endif
 	    }
 	    else if (*c == '{')
 		gflag |= G_CSH;
@@ -682,7 +705,7 @@ dobackp(cp, literal)
     bool    literal;
 {
     register Char *lp, *rp;
-    Char   *ep, word[MAXPATHLEN];
+    Char   *ep, word[LONGBSIZE];
 
     if (pargv) {
 #ifdef notdef
@@ -695,12 +718,12 @@ dobackp(cp, literal)
     pargv[0] = NULL;
     pargcp = pargs = word;
     pargc = 0;
-    pnleft = MAXPATHLEN - 4;
+    pnleft = LONGBSIZE - 4;
     for (;;) {
 	for (lp = cp; *lp != '`'; lp++) {
 	    if (*lp == 0) {
 		if (pargcp != pargs)
-		    pword();
+		    pword(LONGBSIZE);
 		return (pargv);
 	    }
 	    psave(*lp);
@@ -747,17 +770,6 @@ backeval(cp, literal)
     fakecom[0] = STRfakecom1;
     fakecom[1] = 0;
 
-    if (didfds == 0) {
-	/*
-	 * Make sure that we have some file descriptors to
-	 * play with, so that the processes have at least 0, 1, 2
-	 * open
-	 */
-	(void) dcopy(SHIN, 0);
-	(void) dcopy(SHOUT, 1);
-	(void) dcopy(SHDIAG, 2);
-    }
-
     /*
      * We do the psave job to temporarily change the current job so that the
      * following fork is considered a separate job.  This is so that when
@@ -774,7 +786,6 @@ backeval(cp, literal)
      */
     mypipe(pvec);
     if (pfork(&faket, -1) == 0) {
-	struct wordent paraml;
 	struct command *t;
 
 	(void) close(pvec[0]);
@@ -869,7 +880,7 @@ backeval(cp, literal)
 	 * would lose blank lines.
 	 */
 	if (c != -1 && (cnt || literal))
-	    pword();
+	    pword(BUFSIZE);
 	hadnl = 0;
     } while (c >= 0);
     (void) close(pvec[0]);
@@ -883,11 +894,12 @@ psave(c)
 {
     if (--pnleft <= 0)
 	stderror(ERR_WTOOLONG);
-    *pargcp++ = c;
+    *pargcp++ = (Char) c;
 }
 
 static void
-pword()
+pword(bufsiz)
+    int    bufsiz;
 {
     psave(0);
     if (pargc == pargsiz - 1) {
@@ -898,7 +910,7 @@ pword()
     pargv[pargc++] = Strsave(pargs);
     pargv[pargc] = NULL;
     pargcp = pargs;
-    pnleft = MAXPATHLEN - 4;
+    pnleft = bufsiz - 4;
 }
 
 int
@@ -985,7 +997,7 @@ pmatch(string, pattern, estr)
 	    oestr = *estr;
 	    pestr = NULL;
 
-	    while (*string) {
+	    do {
 		switch(pmatch(string, pattern, estr)) {
 		case 0:
 		    break;
@@ -997,9 +1009,9 @@ pmatch(string, pattern, estr)
 		default:
 		    abort();	/* Cannot happen */
 		}
-		string++;
 		*estr = string;
 	    }
+	    while (*string++);
 
 	    if (pestr) {
 		*estr = pestr;
@@ -1072,11 +1084,6 @@ int
 sortscmp(a, b)
     register Char **a, **b;
 {
-#if defined(NLS) && !defined(NOSTRCOLL)
-    char    buf[2048];
-
-#endif
-
     if (!a)			/* check for NULL */
 	return (b ? 1 : 0);
     if (!b)
@@ -1087,12 +1094,7 @@ sortscmp(a, b)
     if (!*b)
 	return (-1);
 
-#if defined(NLS) && !defined(NOSTRCOLL)
-    (void) strcpy(buf, short2str(*a));
-    return ((int) strcoll(buf, short2str(*b)));
-#else
-    return ((int) Strcmp(*a, *b));
-#endif
+    return (int) collate(*a, *b);
 }
 
 #endif
