@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.02/RCS/sh.proc.c,v 3.28 1992/05/15 23:49:22 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.04/RCS/sh.proc.c,v 3.50 1993/07/07 20:53:55 christos Exp $ */
 /*
  * sh.proc.c: Job manipulations
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.proc.c,v 3.28 1992/05/15 23:49:22 christos Exp $")
+RCSID("$Id: sh.proc.c,v 3.50 1993/07/07 20:53:55 christos Exp $")
 
 #include "ed.h"
 #include "tc.h"
@@ -47,7 +47,7 @@ RCSID("$Id: sh.proc.c,v 3.28 1992/05/15 23:49:22 christos Exp $")
 # define HZ 16
 #endif /* aiws */
 
-#if (defined(_BSD) && defined(_BSD_INCLUDES)) || (defined(IRIS4D) && __STDC__)
+#if (defined(_BSD) && defined(_BSD_INCLUDES)) || (defined(IRIS4D) && __STDC__) || defined(__lucid) || defined(linux)
 # define BSDWAIT
 #endif
 #ifndef WTERMSIG
@@ -78,11 +78,11 @@ RCSID("$Id: sh.proc.c,v 3.28 1992/05/15 23:49:22 christos Exp $")
 #define BIGINDEX	9	/* largest desirable job index */
 
 #ifdef BSDTIMES
-# if defined(sun) || defined(hp9000)
+# if defined(SUNOS4) || defined(hp9000)
 static struct rusage zru = {{0L, 0L}, {0L, 0L}, 0L, 0L, 0L, 0L,
 			    0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L};
 
-# else /* !sun && !hp9000 */
+# else /* !SUNOS4 && !hp9000 */
 #  ifdef masscomp
 /*
  * Initialization of this structure under RTU 4.1A & RTU 5.0 is problematic
@@ -94,7 +94,7 @@ static struct rusage zru;
 static struct rusage zru = {{0L, 0L}, {0L, 0L}, 0, 0, 0, 0, 0, 0, 0, 
 			    0, 0, 0, 0, 0, 0};
 #  endif /* masscomp */
-# endif	/* !sun && !hp9000 */
+# endif	/* !SUNOS4 && !hp9000 */
 #else /* ! BSDTIMES */
 # ifdef _SEQUENT_
 static struct process_stats zru = {{0L, 0L}, {0L, 0L}, 0, 0, 0, 0, 0, 0, 0,
@@ -118,7 +118,7 @@ static	void		 pads		__P((Char *));
 static	void		 pkill		__P((Char **, int));
 static	struct process	*pgetcurr	__P((struct process *));
 static	void		 okpcntl	__P((void));
-static  struct process  *pfind		__P((Char *));
+static	void		 setttypgrp	__P((int));
 
 /*
  * pchild - called at interrupt level by the SIGCHLD signal
@@ -154,6 +154,7 @@ int snum;
 # else /* !_SEQUENT_ */
     struct tms proctimes;
 
+    USE(snum);
     if (!timesdone) {
 	timesdone++;
 	(void) times(&shtimes);
@@ -190,7 +191,7 @@ loop:
 #ifdef BSDJOBS
 # ifdef BSDTIMES
     /* both a wait3 and rusage */
-#  if !defined(BSDWAIT) || defined(NeXT) || defined(MACH) || (defined(IRIS4D) && __STDC__)
+#  if !defined(BSDWAIT) || defined(NeXT) || defined(MACH) || defined(linux) || (defined(IRIS4D) && (__STDC__ || defined(FUNCPROTO)) && SYSVREL <= 3) || defined(__lucid)
     pid = wait3(&w,
        (setintr && (intty || insource) ? WNOHANG | WUNTRACED : WNOHANG), &ru);
 #  else /* BSDWAIT */
@@ -259,13 +260,9 @@ loop:
 #endif /* BSDJOBS */
 
 #ifdef JOBDEBUG
-    {
-	char    buffer[100];
-	xsprintf(buffer, "pid %d, retval %x termsig %x retcode %x\n",
-		 pid, w, WTERMSIG(w), WEXITSTATUS(w));
-	xprintf(buffer);
-	flush();
-    }
+    xprintf("parent %d pid %d, retval %x termsig %x retcode %x\n",
+	    getpid(), pid, w, WTERMSIG(w), WEXITSTATUS(w));
+    flush();
 #endif /* JOBDEBUG */
 
     if (pid <= 0) {
@@ -277,11 +274,6 @@ loop:
 	    goto loop;
 	}
 	pnoprocesses = pid == -1;
-#ifdef linux
-# ifdef UNRELSIGS
-	(void) sigset(SIGCHLD, pchild);
-# endif /* UNRELSIGS */
-#endif /* linux */
 #ifndef SIGVOID
 	return (0);
 #else /* !SIGVOID */
@@ -299,8 +291,6 @@ loop:
     goto loop;
 #endif /* BSDJOBS */
 found:
-    if (pid == atoi(short2str(value(STRchild))))
-	unsetv(STRchild);
     pp->p_flags &= ~(PRUNNING | PSTOPPED | PREPORTED);
     if (WIFSTOPPED(w)) {
 	pp->p_flags |= PSTOPPED;
@@ -312,7 +302,12 @@ found:
 # ifdef _SEQUENT_
 	    (void) get_process_stats(&pp->p_etime, PS_SELF, NULL, NULL);
 # else	/* !_SEQUENT_ */
+#  ifndef COHERENT
 	    pp->p_etime = times(&proctimes);
+#  else /* !COHERENT */
+	    pp->p_etime = HZ * time(NULL);
+	    times(&proctimes);
+#  endif /* !COHERENT */
 # endif	/* !_SEQUENT_ */
 #else /* BSDTIMES */
 	    (void) gettimeofday(&pp->p_etime, NULL);
@@ -566,7 +561,8 @@ pjwait(pp)
 	if ((jobflags & PRUNNING) == 0)
 	    break;
 #ifdef JOBDEBUG
-	xprintf("starting to sigpause for  SIGCHLD on %d\n", fp->p_procid);
+	xprintf("%d starting to sigpause for  SIGCHLD on %d\n",
+		getpid(), fp->p_procid);
 #endif /* JOBDEBUG */
 #ifdef BSDSIGS
 	/* sigpause(sigblock((sigmask_t) 0) &~ sigmask(SIGCHLD)); */
@@ -632,7 +628,7 @@ pjwait(pp)
     if ((reason != 0) && (adrof(STRprintexitvalue)) && 
 	(pp->p_flags & PBACKQ) == 0)
 	xprintf("Exit %d\n", reason);
-    set(STRstatus, putn(reason));
+    set(STRstatus, putn(reason), VAR_READWRITE);
     if (reason && exiterr)
 	exitstat();
     pflush(pp);
@@ -653,6 +649,8 @@ dowait(v, c)
     sigmask_t omask;
 #endif /* BSDSIGS */
 
+    USE(c);
+    USE(v);
     pjobs++;
 #ifdef BSDSIGS
     omask = sigblock(sigmask(SIGCHLD));
@@ -834,7 +832,12 @@ palloc(pid, t)
     {
 	struct tms tmptimes;
 
+#  ifndef COHERENT
 	pp->p_btime = times(&tmptimes);
+#  else /* !COHERENT */
+	pp->p_btime = HZ * time(NULL);
+	times(&tmptimes);
+#  endif /* !COHERENT */
     }
 # endif /* !_SEQUENT_ */
 #endif /* !BSDTIMES */
@@ -917,7 +920,7 @@ pads(cp)
     if (cp[0] == STRQNULL[0])
 	cp++;
 
-    i = Strlen(cp);
+    i = (int) Strlen(cp);
 
     if (cmdlen >= PMAXLEN)
 	return;
@@ -1088,10 +1091,10 @@ pprint(pp, flag)
 			goto prcomd;
 		    }
 		    else
-			reason = pp->p_reason;
+			reason = (int) pp->p_reason;
 		else {
 		    status = pstatus;
-		    reason = pp->p_reason;
+		    reason = (int) pp->p_reason;
 		}
 		switch (status) {
 
@@ -1137,7 +1140,7 @@ pprint(pp, flag)
 	}
 prcomd:
 	if (flag & NAME) {
-	    xprintf("%s", short2str(pp->p_command));
+	    xprintf("%S", pp->p_command);
 	    if (pp->p_flags & PPOU)
 		xprintf(" |");
 	    if (pp->p_flags & PDIAG)
@@ -1151,7 +1154,7 @@ prcomd:
 	    if (flag & JOBDIR &&
 		!eq(tp->p_cwd->di_name, dcwd->di_name)) {
 		xprintf(" (wd: ");
-		dtildepr(value(STRhome), tp->p_cwd->di_name);
+		dtildepr(tp->p_cwd->di_name);
 		xprintf(")");
 	    }
 	}
@@ -1182,7 +1185,7 @@ prcomd:
 		xputchar('\n');
 	    if (flag & SHELLDIR && !eq(tp->p_cwd->di_name, dcwd->di_name)) {
 		xprintf("(wd now: ");
-		dtildepr(value(STRhome), dcwd->di_name);
+		dtildepr(dcwd->di_name);
 		xprintf(")\n");
 	    }
 	}
@@ -1221,6 +1224,17 @@ prcomd:
     return (jobflags);
 }
 
+/*
+ * All 4.3 BSD derived implementations are buggy and I've had enough.
+ * The following implementation produces similar code and works in all
+ * cases. The 4.3BSD one works only for <, >, !=
+ */
+# undef timercmp
+#  define timercmp(tvp, uvp, cmp) \
+      (((tvp)->tv_sec == (uvp)->tv_sec) ? \
+	   ((tvp)->tv_usec cmp (uvp)->tv_usec) : \
+	   ((tvp)->tv_sec  cmp (uvp)->tv_sec))
+
 static void
 ptprint(tp)
     register struct process *tp;
@@ -1229,7 +1243,6 @@ ptprint(tp)
     struct timeval tetime, diff;
     static struct timeval ztime;
     struct rusage ru;
-    static struct rusage zru;
     register struct process *pp = tp;
 
     ru = zru;
@@ -1243,13 +1256,9 @@ ptprint(tp)
     prusage(&zru, &ru, &tetime, &ztime);
 #else /* !BSDTIMES */
 # ifdef _SEQUENT_
-#  define timercmp(tvp, uvp, cmp) \
-      ((tvp)->tv_sec cmp (uvp)->tv_sec || \
-       (tvp)->tv_sec == (uvp)->tv_sec && (tvp)->tv_usec cmp (uvp)->tv_usec)
     timeval_t tetime, diff;
     static timeval_t ztime;
     struct process_stats ru;
-    static struct process_stats zru;
     register struct process *pp = tp;
 
     ru = zru;
@@ -1316,6 +1325,7 @@ dojobs(v, c)
     register int flag = NUMBER | NAME | REASON;
     int     i;
 
+    USE(c);
     if (chkstop)
 	chkstop = 2;
     if (*++v) {
@@ -1344,6 +1354,7 @@ dofg(v, c)
 {
     register struct process *pp;
 
+    USE(c);
     okpcntl();
     ++v;
     do {
@@ -1370,6 +1381,7 @@ dofg1(v, c)
 {
     register struct process *pp;
 
+    USE(c);
     okpcntl();
     pp = pfind(v[0]);
     pstart(pp, 1);
@@ -1393,6 +1405,7 @@ dobg(v, c)
 {
     register struct process *pp;
 
+    USE(c);
     okpcntl();
     ++v;
     do {
@@ -1412,6 +1425,7 @@ dobg1(v, c)
 {
     register struct process *pp;
 
+    USE(c);
     pp = pfind(v[0]);
     pstart(pp, 0);
 }
@@ -1425,6 +1439,7 @@ dostop(v, c)
     Char  **v;
     struct command *c;
 {
+    USE(c);
 #ifdef BSDJOBS
     pkill(++v, SIGSTOP);
 #endif /* BSDJOBS */
@@ -1442,11 +1457,13 @@ dokill(v, c)
     register int signum, len = 0;
     register char *name;
     extern int T_Cols;
+    extern int nsig;
 
+    USE(c);
     v++;
     if (v[0] && v[0][0] == '-') {
 	if (v[0][1] == 'l') {
-	    for (signum = 1; signum <= NSIG; signum++) {
+	    for (signum = 1; signum <= nsig; signum++) {
 		if ((name = mesg[signum].iname) != NULL) {
 		    len += strlen(name) + 1;
 		    if (len >= T_Cols - 1) {
@@ -1465,7 +1482,7 @@ dokill(v, c)
 		stderror(ERR_NAME | ERR_BADSIG);
 	}
 	else {
-	    for (signum = 1; signum <= NSIG; signum++)
+	    for (signum = 1; signum <= nsig; signum++)
 		if (mesg[signum].iname &&
 		    eq(&v[0][1], str2short(mesg[signum].iname)))
 		    goto gotsig;
@@ -1529,11 +1546,9 @@ pkill(v, signum)
 	    case SIGTTOU:
 		if ((jobflags & PRUNNING) == 0) {
 # ifdef SUSPENDED
-		    xprintf("%s: Already suspended\n",
-			    short2str(cp));
+		    xprintf("%S: Already suspended\n", cp);
 # else /* !SUSPENDED */
-		    xprintf("%s: Already stopped\n",
-			    short2str(cp));
+		    xprintf("%S: Already stopped\n", cp);
 # endif /* !SUSPENDED */
 		    err1++;
 		    goto cont;
@@ -1551,7 +1566,7 @@ pkill(v, signum)
 	    }
 #endif /* BSDJOBS */
 	    if (killpg(pp->p_jobid, signum) < 0) {
-		xprintf("%s: %s\n", short2str(cp), strerror(errno));
+		xprintf("%S: %s\n", cp, strerror(errno));
 		err1++;
 	    }
 #ifdef BSDJOBS
@@ -1601,7 +1616,8 @@ pstart(pp, foregnd)
 #ifdef BSDSIGS
     sigmask_t omask;
 #endif /* BSDSIGS */
-    long    jobflags = 0;
+    /* We don't use jobflags in this function right now (see below) */
+    /* long    jobflags = 0; */
 
 #ifdef BSDSIGS
     omask = sigblock(sigmask(SIGCHLD));
@@ -1610,7 +1626,8 @@ pstart(pp, foregnd)
 #endif
     np = pp;
     do {
-	jobflags |= np->p_flags;
+	/* We don't use jobflags in this function right now (see below) */
+	/* jobflags |= np->p_flags; */
 	if (np->p_flags & (PRUNNING | PSTOPPED)) {
 	    np->p_flags |= PRUNNING;
 	    np->p_flags &= ~PSTOPPED;
@@ -1624,9 +1641,24 @@ pstart(pp, foregnd)
 	pclrcurr(pp);
     (void) pprint(pp, foregnd ? NAME | JOBDIR : NUMBER | NAME | AMPERSAND);
 #ifdef BSDJOBS
-    if (foregnd)
+    if (foregnd) {
 	(void) tcsetpgrp(FSHTTY, pp->p_jobid);
-    if (jobflags & PSTOPPED)
+    }
+    /*
+     * 1. child process of csh (shell script) receives SIGTTIN/SIGTTOU
+     * 2. parent process (csh) receives SIGCHLD
+     * 3. The "csh" signal handling function pchild() is invoked
+     *    with a SIGCHLD signal.
+     * 4. pchild() calls wait3(WNOHANG) which returns 0.
+     *    The child process is NOT ready to be waited for at this time.
+     *    pchild() returns without picking-up the correct status
+     *    for the child process which generated the SIGCHILD.
+     * 5. CONSEQUENCE : csh is UNaware that the process is stopped
+     * 6. THIS LINE HAS BEEN COMMENTED OUT : if (jobflags&PSTOPPED)
+     * 	  (beto@aixwiz.austin.ibm.com - aug/03/91)
+     */
+
+    /* if (jobflags & PSTOPPED) */
 	(void) killpg(pp->p_jobid, SIGCONT);
 #endif /* BSDJOBS */
 #ifdef BSDSIGS
@@ -1648,7 +1680,7 @@ panystop(neednl)
 	    stderror(ERR_STOPPED, neednl ? "\n" : "");
 }
 
-static struct process *
+struct process *
 pfind(cp)
     Char   *cp;
 {
@@ -1732,6 +1764,7 @@ donotify(v, c)
 {
     register struct process *pp;
 
+    USE(c);
     pp = pfind(*++v);
     pp->p_flags |= PNOTIFY;
 }
@@ -1757,7 +1790,7 @@ pfork(t, wanttty)
     bool    ignint = 0;
     int     pgrp;
 #ifdef BSDSIGS
-    sigmask_t omask;
+    sigmask_t omask = 0;
 #endif /* BSDSIGS */
 #ifdef SIGSYNCH
     sigvec_t osv;
@@ -1775,32 +1808,40 @@ pfork(t, wanttty)
     if (setintr)
 	ignint = (tpgrp == -1 && (t->t_dflg & F_NOINTERRUPT))
 	    || (gointr && eq(gointr, STRminus));
+
+#ifdef COHERENT
+    ignint |= gointr && eq(gointr, STRminus);
+#endif /* COHERENT */
+
     /*
      * Check for maximum nesting of 16 processes to avoid Forking loops
      */
     if (child == 16)
 	stderror(ERR_NESTING, 16);
-    /*
-     * Hold SIGCHLD until we have the process installed in our table.
-     */
 #ifdef SIGSYNCH
     if (mysigvec(SIGSYNCH, &nsv, &osv))
 	stderror(ERR_SYSTEM, "pfork: sigvec set", strerror(errno));
 #endif /* SIGSYNCH */
+    /*
+     * Hold SIGCHLD until we have the process installed in our table.
+     */
+    if (wanttty < 0) {
 #ifdef BSDSIGS
-    omask = sigblock(sigmask(SIGCHLD));
+	omask = sigblock(sigmask(SIGCHLD));
 #else /* !BSDSIGS */
-    (void) sighold(SIGCHLD);
+	(void) sighold(SIGCHLD);
 #endif /* !BSDSIGS */
+    }
     while ((pid = fork()) < 0)
 	if (setintr == 0)
 	    (void) sleep(FORKSLEEP);
 	else {
+	    if (wanttty < 0)
 #ifdef BSDSIGS
-	    (void) sigsetmask(omask);
+		(void) sigsetmask(omask);
 #else /* !BSDSIGS */
+		(void) sigrelse(SIGCHLD);
 	    (void) sigrelse(SIGINT);
-	    (void) sigrelse(SIGCHLD);
 #endif /* !BSDSIGS */
 	    stderror(ERR_NOPROC);
 	}
@@ -1816,7 +1857,8 @@ pfork(t, wanttty)
 	if (setintr) {
 	    setintr = 0;	/* until I think otherwise */
 #ifndef BSDSIGS
-	    (void) sigrelse(SIGCHLD);
+	    if (wanttty < 0)
+		(void) sigrelse(SIGCHLD);
 #endif /* !BSDSIGS */
 	    /*
 	     * Children just get blown away on SIGINT, SIGQUIT unless "onintr
@@ -1858,7 +1900,7 @@ pfork(t, wanttty)
 #endif /* !BSDNICE */
 #ifdef F_VER
         if (t->t_dflg & F_VER) {
-	    Setenv(STRSYSTYPE, t->t_systype ? STRbsd43 : STRsys53);
+	    tsetenv(STRSYSTYPE, t->t_systype ? STRbsd43 : STRsys53);
 	    dohash(NULL, NULL);
 	}
 #endif /* F_VER */
@@ -1871,8 +1913,25 @@ pfork(t, wanttty)
     }
     else {
 #ifdef POSIXJOBS
-	if (wanttty >= 0)
-	    (void) setpgid(pid, pcurrjob ? pcurrjob->p_jobid : pid);
+        if (wanttty >= 0) {
+	    /*
+	     * `Walking' process group fix from Beto Appleton.
+	     * (beto@aixwiz.austin.ibm.com)
+	     * If setpgid fails at this point that means that
+	     * our process leader has died. We flush the current
+	     * job and become the process leader ourselves.
+	     * The parent will figure that out later.
+	     */
+	    pgrp = pcurrjob ? pcurrjob->p_jobid : pid;
+	    if (setpgid(pid, pgrp) == -1 && errno == EPERM) {
+		pcurrjob = NULL;
+		/* 
+		 * We don't care if this causes an error here;
+		 * then we are already in the right process group
+		 */
+		(void) setpgid(pid, pgrp = pid);
+	    }
+	}
 #endif /* POSIXJOBS */
 	palloc(pid, t);
 #ifdef SIGSYNCH
@@ -1890,11 +1949,13 @@ pfork(t, wanttty)
 		     strerror(errno));
 #endif /* SIGSYNCH */
 
+	if (wanttty < 0) {
 #ifdef BSDSIGS
-	(void) sigsetmask(omask);
+	    (void) sigsetmask(omask);
 #else /* !BSDSIGS */
-	(void) sigrelse(SIGCHLD);
+	    (void) sigrelse(SIGCHLD);
 #endif /* !BSDSIGS */
+	}
     }
     return (pid);
 }
@@ -1908,6 +1969,43 @@ okpcntl()
 	stderror(ERR_JOBCTRLSUB);
 }
 
+
+static void
+setttypgrp(pgrp)
+    int pgrp;
+{
+#ifdef BSDJOBS
+    /*
+     * If we are piping out a builtin, eg. 'echo | more' things can go
+     * out of sequence, i.e. the more can run before the echo. This
+     * can happen even if we have vfork, since the echo will be forked
+     * with the regular fork. In this case, we need to set the tty
+     * pgrp ourselves. If that happens, then the process will be still
+     * alive. And the tty process group will already be set.
+     * This should fix the famous sequent problem as a side effect:
+     *    The controlling terminal is lost if all processes in the
+     *    terminal process group are zombies. In this case tcgetpgrp()
+     *    returns 0. If this happens we must set the terminal process
+     *    group again.
+     */
+    if (tcgetpgrp(FSHTTY) != pgrp) {
+# ifdef POSIXJOBS
+        /*
+	 * tcsetpgrp will set SIGTTOU to all the the processes in 
+	 * the background according to POSIX... We ignore this here.
+	 */
+	sigret_t (*old)() = sigset(SIGTTOU, SIG_IGN);
+# endif /* POSIXJOBS */
+	(void) tcsetpgrp(FSHTTY, pgrp);
+# ifdef POSIXJOBS
+	(void) sigset(SIGTTOU, old);
+# endif /* POSIXJOBS */
+
+    }
+#endif /* BSDJOBS */
+}
+
+
 /*
  * if we don't have vfork(), things can still go in the wrong order
  * resulting in the famous 'Stopped (tty output)'. But some systems
@@ -1926,9 +2024,9 @@ pgetty(wanttty, pgrp)
 # endif /* BSDSIGS && POSIXJOBS */
 
 # ifdef JOBDEBUG
-    xprintf("wanttty %d\n", wanttty);
-# endif
-
+    xprintf("wanttty %d pid %d opgrp%d pgrp %d tpgrp %d\n", 
+	    wanttty, getpid(), pgrp, mygetpgrp(), tcgetpgrp(FSHTTY));
+# endif /* JOBDEBUG */
 # ifdef POSIXJOBS
     /*
      * christos: I am blocking the tty signals till I've set things
@@ -1936,19 +2034,18 @@ pgetty(wanttty, pgrp)
      */
     if (wanttty > 0)
 #  ifdef BSDSIGS
-	omask = sigblock(sigmask(SIGTSTP)|sigmask(SIGTTIN)|sigmask(SIGTTOU));
+	omask = sigblock(sigmask(SIGTSTP)|sigmask(SIGTTIN));
 #  else /* !BSDSIGS */
     {
 	(void) sighold(SIGTSTP);
 	(void) sighold(SIGTTIN);
-	(void) sighold(SIGTTOU);
     }
 #  endif /* !BSDSIGS */
 # endif /* POSIXJOBS */
 
 # ifndef POSIXJOBS
     if (wanttty > 0)
-	(void) tcsetpgrp(FSHTTY, pgrp);
+	setttypgrp(pgrp);
 # endif /* !POSIXJOBS */
 
     /*
@@ -1957,41 +2054,39 @@ pgetty(wanttty, pgrp)
      * background jobs process groups Same for the comparison in the other part
      * of the #ifdef
      */
-    if (wanttty >= 0)
+    if (wanttty >= 0) {
 	if (setpgid(0, pgrp) == -1) {
-# ifdef BACKPIPE
-	    /*
-     	     * This usually happens in svr4 when the last command in a pipe
-	     * either couldn't be started, or exits without waiting for input.
-	     * Putting in the xexit() hangs the shell, so leave it out.
-	     * (DHD)
-     	     */
-#  ifdef JOBDEBUG
-	    xprintf("tcsh: setpgid error (%s).\n", strerror(errno));
-	    xprintf("pgrp = %d, shell pid = %d\n",pgrp,getpid());
-#  endif /* JOBDEBUG */
-# else /* !BACKPIPE */
-#  if !defined(ISC) && !defined(SCO) && !defined(cray)
-	    /* XXX: Wrong but why? */
-	    xprintf("tcsh: setpgid error (%s).\n", strerror(errno));
-#  endif /* !ISC && !SCO && !cray */
-	    xexit(0);
-# endif /* BACKPIPE */
+# ifdef POSIXJOBS
+	    /* Walking process group fix; see above */
+	    if (setpgid(0, pgrp = getpid()) == -1) {
+# endif /* POSIXJOBS */
+		stderror(ERR_SYSTEM, "setpgid child:\n", strerror(errno));
+		xexit(0);
+# ifdef POSIXJOBS
+	    }
+	    wanttty = pgrp;  /* Now we really want the tty, since we became the
+			      * the process group leader
+			      */
+# endif /* POSIXJOBS */
 	}
+    }
 
 # ifdef POSIXJOBS
-    if (wanttty > 0) {
-	(void) tcsetpgrp(FSHTTY, pgrp);
+    if (wanttty > 0)
+	setttypgrp(pgrp);
 #  ifdef BSDSIGS
-	(void) sigsetmask(omask);
+    (void) sigsetmask(omask);
 #  else /* BSDSIGS */
-	(void) sigrelse(SIGTSTP);
-	(void) sigrelse(SIGTTIN);
-	(void) sigrelse(SIGTTOU);
+    (void) sigrelse(SIGTSTP);
+    (void) sigrelse(SIGTTIN);
 #  endif /* !BSDSIGS */
-    }
 # endif /* POSIXJOBS */
 
+# ifdef JOBDEBUG
+    xprintf("wanttty %d pid %d pgrp %d tpgrp %d\n", 
+	    wanttty, getpid(), mygetpgrp(), tcgetpgrp(FSHTTY));
+# endif /* JOBDEBUG */
+
     if (tpgrp > 0)
 	tpgrp = 0;		/* gave tty away */
 #endif /* BSDJOBS */
