@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.02/RCS/sh.sem.c,v 3.18 1992/03/08 02:17:22 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.04/RCS/sh.sem.c,v 3.32 1993/07/03 23:47:53 christos Exp $ */
 /*
  * sh.sem.c: I/O redirections and job forking. A touchy issue!
  *	     Most stuff with builtins is incorrect
@@ -37,23 +37,23 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.sem.c,v 3.18 1992/03/08 02:17:22 christos Exp $")
+RCSID("$Id: sh.sem.c,v 3.32 1993/07/03 23:47:53 christos Exp $")
 
 #include "tc.h"
 
-#ifdef FIOCLEX
+#ifdef CLOSE_ON_EXEC
 # ifndef SUNOS4
 #  ifndef CLEX_DUPS
 #   define CLEX_DUPS
 #  endif /* CLEX_DUPS */
 # endif /* !SUNOS4 */
-#endif /* FIOCLEX */
+#endif /* CLOSE_ON_EXEC */
 
-#ifdef sparc
-# ifndef MACH
+#if defined(__sparc__) || defined(sparc)
+# if !defined(MACH) && SYSVREL == 0 && !defined(Lynx)
 #  include <vfork.h>
-# endif /* !MACH */
-#endif /* sparc */
+# endif /* !MACH && SYSVREL == 0 */
+#endif /* __sparc__ || sparc */
 
 #ifdef VFORK
 static	sigret_t	vffree	__P((int));
@@ -95,6 +95,7 @@ static struct {
 			_gv.forked = forked, \
 			_gv.wanttty = owanttty )
 
+
 /*VARARGS 1*/
 void
 execute(t, wanttty, pipein, pipeout)
@@ -102,9 +103,9 @@ execute(t, wanttty, pipein, pipeout)
     int     wanttty;
     int *pipein, *pipeout;
 {
-#if defined(convex) || defined(__convex__)
+#ifdef VFORK
     extern bool use_fork;	/* use fork() instead of vfork()? */
-#endif 
+#endif
 
     bool    forked;
     struct biltins *bifunc;
@@ -112,9 +113,6 @@ execute(t, wanttty, pipein, pipeout)
     int     pv[2];
 #ifdef BSDSIGS
     static sigmask_t csigmask;
-# ifdef VFORK
-    static sigmask_t ocsigmask;
-# endif /* VFORK */
 #endif /* BSDSIGS */
 #ifdef VFORK
     static int onosigchld = 0;
@@ -167,7 +165,7 @@ execute(t, wanttty, pipein, pipeout)
 		(void) close(0);
 	}
 
-	set(STRstatus, Strsave(STR0));
+	set(STRstatus, Strsave(STR0), VAR_READWRITE);
 
 	/*
 	 * This mess is the necessary kludge to handle the prefix builtins:
@@ -229,7 +227,7 @@ execute(t, wanttty, pipein, pipeout)
 	     * Check if we have a builtin function and remember which one.
 	     */
 	    _gv.bifunc = isbfunc(t);
- 	    if (noexec) {
+ 	    if (noexec && _gv.bifunc) {
 		/*
 		 * Continue for builtins that are part of the scripting language
 		 */
@@ -299,7 +297,7 @@ execute(t, wanttty, pipein, pipeout)
 #ifdef BSDSIGS
 		    csigmask = sigblock(sigmask(SIGCHLD));
 #else /* !BSDSIGS */
-		    sighold(SIGCHLD);
+		    (void) sighold(SIGCHLD);
 #endif /* BSDSIGS */
 
 		    nosigchld = 1;
@@ -324,12 +322,11 @@ execute(t, wanttty, pipein, pipeout)
 		int     oSHIN, oSHOUT, oSHDIAG, oOLDSTD, otpgrp;
 		int     oisoutatty, oisdiagatty;
 
-# ifndef FIOCLEX
+# ifndef CLOSE_ON_EXEC
 		int     odidcch;
-
-# endif  /* !FIOCLEX */
+# endif  /* !CLOSE_ON_EXEC */
 # ifdef BSDSIGS
-		sigmask_t omask;
+		sigmask_t omask, ocsigmask;
 # endif /* BSDSIGS */
 
 		/*
@@ -340,33 +337,37 @@ execute(t, wanttty, pipein, pipeout)
 		 * the current sigvec's for the signals the child touches
 		 * before it exec's.
 		 */
-# ifdef BSDSIGS
 
 		/*
 		 * Sooooo true... If this is a Sun, save the sigvec's. (Skip
 		 * Gilbrech - 11/22/87)
 		 */
-#  ifdef SAVESIGVEC
+# ifdef SAVESIGVEC
 		sigvec_t savesv[NSIGSAVED];
 		sigmask_t savesm;
 
-#  endif /* SAVESIGVEC */
+# endif /* SAVESIGVEC */
 		if (_gv.wanttty >= 0 && !nosigchld && !noexec) {
+# ifdef BSDSIGS
 		    csigmask = sigblock(sigmask(SIGCHLD));
+# else /* !BSDSIGS */
+		    (void) sighold(SIGCHLD);
+# endif  /* BSDSIGS */
 		    nosigchld = 1;
 		}
-		omask = sigblock(sigmask(SIGCHLD) | sigmask(SIGINT));
+# ifdef BSDSIGS
+		omask = sigblock(sigmask(SIGCHLD)|sigmask(SIGINT));
 # else /* !BSDSIGS */
 		(void) sighold(SIGCHLD);
 		(void) sighold(SIGINT);
-# endif  /* !BSDSIGS */
+# endif  /* BSDSIGS */
 		ochild = child;
 		osetintr = setintr;
 		ohaderr = haderr;
 		odidfds = didfds;
-# ifndef FIOCLEX
+# ifndef CLOSE_ON_EXEC
 		odidcch = didcch;
-# endif /* !FIOCLEX */
+# endif /* !CLOSE_ON_EXEC */
 		oSHIN = SHIN;
 		oSHOUT = SHOUT;
 		oSHDIAG = SHDIAG;
@@ -384,14 +385,10 @@ execute(t, wanttty, pipein, pipeout)
 # ifdef SAVESIGVEC
 		savesm = savesigvec(savesv);
 # endif /* SAVESIGVEC */
-# if defined(convex) || defined(__convex__)
 		if (use_fork)
 		    pid = fork();
 		else
 		    pid = vfork();
-# else /* !convex && !__convex__ */
-		pid = vfork();
-# endif /* convex || __CONVEX__ */
 
 		if (pid < 0) {
 # ifdef BSDSIGS
@@ -402,24 +399,22 @@ execute(t, wanttty, pipein, pipeout)
 # else /* !BSDSIGS */
 		    (void) sigrelse(SIGCHLD);
 		    (void) sigrelse(SIGINT);
-#endif  /* BSDSIGS */
+# endif  /* BSDSIGS */
 		    stderror(ERR_NOPROC);
 		}
 		_gv.forked++;
 		if (pid) {	/* parent */
-# ifdef BSDSIGS
-#  ifdef SAVESIGVEC
+# ifdef SAVESIGVEC
 		    restoresigvec(savesv, savesm);
-#  endif /* SAVESIGVEC */
-# endif /* BSDSIGS */
+# endif /* SAVESIGVEC */
 		    child = ochild;
 		    setintr = osetintr;
 		    haderr = ohaderr;
 		    didfds = odidfds;
 		    SHIN = oSHIN;
-# ifndef FIOCLEX
+# ifndef CLOSE_ON_EXEC
 		    didcch = odidcch;
-# endif /* !FIOCLEX */
+# endif /* !CLOSE_ON_EXEC */
 		    SHOUT = oSHOUT;
 		    SHDIAG = oSHDIAG;
 		    OLDSTD = oOLDSTD;
@@ -452,7 +447,6 @@ execute(t, wanttty, pipein, pipeout)
 		    /* this is from pfork() */
 		    int     pgrp;
 		    bool    ignint = 0;
-
 		    if (nosigchld) {
 # ifdef BSDSIGS
 			(void) sigsetmask(csigmask);
@@ -469,9 +463,6 @@ execute(t, wanttty, pipein, pipeout)
 		    child++;
 		    if (setintr) {
 			setintr = 0;
-# ifdef notdef
-			(void) signal(SIGCHLD, SIG_DFL);
-# endif 
 /*
  * casts made right for SunOS 4.0 by Douglas C. Schmidt
  * <schmidt%sunshine.ics.uci.edu@ROME.ICS.UCI.EDU>
@@ -504,31 +495,19 @@ execute(t, wanttty, pipein, pipeout)
 			(void) signal(SIGQUIT, SIG_IGN);
 		    }
 
-# ifdef _SEQUENT_
-		    /*
-		     * On some machines (POSIX) the process group leader
-		     * cannot be a zombie. On those machines, the following
-		     * might help. Note that BACKPIPE will break if the
-		     * last process exits too soon.
-		     * (From Jaap)
-		     */
-		    pgetty(_gv.wanttty ? _gv.wanttty : 1, pgrp);
-# else /* _SEQUENT_ */
 		    pgetty(_gv.wanttty, pgrp);
-# endif /* _SEQUENT_ */
 
 		    if (t->t_dflg & F_NOHUP)
 			(void) signal(SIGHUP, SIG_IGN);
 		    if (t->t_dflg & F_NICE)
 # ifdef BSDNICE
-			(void) setpriority(PRIO_PROCESS,
-					   0, t->t_nice);
+			(void) setpriority(PRIO_PROCESS, 0, t->t_nice);
 # else /* !BSDNICE */
 			(void) nice(t->t_nice);
 # endif /* BSDNICE */
 # ifdef F_VER
 		    if (t->t_dflg & F_VER) {
-			Setenv(STRSYSTYPE, t->t_systype ? STRbsd43 : STRsys53);
+			tsetenv(STRSYSTYPE, t->t_systype ? STRbsd43 : STRsys53);
 			dohash(NULL, NULL);
 		    }
 # endif /* F_VER */
@@ -608,9 +587,9 @@ execute(t, wanttty, pipein, pipeout)
 	isdiagatty = isatty(SHDIAG);
 	(void) close(SHIN);
 	SHIN = -1;
-#ifndef FIOCLEX
+#ifndef CLOSE_ON_EXEC
 	didcch = 0;
-#endif /* !FIOCLEX */
+#endif /* !CLOSE_ON_EXEC */
 	didfds = 0;
 	_gv.wanttty = -1;
 	t->t_dspr->t_dflg |= t->t_dflg & F_NOINTERRUPT;
@@ -624,8 +603,6 @@ execute(t, wanttty, pipein, pipeout)
 	execute(t->t_dcdr, _gv.wanttty, pv, pipeout);
 	t->t_dcar->t_dflg |= F_PIPEOUT |
 	    (t->t_dflg & (F_PIPEIN | F_AMPERSAND | F_STDERR | F_NOINTERRUPT));
-	if (_gv.wanttty > 0)
-	    _gv.wanttty = 0;	/* got tty already */
 	execute(t->t_dcar, _gv.wanttty, pipein, pv);
 #else /* !BACKPIPE */
 	t->t_dcar->t_dflg |= F_PIPEOUT |
@@ -633,8 +610,6 @@ execute(t, wanttty, pipein, pipeout)
 	execute(t->t_dcar, _gv.wanttty, pipein, pv);
 	t->t_dcdr->t_dflg |= F_PIPEIN | (t->t_dflg &
 			(F_PIPEOUT | F_AMPERSAND | F_NOFORK | F_NOINTERRUPT));
-	if (_gv.wanttty > 0)
-	    _gv.wanttty = 0;	/* got tty already */
 	execute(t->t_dcdr, _gv.wanttty, pv, pipeout);
 #endif /* BACKPIPE */
 	break;
@@ -698,11 +673,16 @@ int snum;
 {
     register Char **v;
 
-    if ((v = gargv) != 0)
-	gargv = 0, xfree((ptr_t) v);
+    USE(snum);
+    if ((v = gargv) != 0) {
+	gargv = 0;
+	xfree((ptr_t) v);
+    }
 
-    if ((v = pargv) != 0)
-	pargv = 0, xfree((ptr_t) v);
+    if ((v = pargv) != 0) {
+	pargv = 0;
+	xfree((ptr_t) v);
+    }
 
     _exit(1);
 #ifndef SIGVOID
@@ -815,17 +795,15 @@ doio(t, pipein, pipeout)
 	else {
 	    (void) close(0);
 	    (void) dup(OLDSTD);
-#ifdef FIONCLEX
-# ifdef CLEX_DUPS
+#if defined(CLOSE_ON_EXEC) && defined(CLEX_DUPS)
 	    /*
 	     * PWP: Unlike Bezerkeley 4.3, FIONCLEX for Pyramid is preserved
 	     * across dup()s, so we have to UNSET it here or else we get a
 	     * command with NO stdin, stdout, or stderr at all (a bad thing
 	     * indeed)
 	     */
-	    (void) ioctl(0, FIONCLEX, NULL);
-# endif /* CLEX_DUPS */
-#endif /* FIONCLEX */
+	    (void) close_on_exec(0, 0);
+#endif /* CLOSE_ON_EXEC && CLEX_DUPS */
 	}
     }
     if (t->t_drit) {
@@ -871,11 +849,9 @@ doio(t, pipein, pipeout)
 	(void) close(1);
 	(void) dup(SHOUT);
 	is1atty = isoutatty;
-#ifdef FIONCLEX
-# ifdef CLEX_DUPS
-	(void) ioctl(1, FIONCLEX, NULL);
-# endif /* CLEX_DUPS */
-#endif /* FIONCLEX */
+# if defined(CLOSE_ON_EXEC) && defined(CLEX_DUPS)
+	(void) close_on_exec(1, 0);
+# endif /* CLOSE_ON_EXEC && CLEX_DUPS */
     }
 
     (void) close(2);
@@ -886,11 +862,9 @@ doio(t, pipein, pipeout)
     else {
 	(void) dup(SHDIAG);
 	is2atty = isdiagatty;
-#ifdef FIONCLEX
-# ifdef CLEX_DUPS
-	(void) ioctl(2, FIONCLEX, NULL);
-# endif /* CLEX_DUPS */
-#endif /* FIONCLEX */
+# if defined(CLOSE_ON_EXEC) && defined(CLEX_DUPS)
+	(void) close_on_exec(2, 0);
+# endif /* CLOSE_ON_EXEC && CLEX_DUPS */
     }
     didfds = 1;
 }
