@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.02/RCS/sh.set.c,v 3.15 1992/05/09 04:03:53 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.04/RCS/sh.set.c,v 3.23 1993/07/03 23:47:53 christos Exp $ */
 /*
  * sh.set.c: Setting and Clearing of variables
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.set.c,v 3.15 1992/05/09 04:03:53 christos Exp $")
+RCSID("$Id: sh.set.c,v 3.23 1993/07/03 23:47:53 christos Exp $")
 
 #include "ed.h"
 #include "tw.h"
@@ -44,6 +44,7 @@ RCSID("$Id: sh.set.c,v 3.15 1992/05/09 04:03:53 christos Exp $")
 extern Char HistLit;
 extern bool GotTermCaps;
 
+static	void		 update_vars	__P((Char *));
 static	Char		*getinx		__P((Char *, int *));
 static	void		 asx		__P((Char *, int, Char *));
 static	struct varent 	*getvx		__P((Char *, int));
@@ -59,6 +60,97 @@ static	void		 balance	__P((struct varent *, int, int));
  * C Shell
  */
 
+static void
+update_vars(vp)
+    Char *vp;
+{
+    if (eq(vp, STRpath)) {
+	exportpath(adrof(STRpath)->vec);
+	dohash(NULL, NULL);
+    }
+    else if (eq(vp, STRhistchars)) {
+	register Char *pn = value(vp);
+
+	HIST = *pn++;
+	HISTSUB = *pn;
+    }
+    else if (eq(vp, STRhistlit)) {
+	HistLit = 1;
+    }
+    else if (eq(vp, STRuser)) {
+	tsetenv(STRKUSER, value(vp));
+	tsetenv(STRLOGNAME, value(vp));
+    }
+    else if (eq(vp, STRwordchars)) {
+	word_chars = value(vp);
+    }
+    else if (eq(vp, STRsymlinks)) {
+	register Char *pn = value(vp);
+
+	if (eq(pn, STRignore))
+	    symlinks = SYM_IGNORE;
+	else if (eq(pn, STRexpand))
+	    symlinks = SYM_EXPAND;
+	else if (eq(pn, STRchase))
+	    symlinks = SYM_CHASE;
+	else
+	    symlinks = 0;
+    }
+    else if (eq(vp, STRterm)) {
+#ifdef DOESNT_WORK_RIGHT
+	register Char *cp;
+#endif /* DOESNT_WORK_RIGHT */
+	tsetenv(STRKTERM, value(vp));
+#ifdef DOESNT_WORK_RIGHT
+	cp = getenv("TERMCAP");
+	if (cp && (*cp != '/'))	/* if TERMCAP and not a path */
+	    Unsetenv(STRTERMCAP);
+#endif /* DOESNT_WORK_RIGHT */
+	GotTermCaps = 0;
+	ed_Init();		/* reset the editor */
+    }
+    else if (eq(vp, STRhome)) {
+	register Char *cp;
+
+	cp = Strsave(value(vp));	/* get the old value back */
+
+	/*
+	 * convert to cononical pathname (possibly resolving symlinks)
+	 */
+	cp = dcanon(cp, cp);
+
+	set(vp, Strsave(cp), VAR_READWRITE);	/* have to save the new val */
+
+	/* and now mirror home with HOME */
+	tsetenv(STRKHOME, cp);
+	/* fix directory stack for new tilde home */
+	dtilde();
+	xfree((ptr_t) cp);
+    }
+    else if (eq(vp, STRedit)) {
+	editing = 1;
+	/* PWP: add more stuff in here later */
+    }
+    else if (eq(vp, STRshlvl)) {
+	tsetenv(STRKSHLVL, value(vp));
+    }
+    else if (eq(vp, STRbackslash_quote)) {
+	bslash_quote = 1;
+    }
+    else if (eq(vp, STRdirstack)) {
+	dsetstack();
+    }
+    else if (eq(vp, STRrecognize_only_executables)) {
+	tw_cmd_free();
+    }
+#ifndef HAVENOUTMP
+    else if (eq(vp, STRwatch)) {
+	resetwatch();
+    }
+#endif /* HAVENOUTMP */
+}
+
+
 /*ARGSUSED*/
 void
 doset(v, c)
@@ -70,11 +162,20 @@ doset(v, c)
     Char  **vecp;
     bool    hadsub;
     int     subscr;
+    int	    flags = VAR_READWRITE;
 
+    USE(c);
     v++;
+    /*
+     * Readonly addition From: Tim P. Starrin <noid@cyborg.larc.nasa.gov>
+     */
+    if (*v && eq(*v, STRmr)) {
+	flags = VAR_READONLY;
+	v++;
+    }
     p = *v++;
     if (p == 0) {
-	prvars();
+	plist(&shvhed, flags);
 	return;
     }
     do {
@@ -120,93 +221,15 @@ doset(v, c)
 	    p = *e;
 	    *e = 0;
 	    vecp = saveblk(v);
-	    set1(vp, vecp, &shvhed);
+	    set1(vp, vecp, &shvhed, flags);
 	    *e = p;
 	    v = e + 1;
 	}
 	else if (hadsub)
 	    asx(vp, subscr, Strsave(p));
 	else
-	    set(vp, Strsave(p));
-	if (eq(vp, STRpath)) {
-	    exportpath(adrof(STRpath)->vec);
-	    dohash(NULL, NULL);
-	}
-	else if (eq(vp, STRhistchars)) {
-	    register Char *pn = value(vp);
-
-	    HIST = *pn++;
-	    HISTSUB = *pn;
-	}
-	else if (eq(vp, STRhistlit)) {
-	    HistLit = 1;
-	}
-	else if (eq(vp, STRuser)) {
-	    Setenv(STRUSER, value(vp));
-	    Setenv(STRLOGNAME, value(vp));
-	}
-	else if (eq(vp, STRwordchars)) {
-	    word_chars = value(vp);
-	}
-	else if (eq(vp, STRsymlinks)) {
-	    register Char *pn = value(vp);
-
-	    if (eq(pn, STRignore))
-		symlinks = SYM_IGNORE;
-	    else if (eq(pn, STRexpand))
-		symlinks = SYM_EXPAND;
-	    else if (eq(pn, STRchase))
-		symlinks = SYM_CHASE;
-	    else
-		symlinks = 0;
-	}
-	else if (eq(vp, STRterm)) {
-#ifdef DOESNT_WORK_RIGHT
-	    register Char *cp;
-#endif /* DOESNT_WORK_RIGHT */
-	    Setenv(STRTERM, value(vp));
-#ifdef DOESNT_WORK_RIGHT
-	    cp = getenv("TERMCAP");
-	    if (cp && (*cp != '/'))	/* if TERMCAP and not a path */
-		Unsetenv(STRTERMCAP);
-#endif /* DOESNT_WORK_RIGHT */
-	    GotTermCaps = 0;
-	    ed_Init();		/* reset the editor */
-	}
-	else if (eq(vp, STRhome)) {
-	    register Char *cp;
-
-	    cp = Strsave(value(vp));	/* get the old value back */
-
-	    /*
-	     * convert to cononical pathname (possibly resolving symlinks)
-	     */
-	    cp = dcanon(cp, cp);
-
-	    set(vp, Strsave(cp));	/* have to save the new val */
-
-	    /* and now mirror home with HOME */
-	    Setenv(STRHOME, cp);
-	    /* fix directory stack for new tilde home */
-	    dtilde();
-	    xfree((ptr_t) cp);
-	}
-	else if (eq(vp, STRedit)) {
-	    editing = 1;
-	    /* PWP: add more stuff in here later */
-	}
-	else if (eq(vp, STRshlvl)) {
-	    Setenv(STRSHLVL, value(vp));
-	}
-	else if (eq(vp, STRbackslash_quote)) {
-	    bslash_quote = 1;
-	}
-	else if (eq(vp, STRrecognize_only_executables)) {
-	    tw_cmd_free();
-	}
-	else if (eq(vp, STRwatch)) {
-	    resetwatch();
-	}
+	    set(vp, Strsave(p), flags);
+	update_vars(vp);
     } while ((p = *v++) != NULL);
 }
 
@@ -232,6 +255,8 @@ asx(vp, subscr, p)
 {
     register struct varent *v = getvx(vp, subscr);
 
+    if (v->v_flags & VAR_READONLY)
+	stderror(ERR_READONLY|ERR_NAME, v->v_name);
     xfree((ptr_t) v->vec[subscr - 1]);
     v->vec[subscr - 1] = globone(p, G_APPEND);
 }
@@ -261,6 +286,7 @@ dolet(v, dummy)
     bool    hadsub;
     int     subscr;
 
+    USE(dummy);
     v++;
     p = *v++;
     if (p == 0) {
@@ -323,18 +349,15 @@ dolet(v, dummy)
 	    if (hadsub)
 		asx(vp, subscr, p);
 	    else
-		set(vp, p);
+		set(vp, p, VAR_READWRITE);
 	else if (hadsub) {
 	    struct varent *gv = getvx(vp, subscr);
 
 	    asx(vp, subscr, operate(op, gv->vec[subscr - 1], p));
 	}
 	else
-	    set(vp, operate(op, value(vp), p));
-	if (eq(vp, STRpath)) {
-	    exportpath(adrof(STRpath)->vec);
-	    dohash(NULL, NULL);
-	}
+	    set(vp, operate(op, value(vp), p), VAR_READWRITE);
+	update_vars(vp);
 	xfree((ptr_t) vp);
 	if (c != '=')
 	    xfree((ptr_t) p);
@@ -370,7 +393,7 @@ operate(op, vp, p)
     if (op != '=') {
 	if (*vp)
 	    *v++ = vp;
-	opr[0] = op;
+	opr[0] = (Char) op;
 	opr[1] = 0;
 	*v++ = opr;
 	if (op == '<' || op == '>')
@@ -398,7 +421,7 @@ putn(n)
 	*putp++ = '-';
     }
     num = 2;			/* confuse lint */
-    if (sizeof(int) == num && ((unsigned int) n) == 32768) {
+    if (sizeof(int) == num && ((unsigned int) n) == 0x8000) {
 	*putp++ = '3';
 	n = 2768;
 #ifdef pdp11
@@ -407,7 +430,7 @@ putn(n)
     }
     else {
 	num = 4;		/* confuse lint */
-	if (sizeof(int) == num && ((unsigned int) n) == 2147483648) {
+	if (sizeof(int) == num && ((unsigned int) n) == 0x80000000) {
 	    *putp++ = '2';
 	    n = 147483648;
 	}
@@ -505,20 +528,22 @@ adrof1(name, v)
  * The caller is responsible for putting value in a safe place
  */
 void
-set(var, val)
+set(var, val, flags)
     Char   *var, *val;
+    int	   flags;
 {
     register Char **vec = (Char **) xmalloc((size_t) (2 * sizeof(Char **)));
 
     vec[0] = val;
     vec[1] = 0;
-    set1(var, vec, &shvhed);
+    set1(var, vec, &shvhed, flags);
 }
 
 void
-set1(var, vec, head)
+set1(var, vec, head, flags)
     Char   *var, **vec;
     struct varent *head;
+    int flags;
 {
     register Char **oldv = vec;
 
@@ -534,14 +559,15 @@ set1(var, vec, head)
 	blkfree(oldv);
 	gargv = 0;
     }
-    setq(var, vec, head);
+    setq(var, vec, head, flags);
 }
 
 
 void
-setq(name, vec, p)
+setq(name, vec, p, flags)
     Char   *name, **vec;
     register struct varent *p;
+    int flags;
 {
     register struct varent *c;
     register f;
@@ -550,19 +576,23 @@ setq(name, vec, p)
     while ((c = p->v_link[f]) != 0) {
 	if ((f = *name - *c->v_name) == 0 &&
 	    (f = Strcmp(name, c->v_name)) == 0) {
+	    if (c->v_flags & VAR_READONLY)
+		stderror(ERR_READONLY|ERR_NAME, c->v_name);
 	    blkfree(c->vec);
-	    goto found;
+	    c->v_flags = flags;
+	    trim(c->vec = vec);
+	    return;
 	}
 	p = c;
 	f = f > 0;
     }
     p->v_link[f] = c = (struct varent *) xmalloc((size_t)sizeof(struct varent));
     c->v_name = Strsave(name);
+    c->v_flags = flags;
     c->v_bal = 0;
     c->v_left = c->v_right = 0;
     c->v_parent = p;
     balance(p, f, 0);
-found:
     trim(c->vec = vec);
 }
 
@@ -574,6 +604,7 @@ unset(v, c)
 {
     bool did_only;
 
+    USE(c);
     did_only = adrof(STRrecognize_only_executables) != NULL;
     unset1(v, &shvhed);
     if (adrof(STRhistchars) == 0) {
@@ -605,7 +636,10 @@ unset1(v, head)
     while (*++v) {
 	cnt = 0;
 	while ((vp = madrof(*v, head)) != NULL)
-	    unsetv1(vp), cnt++;
+	    if (vp->v_flags & VAR_READONLY)
+		stderror(ERR_READONLY|ERR_NAME, vp->v_name);
+	    else
+		unsetv1(vp), cnt++;
 	if (cnt == 0)
 	    setname(short2str(*v));
     }
@@ -670,7 +704,7 @@ void
 setNS(cp)
     Char   *cp;
 {
-    set(cp, Strsave(STRNULL));
+    set(cp, Strsave(STRNULL), VAR_READWRITE);
 }
 
 /*ARGSUSED*/
@@ -682,6 +716,7 @@ shift(v, c)
     register struct varent *argv;
     register Char *name;
 
+    USE(c);
     v++;
     name = *v;
     if (name == 0)
@@ -694,8 +729,11 @@ shift(v, c)
     if (argv->vec[0] == 0)
 	stderror(ERR_NAME | ERR_NOMORE);
     lshift(argv->vec, 1);
+    update_vars(name);
 }
 
+static Char STRsep[2] = { PATHSEP, '\0' };
+
 static void
 exportpath(val)
     Char  **val;
@@ -712,9 +750,9 @@ exportpath(val)
 	    (void) Strcat(exppath, *val++);
 	    if (*val == 0 || eq(*val, STRRparen))
 		break;
-	    (void) Strcat(exppath, STRcolon);
+	    (void) Strcat(exppath, STRsep);
 	}
-    Setenv(STRPATH, exppath);
+    tsetenv(STRKPATH, exppath);
 }
 
 #ifndef lint
@@ -864,8 +902,9 @@ balance(p, f, d)
 }
 
 void
-plist(p)
+plist(p, what)
     register struct varent *p;
+    int what;
 {
     register struct varent *c;
     register len;
@@ -883,14 +922,16 @@ plist(p)
 x:
 	if (p->v_parent == 0)	/* is it the header? */
 	    return;
-	len = blklen(p->vec);
-	xprintf("%s\t", short2str(p->v_name));
-	if (len != 1)
-	    xputchar('(');
-	blkpr(p->vec);
-	if (len != 1)
-	    xputchar(')');
-	xputchar('\n');
+	if ((p->v_flags & what) != 0) {
+	    len = blklen(p->vec);
+	    xprintf("%S\t", p->v_name);
+	    if (len != 1)
+		xputchar('(');
+	    blkpr(p->vec);
+	    if (len != 1)
+		xputchar(')');
+	    xputchar('\n');
+	}
 	if (p->v_right) {
 	    p = p->v_right;
 	    continue;
