@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.02/RCS/tc.bind.c,v 3.8 1992/04/10 16:38:09 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.04/RCS/tc.bind.c,v 3.15 1993/07/03 23:47:53 christos Exp $ */
 /*
  * tc.bind.c: Key binding functions
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tc.bind.c,v 3.8 1992/04/10 16:38:09 christos Exp $")
+RCSID("$Id: tc.bind.c,v 3.15 1993/07/03 23:47:53 christos Exp $")
 
 #include "ed.h"
 #include "ed.defns.h"
@@ -106,7 +106,7 @@ unparsekey(c)			/* 'c' -> "c", '^C' -> "^" + "C" */
 	c &= ASCII;
     }
     if (Isprint(c)) {
-	*cp++ = c;
+	*cp++ = (char) c;
 	*cp = '\0';
 	return (tmp);
     }
@@ -315,7 +315,7 @@ parsekey(sp)
 	    else if (!strcmp(ts, "delete"))
 		c = '\177';
 	    else {
-		xprintf("bad key specification -- unknown name \"%s\"\n", s);
+		xprintf("bad key specification -- unknown name \"%S\"\n", s);
 		return -1;	/* error */
 	    }
 	}
@@ -340,7 +340,7 @@ dobindkey(v, c)
     struct command *c;
 {
     KEYCMD *map;
-    int     ntype, no, remove;
+    int     ntype, no, remove, key;
     Char   *par;
     Char    p;
     Char    inbuf[200];
@@ -349,18 +349,22 @@ dobindkey(v, c)
     Char   *out;
     KEYCMD  cmd;
 
+    USE(c);
     if (!MapsAreInited)
 	ed_InitMaps();
 
     map = CcKeyMap;
     ntype = XK_CMD;
-    remove = 0;
+    key = remove = 0;
     for (no = 1, par = v[no]; 
 	 par != NULL && (*par++ & CHAR) == '-'; no++, par = v[no]) {
 	if ((p = (*par & CHAR)) == '-')
 	    break;
 	else 
 	    switch (p) {
+	    case 'k':
+		key = 1;
+		break;
 	    case 'a':
 		map = CcAltMap;
 		break;
@@ -400,9 +404,17 @@ dobindkey(v, c)
 	return;
     }
 
-    if ((in = parsestring(v[no++], inbuf)) == NULL)
-	return;
+    if (key) 
+	in = v[no++];
+    else
+	if ((in = parsestring(v[no++], inbuf)) == NULL)
+	    return;
+
     if (remove) {
+	if (key) {
+	    (void) ClearArrowKeys(in);
+	    return;
+	}
 	if (in[1]) {
 	    (void) DeleteXkey(in);
 	}
@@ -416,7 +428,10 @@ dobindkey(v, c)
 	return;
     }
     if (!v[no]) {
-	printkey(map, in);
+	if (key)
+	    PrintArrowKeys(in);
+	else
+	    printkey(map, in);
 	return;
     }
     if (v[no + 1]) {
@@ -428,25 +443,34 @@ dobindkey(v, c)
     case XK_EXE:
 	if ((out = parsestring(v[no], outbuf)) == NULL)
 	    return;
-	AddXkey(in, XmapStr(out), ntype);
+	if (key)
+	    (void) SetArrowKeys(in, XmapStr(out), ntype);
+	else
+	    AddXkey(in, XmapStr(out), ntype);
 	map[(unsigned char) *in] = F_XKEY;
 	break;
     case XK_CMD:
 	if ((cmd = parsecmd(v[no])) == 0)
 	    return;
-	if (in[1]) {
-	    AddXkey(in, XmapCmd((int) cmd), ntype);
-	    map[(unsigned char) *in] = F_XKEY;
-	}
+	if (key)
+	    (void) SetArrowKeys(in, XmapCmd((int) cmd), ntype);
 	else {
-	    ClearXkey(map, in);
-	    map[(unsigned char) *in] = cmd;
+	    if (in[1]) {
+		AddXkey(in, XmapCmd((int) cmd), ntype);
+		map[(unsigned char) *in] = F_XKEY;
+	    }
+	    else {
+		ClearXkey(map, in);
+		map[(unsigned char) *in] = cmd;
+	    }
 	}
 	break;
     default:
 	abort();
 	break;
     }
+    if (key)
+	BindArrowKeys();
 }
 
 static void
@@ -477,10 +501,10 @@ parsecmd(str)
 
     for (fp = FuncNames; fp->name; fp++) {
 	if (strcmp(short2str(str), fp->name) == 0) {
-	    return fp->func;
+	    return (KEYCMD) fp->func;
 	}
     }
-    xprintf("Bad command name: %s\n", short2str(str));
+    xprintf("Bad command name: %S\n", str);
     return 0;
 }
 
@@ -494,7 +518,7 @@ parseescape(ptr)
 
     if ((p[1] & CHAR) == 0) {
 	xprintf("Something must follow: %c\n", *p);
-	return 0;
+	return -1;
     }
     if ((*p & CHAR) == '\\') {
 	p++;
@@ -555,7 +579,8 @@ parseescape(ptr)
 	    break;
 	}
     }
-    else if ((*p & CHAR) == '^') {
+    else if ((*p & CHAR) == '^' && (Isalpha(p[1] & CHAR) || 
+				    strchr("@^_?\\|[{]}", p[1] & CHAR))) {
 	p++;
 	c = (*p == '?') ? '\177' : ((*p & CHAR) & 0237);
     }
@@ -572,6 +597,7 @@ parsestring(str, buf)
 {
     Char   *b;
     Char   *p;
+    int    es;
 
     b = buf;
     if (*str == 0) {
@@ -581,12 +607,13 @@ parsestring(str, buf)
 
     for (p = str; *p != 0; p++) {
 	if ((*p & CHAR) == '\\' || (*p & CHAR) == '^') {
-	    if ((*b++ = parseescape(&p)) == 0)
+	    if ((es = parseescape(&p)) == -1)
 		return 0;
+	    else
+		*b++ = (Char) es;
 	}
-	else {
+	else
 	    *b++ = *p & CHAR;
-	}
     }
     *b = 0;
     return buf;
@@ -664,6 +691,8 @@ print_all_keys()
     printkeys(CcAltMap, prev, i - 1);
     xprintf("Multi-character bindings\n");
     PrintXkey(STRNULL);	/* print all Xkey bindings */
+    xprintf("Arrow key bindings\n");
+    PrintArrowKeys(STRNULL);
 }
 
 static void
@@ -675,9 +704,9 @@ printkeys(map, first, last)
     Char    firstbuf[2], lastbuf[2];
     unsigned char unparsbuf[10], extrabuf[10];
 
-    firstbuf[0] = first;
+    firstbuf[0] = (Char) first;
     firstbuf[1] = 0;
-    lastbuf[0] = last;
+    lastbuf[0] = (Char) last;
     lastbuf[1] = 0;
     if (map[first] == F_UNASSIGNED) {
 	if (first == last)
@@ -725,6 +754,7 @@ bindkey_usage()
     xprintf("    -d   initialized maps to default bindings\n");
     xprintf("    -l   list available functions with descriptions\n");
     xprintf("    -r   remove the binding of in-string\n");
+    xprintf("    -k   bind arrow key with name in in-string\n");
     xprintf(
        "\nIn no out-string or command is given, the binding for in-string\n");
     xprintf("is printed or all bindings if in-strings is not given.\n");
@@ -752,6 +782,7 @@ dobind(v, dummy)
     Char   *p, *l;
     Char    buf[1000];
 
+    USE(dummy);
     /*
      * Assume at this point that i'm given 2 or 3 args - 'bind', the f-name,
      * and the key; or 'bind' key to print the func for that key.
