@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.02/RCS/tc.os.h,v 3.29 1992/05/09 04:03:53 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.04/RCS/tc.os.h,v 3.48 1993/07/07 19:16:17 christos Exp $ */
 /*
  * tc.os.h: Shell os dependent defines
  */
@@ -39,21 +39,50 @@
 
 #define NEEDstrerror		/* Too hard to find which systems have it */
 
-#if SYSVREL > 3
+
+#ifdef notdef 
 /*
- * for SVR4 we fork pipelines backwards. 
+ * for SVR4 and linux we used to fork pipelines backwards. 
+ * This should not be needed any more.
  * more info in sh.sem.c
  */
 # define BACKPIPE
-#endif /* SYSVREL > 3 */
+#endif /* notdef */
+
+#ifdef   _VMS_POSIX
+# ifndef  NOFILE 
+#  define  NOFILE 64
+# endif /* NOFILE */
+# define  nice(a)       setprio((getpid()),a)
+# undef   NEEDstrerror    /* won't get sensible error messages otherwise */
+# define  NEEDgethostname 
+# include <sys/time.h>    /* for time stuff in tc.prompt.c */
+# include <limits.h>
+#endif /*atp vmsposix*/
+
+#if defined(OPEN_MAX) && !defined(NOFILE)
+# define NOFILE OPEN_MAX
+#endif /* OPEN_MAX && !NOFILE */
+
+#if defined(USR_NFDS) && !defined(NOFILE)
+# define NOFILE USR_NFDS
+#endif /* USR_NFDS && !NOFILE */
+
+#ifndef NOFILE
+# define NOFILE 256
+#endif /* NOFILE */
+
+#ifdef linux
+# undef NEEDstrerror
+#endif /* linux */
 
 #ifdef OREO
 # include <sys/time.h>
-# include <sys/resource.h>
+# ifdef notdef
+  /* Don't include it, because it defines things we don't really have */
+#  include <sys/resource.h>	
+# endif /* notdef */
 # ifdef POSIX
-#  ifdef T_BREAK
-#   undef T_BREAK
-#  endif /* T_BREAK */
 #  include <sys/tty.h>
 #  include <termios.h>
 # endif /* POSIX */
@@ -62,7 +91,7 @@
 #ifndef NCARGS
 # ifdef ARG_MAX
 #  define NCARGS ARG_MAX
-# else
+# else /* !ARG_MAX */
 #  ifdef _MINIX
 #   define NCARGS 80
 #  else /* !_MINIX */
@@ -97,7 +126,7 @@ struct ucred {
 
 # ifndef hp9000s500
 #  include <sys/bsdtty.h>
-# endif
+# endif /* hp9000s500 */
 
 # ifndef POSIX
 #  ifdef BSDJOBS
@@ -114,6 +143,13 @@ struct ucred {
 # ifndef CSUSP
 #  define CSUSP 032
 # endif	/* CSUSP */
+# if defined(POSIX) && !defined(TIOCGWINSZ)
+/*
+ * ISC defines this only in termio.h. If we are using POSIX and include
+ * termios.h, then we define it ourselves so that window resizing works.
+ */
+#  define TIOCGWINSZ      (('T'<<8)|104)
+# endif /* POSIX && !TIOCGWINSZ */
 #endif /* ISC */
 
 #ifdef ISC202
@@ -124,23 +160,28 @@ struct ucred {
  * XXX: This will be changed soon to 
  * #if (SYSVREL > 0) && defined(TIOCGWINSZ)
  * If that breaks on your machine, let me know.
+ *
+ * It would break on linux, where all this is
+ * defined in <termios.h>. Wrapper added.
  */
-#if defined(INTEL) || defined(u3b2) || defined (u3b5) || defined(ub15) || defined(u3b20d) || defined(ISC) || defined(SCO) 
-#ifdef TIOCGWINSZ
+#if !defined(linux) && !defined(_VMS_POSIX)
+# if defined(INTEL) || defined(u3b2) || defined (u3b5) || defined(ub15) || defined(u3b20d) || defined(ISC) || defined(SCO) || defined(tower32)
+#  ifdef TIOCGWINSZ
 /*
  * for struct winsiz
  */
-# include <sys/stream.h>
-# include <sys/ptem.h>
-#endif /* TIOCGWINSZ */
-# ifndef ODT
-#  define NEEDgethostname
-# endif /* ODT */
-#endif /* INTEL || att || isc || sco */
+#   include <sys/stream.h>
+#   include <sys/ptem.h>
+#  endif /* TIOCGWINSZ */
+#  ifndef ODT
+#   define NEEDgethostname
+#  endif /* ODT */
+# endif /* INTEL || u3b2 || u3b5 || ub15 || u3b20d || ISC || SCO || tower32 */
+#endif /* !linux && !_VMS_POSIX */
 
-#ifdef UNIXPC
+#if defined(UNIXPC) || defined(COHERENT)
 # define NEEDgethostname
-#endif /* UNIXPC */
+#endif /* UNIXPC || COHERENT */
 
 #ifdef IRIS4D
 # include <sys/time.h>
@@ -151,9 +192,28 @@ struct ucred {
  */
 #  define setpgrp BSDsetpgrp
 #  define getpgrp BSDgetpgrp
-# endif
+# endif /* POSIX */
 #endif /* IRIS4D */
 
+/*
+ * For some versions of system V software, specially ones that use the 
+ * Wollongong Software TCP/IP, the FIOCLEX, FIONCLEX, FIONBIO calls
+ * might not work correctly for file descriptors [they work only for
+ * sockets]. So we try to use first the fcntl() and we only use the
+ * ioctl() form, only if we don't have the fcntl() one.
+ *
+ * From: scott@craycos.com (Scott Bolte)
+ */
+#ifdef F_SETFD
+# define close_on_exec(fd, v) fcntl((fd), F_SETFD, v)
+#else /* !F_SETFD */
+# ifdef FIOCLEX
+# define close_on_exec(fd, v) ioctl((fd), ((v) ? FIOCLEX : FIONCLEX), NULL)
+# else /* !FIOCLEX */
+# define close_on_exec(fd, v)	/* Nothing */
+# endif /* FIOCLEX */
+#endif /* F_SETFD */
+
 /*
  * Stat
  */
@@ -170,13 +230,19 @@ struct ucred {
 # endif /* S_IFMT */
 #endif /* ISC */
 
-#ifdef uts
+#if defined(uts) || defined(UTekV) || defined(sysV88)
 /*
  * The uts 2.1.2 macros (Amdahl) are busted!
  * You should fix <sys/stat.h>, cause other programs will break too!
  *
  * From: creiman@ncsa.uiuc.edu (Charlie Reiman)
  */
+
+/*
+ * The same applies to Motorola MPC (System V/88 R32V2, UTekV 3.2e) 
+ * workstations, the stat macros are broken.
+ * Kaveh Ghazi (ghazi@caip.rutgers.edu)
+ */
 # undef S_ISDIR
 # undef S_ISCHR
 # undef S_ISBLK
@@ -185,7 +251,7 @@ struct ucred {
 # undef S_ISNAM
 # undef S_ISLNK
 # undef S_ISSOCK
-#endif /* uts */
+#endif /* uts || UTekV || sysV88 */
 
 #ifdef S_IFMT
 # if !defined(S_ISDIR) && defined(S_IFDIR)
@@ -214,6 +280,38 @@ struct ucred {
 # endif	/* ! S_ISSOCK && S_IFSOCK */
 #endif /* S_IFMT */
 
+#ifdef tower32
+/* The header files lie; we really don't have symlinks */
+# undef S_ISLNK
+# undef S_IFLNK
+#endif /* tower32 */
+
+#ifndef S_IREAD
+# define S_IREAD 0000400
+#endif /* S_IREAD */
+#ifndef S_IROTH
+# define S_IROTH (S_IREAD >> 6)
+#endif /* S_IROTH */
+#ifndef S_IRGRP
+# define S_IRGRP (S_IREAD >> 3)
+#endif /* S_IRGRP */
+#ifndef S_IRUSR
+# define S_IRUSR S_IREAD
+#endif /* S_IRUSR */
+
+#ifndef S_IWRITE
+# define S_IWRITE 0000200
+#endif /* S_IWRITE */
+#ifndef S_IWOTH
+# define S_IWOTH (S_IWRITE >> 6)
+#endif /* S_IWOTH */
+#ifndef S_IWGRP
+# define S_IWGRP (S_IWRITE >> 3)
+#endif /* S_IWGRP */
+#ifndef S_IWUSR
+# define S_IWUSR S_IWRITE
+#endif /* S_IWUSR */
+
 #ifndef S_IEXEC
 # define S_IEXEC 0000100
 #endif /* S_IEXEC */
@@ -227,6 +325,16 @@ struct ucred {
 # define S_IXUSR S_IEXEC
 #endif /* S_IXUSR */
 
+#ifndef S_ISUID
+# define S_ISUID 0004000 	/* setuid */
+#endif /* S_ISUID */
+#ifndef S_ISGID	
+# define S_ISGID 0002000	/* setgid */
+#endif /* S_ISGID */
+#ifndef S_ISVTX
+# define S_ISVTX 0001000	/* sticky */
+#endif /* S_ISVTX */
+
 /*
  * Access()
  */
@@ -262,21 +370,21 @@ struct ucred {
 #ifndef L_SET
 # ifdef SEEK_SET
 #  define L_SET		SEEK_SET
-# else
+# else /* !SEEK_SET */
 #  define L_SET		0
 # endif	/* SEEK_SET */
 #endif /* L_SET */
 #ifndef L_INCR
 # ifdef SEEK_CUR
 #  define L_INCR	SEEK_CUR
-# else
+# else /* !SEEK_CUR */
 #  define L_INCR	1
 # endif	/* SEEK_CUR */
 #endif /* L_INCR */
 #ifndef L_XTND
 # ifdef SEEK_END
 #  define L_XTND	SEEK_END
-# else
+# else /* !SEEK_END */
 #  define L_XTND	2
 # endif /* SEEK_END */
 #endif /* L_XTND */
@@ -285,9 +393,9 @@ struct ucred {
 # define NEEDgethostname
 #endif /* _SEQUENT_ */
 
-#if defined(BSD) && defined(POSIXJOBS) 
+#if defined(BSD) && defined(POSIXJOBS) && !defined(__hp_osf)
 # define setpgid(pid, pgrp)	setpgrp(pid, pgrp)
-#endif /* BSD && POSIXJOBS */
+#endif /* BSD && POSIXJOBS && !__hp_osf */
 
 #if defined(BSDJOBS) && !(defined(POSIX) && defined(POSIXJOBS))
 # if !defined(_AIX370) && !defined(_AIXPS2)
@@ -316,38 +424,37 @@ struct ucred {
 #  define setrlimit 	bsd_setrlimit
 #  define getrlimit	bsd_getrlimit
 # endif	/* _BSDX_ */
-# ifndef NOFILE
-#  define	NOFILE	64
-# endif	/* NOFILE */
 #endif /* SXA */
 
-#ifdef _MINIX
-# ifndef NOFILE
-#  define NOFILE 64
-# endif /* NOFILE */
+#if defined(_MINIX) || defined(__EMX__)
 # define NEEDgethostname
 # define NEEDnice
+# define HAVENOLIMIT
 /*
  * Minix does not have these, so...
  */
-# define ulimit(a, b)		(0x003fffff)
-# define getpgrp()		getpid()
-#endif /* _MINIX */
+# define getpgrp		getpid
+#endif /* _MINIX || __EMX__ */
+
+#ifdef __EMX__
+/* XXX: How can we get the tty name in emx? */
+# define ttyname(fd) (isatty(fd) ? "/dev/tty" : NULL)
+#endif /* __EMX__ */
 
 #ifndef POSIX
 # define mygetpgrp()    getpgrp(0)
 #else /* POSIX */
-# if defined(BSD) || defined(sun) || defined(IRIS4D)
+# if defined(BSD) || defined(SUNOS4) || defined(IRIS4D) || defined(DGUX)
 #  define mygetpgrp()    getpgrp(0)
-# else /* BSD || sun || IRIS4D */
+# else /* !(BSD || SUNOS4 || IRIS4D || DGUX) */
 #  define mygetpgrp()    getpgrp()
-# endif	/* BSD || sun */
+# endif	/* BSD || SUNOS4 || IRISD || DGUX */
 #endif /* POSIX */
 
 
-#if SYSVREL > 0 && !defined(OREO) && !defined(sgi)
+#if SYSVREL > 0 && !defined(OREO) && !defined(sgi) && !defined(linux)
 # define NEEDgetwd
-#endif /* SYSVREL > 0 && !OREO && !sgi */
+#endif /* SYSVREL > 0 && !OREO && !sgi && !linux */
 
 #ifndef S_IFLNK
 # define lstat stat
@@ -369,41 +476,54 @@ typedef struct timeval timeval_t;
 #endif /* NeXT */
 
 
-#if !defined(POSIX) || defined(sun)
+#if !defined(POSIX) || defined(SUNOS4) || defined(UTekV) || defined(sysV88)
 extern time_t time();
 extern char *getenv();
 extern int atoi();
+# ifndef __EMX__
 extern char *ttyname();
+# endif /* __EMX__ */
 
-# ifndef hpux
-#  if __GNUC__ != 2
-extern int abort();
-#  endif /* __GNUC__ != 2 */
-# ifndef fps500
+# if defined(SUNOS4)
+#  ifndef toupper
+extern int toupper __P((int));
+#  endif /* toupper */
+#  ifndef tolower
+extern int tolower __P((int));
+#  endif /* tolower */
+extern caddr_t sbrk __P((int));
+#  if SYSVREL == 0 && !defined(__lucid)
 extern int qsort();
-# endif /* fps500 */
-# else
+#  endif /* SYSVREL == 0 && !__lucid */
+# else /* !SUNOS4 */
+#  ifndef hpux
+#   if __GNUC__ != 2
+extern int abort();
+#   endif /* __GNUC__ != 2 */
+#   ifndef fps500
+extern int qsort();
+#   endif /* !fps500 */
+#  else /* !hpux */
 extern void abort();
 extern void qsort();
-# endif
+#  endif /* hpux */
+# endif	/* SUNOS4 */
 extern void perror();
 
-#ifndef NEEDgethostname
+# ifndef NEEDgethostname
 extern int gethostname();
-#endif
+# endif /* NEEDgethostname */
 
 # ifdef BSDSIGS
-#  if defined(_AIX370) || defined(MACH) || defined(NeXT) || defined(_AIXPS2) || defined(ardent)
+#  if defined(_AIX370) || defined(MACH) || defined(NeXT) || defined(_AIXPS2) || defined(ardent) || defined(SUNOS4)
 extern int sigvec();
 extern int sigpause();
-#  else	/* _AIX370 || MACH || NeXT || _AIXPS2 || ardent */
-#   if !defined(apollo) || !defined(__STDC__)
-#    ifndef fps500
+#  else	/* !(_AIX370 || MACH || NeXT || _AIXPS2 || ardent || SUNOS4) */
+#   if (!defined(apollo) || !defined(__STDC__)) && !defined(__DGUX__) && !defined(fps500)
 extern sigret_t sigvec();
 extern void sigpause();
-#    endif /* fps500 */
-#   endif /* !apollo || !__STDC__ */
-#  endif /* _AIX370 || MACH || NeXT || _AIXPS2 */
+#   endif /* (!apollo || !__STDC__) && !__DGUX__ && !fps500 */
+#  endif /* _AIX370 || MACH || NeXT || _AIXPS2 || ardent || SUNOS4 */
 extern sigmask_t sigblock();
 extern sigmask_t sigsetmask();
 # endif	/* BSDSIGS */
@@ -416,10 +536,10 @@ extern int killpg();
 extern int lstat();
 # endif	/* lstat */
 
-#ifdef BSD
+# ifdef BSD
 extern uid_t getuid(), geteuid();
 extern gid_t getgid(), getegid();
-#endif /* BSD */
+# endif /* BSD */
 
 # ifdef SYSMALLOC
 extern memalign_t malloc();
@@ -437,76 +557,98 @@ extern int gettimeofday();
 
 # if defined(NLS) && !defined(NOSTRCOLL) && !defined(NeXT)
 extern int strcoll();
-# endif
+# endif /* NLS && !NOSTRCOLL && !NeXT */
 
 # ifdef BSDJOBS
 #  ifdef BSDTIMES
 extern int wait3();
-#  else	/* ! BSDTIMES */
+#  else	/* !BSDTIMES */
 #   if !defined(POSIXJOBS) && !defined(_SEQUENT_)
 extern int wait3();
 #   else /* POSIXJOBS || _SEQUENT_ */
 extern int waitpid();
 #   endif /* POSIXJOBS || _SEQUENT_ */
-#  endif /* ! BSDTIMES */
+#  endif /* BSDTIMES */
 # else /* !BSDJOBS */
 #  if SYSVREL < 3
 extern int ourwait();
 #  else	/* SYSVREL >= 3 */
 extern int wait();
-#  endif /* SYSVREL >= 3 */
-# endif	/* ! BSDJOBS */
+#  endif /* SYSVREL < 3 */
+# endif	/* BSDJOBS */
 
 # ifdef BSDNICE
 extern int setpriority();
 # else /* !BSDNICE */
 extern int nice();
-# endif	/* !BSDNICE */
+# endif	/* BSDNICE */
 
-# ifndef fps500
+# if (!defined(fps500) && !defined(apollo) && !defined(__lucid))
 extern void setpwent();
 extern void endpwent();
-# endif /* fps500 */
+# endif /* !fps500 && !apollo && !__lucid */
 
-#ifndef __STDC__
+# ifndef __STDC__
 extern struct passwd *getpwuid(), *getpwnam(), *getpwent();
-#ifdef PW_SHADOW
+#  ifdef PW_SHADOW
 extern struct spwd *getspnam(), *getspent();
-#endif /* PW_SHADOW */
-#ifdef PW_AUTH
+#  endif /* PW_SHADOW */
+#  ifdef PW_AUTH
 extern struct authorization *getauthuid();
-#endif /* PW_AUTH */
-#endif /* __STDC__ */
+#  endif /* PW_AUTH */
+# endif /* __STDC__ */
 
 # ifndef getwd
 extern char *getwd();
 # endif	/* getwd */
-#else /* POSIX */
 
-# if (defined(sun) && !defined(__GNUC__)) || defined(_IBMR2) || defined(_IBMESA)
+#else /* POSIX || !SUNOS4 || !UTekV || !sysV88 */
+
+# if (defined(SUNOS4) && !defined(__GNUC__)) || defined(_IBMR2) || defined(_IBMESA)
 extern char *getwd();
-# endif	/* (sun && ! __GNUC__) || _IBMR2 || _IBMESA */
+# endif	/* (SUNOS4 && ! __GNUC__) || _IBMR2 || _IBMESA */
 
 # ifdef SCO
 extern char *ttyname();   
 # endif /* SCO */
 
+#endif /* !POSIX || SUNOS4 || UTekV || sysV88 */
 
-#endif /* POSIX */
-
-# if defined(sun) && __GNUC__ == 2
+#if defined(SUNOS4) && __GNUC__ == 2
 /*
  * Somehow these are missing
  */
 extern int ioctl __P((int, int, ...));
 extern int readlink __P((const char *, char *, size_t));
-# endif /* sun && __GNUC__ == 2 */
+#endif /* SUNOS4 && __GNUC__ == 2 */
 
-#ifdef linux
-extern int		tcgetpgrp	__P((int));
-extern int		tcsetpgrp	__P((int, int));
-extern int		gethostname	__P((char *, int));
-extern int		readlink	__P(());
-#endif /* linux */
+#if (defined(BSD) && !defined(__386BSD__)) || defined(SUNOS4) 
+extern void bcopy	__P((const void *, void *, size_t));
+# define memmove(a, b, c) (bcopy((char *) (b), (char *) (a), (int) (c)), a)
+#endif /* (BSD && !__386BSD__) || SUNOS4 */
+
+#if !defined(hpux) && !defined(COHERENT) && ((SYSVREL < 4) || defined(_SEQUENT_)) && !defined(__386BSD__) && !defined(memmove)
+# define NEEDmemmove
+#endif /* !hpux && !COHERENT && (SYSVREL < 4 || _SEQUENT_) && !__36BSD__ && !memmove */
+
+#if SYSVREL == 4
+extern int gethostname();
+extern int getrlimit();
+extern int setrlimit();
+extern int getrusage();
+# ifndef IRIS4D
+extern int gettimeofday();
+extern int wait3();
+# endif
+#endif /* SYSVREL == 4 */
+
+#if defined(__osf__) && defined(__alpha)
+extern int gethostname __P((char *, int));
+extern void *sbrk __P((ssize_t));
+extern int ioctl __P((int, unsigned long, char *));
+extern pid_t vfork __P((void));
+extern int killpg __P((pid_t, int));
+extern char *getwd __P((char *));
+#endif /* __osf__ && __alpha */
 
 #endif /* _h_tc_os */
