@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.02/RCS/tc.printf.c,v 3.6 1992/04/03 22:15:14 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.04/RCS/tc.printf.c,v 3.13 1993/06/25 21:17:12 christos Exp $ */
 /*
  * tc.printf.c: A public-domain, minimal printf/sprintf routine that prints
  *	       through the putchar() routine.  Feel free to use for
@@ -38,7 +38,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tc.printf.c,v 3.6 1992/04/03 22:15:14 christos Exp $")
+RCSID("$Id: tc.printf.c,v 3.13 1993/06/25 21:17:12 christos Exp $")
 
 #ifdef lint
 #undef va_arg
@@ -50,15 +50,19 @@ RCSID("$Id: tc.printf.c,v 3.6 1992/04/03 22:15:14 christos Exp $")
 static char buf[128];
 
 static	void	xaddchar	__P((int));
-static	void	doprnt		__P((void (*) __P((int)), char *, va_list));
+static	void	doprnt		__P((void (*) __P((int)), const char *, va_list));
 
 static void
 doprnt(addchar, sfmt, ap)
     void    (*addchar)();
-    char   *sfmt;
+    const char   *sfmt;
     va_list ap;
 {
-    register char *f, *bp;
+    register char *bp;
+    register const char *f;
+#ifdef SHORT_STRINGS
+    register Char *Bp;
+#endif /* SHORT_STRINGS */
     register long l;
     register unsigned long u;
     register int i;
@@ -121,7 +125,7 @@ doprnt(addchar, sfmt, ap)
 	    }
 
 	    fmt = (unsigned char) *f;
-	    if (Isupper(fmt)) {
+	    if (fmt != 'S' && Isupper(fmt)) {
 		do_long = 1;
 		fmt = Tolower(fmt);
 	    }
@@ -137,15 +141,15 @@ doprnt(addchar, sfmt, ap)
 		    l = -l;
 		}
 		do {
-		    *bp++ = l % 10 + '0';
+		    *bp++ = (char) (l % 10) + '0';
 		} while ((l /= 10) > 0);
 		if (sign)
 		    *bp++ = '-';
 		f_width = f_width - (bp - buf);
 		if (!flush_left)
-		    while (f_width-- > 0)
+		    while (f_width-- > 0) 
 			(*addchar) ((int) (pad | attributes));
-		for (bp--; bp >= buf; bp--)
+		for (bp--; bp >= buf; bp--) 
 		    (*addchar) ((int) (((unsigned char) *bp) | attributes));
 		if (flush_left)
 		    while (f_width-- > 0)
@@ -161,12 +165,12 @@ doprnt(addchar, sfmt, ap)
 		    u = (unsigned long) (va_arg(ap, unsigned int));
 		if (fmt == 'u') {	/* unsigned decimal */
 		    do {
-			*bp++ = u % 10 + '0';
+			*bp++ = (char) (u % 10) + '0';
 		    } while ((u /= 10) > 0);
 		}
 		else if (fmt == 'o') {	/* octal */
 		    do {
-			*bp++ = u % 8 + '0';
+			*bp++ = (char) (u % 8) + '0';
 		    } while ((u /= 8) > 0);
 		    if (hash)
 			*bp++ = '0';
@@ -201,8 +205,35 @@ doprnt(addchar, sfmt, ap)
 		(*addchar) ((int) (i | attributes));
 		break;
 
+	    case 'S':
+#ifdef SHORT_STRINGS
+		Bp = va_arg(ap, Char *);
+		if (!Bp) {
+		    bp = NULL;
+		    goto lcase_s;
+	        }
+		f_width = f_width - Strlen(Bp);
+		if (!flush_left)
+		    while (f_width-- > 0)
+			(*addchar) ((int) (pad | attributes));
+		for (i = 0; *Bp && i < prec; i++) {
+		    (*addchar) ((int) ((unsigned char)*Bp | attributes));
+		    Bp++;
+		}
+		if (flush_left)
+		    while (f_width-- > 0)
+			(*addchar) ((int) (' ' | attributes));
+		break;
+#else
+		bp = va_arg(ap, Char *);
+		if (bp)
+		    (void) strip((Char *) bp);
+		goto lcase_s;
+#endif /* SHORT_STRINGS */
+
 	    case 's':
 		bp = va_arg(ap, char *);
+lcase_s:
 		if (!bp)
 		    bp = "(nil)";
 		f_width = f_width - strlen((char *) bp);
@@ -247,10 +278,10 @@ xaddchar(c)
 }
 
 
-void
+pret_t
 /*VARARGS*/
 #if __STDC__
-xsprintf(char *str, char *fmt, ...)
+xsprintf(char *str, const char *fmt, ...)
 #else
 xsprintf(va_alist)
     va_dcl
@@ -271,13 +302,16 @@ xsprintf(va_alist)
     doprnt(xaddchar, fmt, va);
     va_end(va);
     *xstring++ = '\0';
+#ifdef PURIFY
+    return 1;
+#endif
 }
 
 
-void
+pret_t
 /*VARARGS*/
 #if __STDC__
-xprintf(char *fmt, ...)
+xprintf(const char *fmt, ...)
 #else
 xprintf(va_alist)
     va_dcl
@@ -294,24 +328,82 @@ xprintf(va_alist)
 #endif
     doprnt(xputchar, fmt, va);
     va_end(va);
+#ifdef PURIFY
+    return 1;
+#endif
 }
 
 
-void
+pret_t
 xvprintf(fmt, va)
-    char   *fmt;
+    const char   *fmt;
     va_list va;
 {
     doprnt(xputchar, fmt, va);
+#ifdef PURIFY
+    return 1;
+#endif
 }
 
-void
+pret_t
 xvsprintf(str, fmt, va)
     char   *str;
-    char   *fmt;
+    const char   *fmt;
     va_list va;
 {
     xstring = (char *) str;
     doprnt(xaddchar, fmt, va);
     *xstring++ = '\0';
+#ifdef PURIFY
+    return 1;
+#endif
 }
+
+
+
+#ifdef PURIFY
+/* Purify uses (some of..) the following functions to output memory-use
+ * debugging info.  Given all the messing with file descriptors that
+ * tcsh does, the easiest way I could think of to get it (Purify) to
+ * print anything was by replacing some standard functions with
+ * ones that do tcsh output directly - see dumb hook in doreaddirs()
+ * (sh.dir.c) -sg
+ */
+#ifndef FILE
+#define FILE int
+#endif
+int 
+#if __STDC__
+fprintf(FILE *fp, const char* fmt, ...)
+#else
+fprintf(va_alist)
+    va_dcl
+#endif
+{
+    va_list va;
+#if __STDC__
+    va_start(va, fmt);
+#else
+    FILE *fp;
+    const char   *fmt;
+
+    va_start(va);
+    fp = va_arg(va, FILE *);
+    fmt = va_arg(va, const char *);
+#endif
+    doprnt(xputchar, fmt, va);
+    va_end(va);
+    return 1;
+}
+
+int 
+vfprintf(fp, fmt, va)
+    FILE *fp;
+    const char   *fmt;
+    va_list va;
+{
+    doprnt(xputchar, fmt, va);
+    return 1;
+}
+
+#endif	/* PURIFY */
