@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.02/RCS/tw.h,v 3.9 1992/04/03 22:15:14 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.04/RCS/tw.h,v 3.13 1993/04/26 21:13:10 christos Exp $ */
 /*
  * tw.h: TwENEX functions headers
  */
@@ -56,12 +56,16 @@
 #define TW_SIGNAL	15
 #define TW_JOB		16
 #define TW_EXPLAIN	17
+#define TW_PATHNAME	18
+#define TW_TEXT		19
 
 #define TW_EXEC_CHK	0x01
 #define TW_DIR_CHK	0x02
-#define TW_DIR_OK	0x04
-#define TW_PAT_OK	0x08
-#define TW_IGN_OK	0x10
+#define TW_TEXT_CHK	0x04
+
+#define TW_DIR_OK	0x10
+#define TW_PAT_OK	0x20
+#define TW_IGN_OK	0x40
 
 #ifndef TRUE
 # define TRUE		1
@@ -84,9 +88,17 @@
 extern Char PromptBuf[];
 
 typedef enum {
-    LIST, RECOGNIZE, PRINT_HELP, SPELL, GLOB, GLOB_EXPAND,
-    VARS_EXPAND, PATH_NORMALIZE
-}       COMMAND;
+    LIST, LIST_ALL, RECOGNIZE, RECOGNIZE_ALL, RECOGNIZE_SCROLL, 
+    PRINT_HELP, SPELL, GLOB, GLOB_EXPAND, VARS_EXPAND, PATH_NORMALIZE
+} COMMAND;
+
+struct scroll_tab_list {
+	Char *element;
+	struct scroll_tab_list *next;
+} ;
+
+extern struct scroll_tab_list *scroll_tab;
+extern int curchoice;
 
 extern int non_unique_match;
 
