@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.02/RCS/tw.init.c,v 3.12 1992/05/15 23:49:22 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.04/RCS/tw.init.c,v 3.16 1993/07/03 23:47:53 christos Exp $ */
 /*
  * tw.init.c: Handle lists of things to complete
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tw.init.c,v 3.12 1992/05/15 23:49:22 christos Exp $")
+RCSID("$Id: tw.init.c,v 3.16 1993/07/03 23:47:53 christos Exp $")
 
 #include "tw.h"
 #include "ed.h"
@@ -216,7 +216,7 @@ tw_cmd_add(name)
 
     if (name[0] == '#' || name[0] == '.')	/* emacs temp's, .files	*/
 	return;
-    len = Strlen(name) + 2;
+    len = (int) Strlen(name) + 2;
     if (name[len - 3] == '~')			/* No emacs backups */
 	return;
     
@@ -361,6 +361,7 @@ tw_cmd_start(dfd, pat)
     Char *pat;
 {
     static Char *defpath[] = { STRNULL, 0 };
+    USE(pat);
     SETDIR(dfd)
     if ((tw_cmd_got & TW_FL_CMD) == 0) {
 	tw_cmd_free();
@@ -491,6 +492,8 @@ tw_shvar_next(dir, flags)
     register struct varent *c;
     register Char *cp;
 
+    USE(flags);
+    USE(dir);
     if ((p = tw_vptr) == NULL)
 	return (NULL);		/* just in case */
 
@@ -532,6 +535,8 @@ tw_envvar_next(dir, flags)
 {
     Char   *ps, *pd;
 
+    USE(flags);
+    USE(dir);
     if (tw_env == NULL || *tw_env == NULL)
 	return (NULL);
     for (ps = *tw_env, pd = tw_retname;
@@ -552,6 +557,7 @@ tw_var_start(dfd, pat)
     DIR *dfd;
     Char *pat;
 {
+    USE(pat);
     SETDIR(dfd)
     tw_vptr_start(&shvhed);
     tw_env = STR_environ;
@@ -567,6 +573,7 @@ tw_alias_start(dfd, pat)
     DIR *dfd;
     Char *pat;
 {
+    USE(pat);
     SETDIR(dfd)
     tw_vptr_start(&aliases);
     tw_env = NULL;
@@ -583,6 +590,8 @@ tw_complete_start(dfd, pat)
     Char *pat;
 {
     extern struct varent completions;
+
+    USE(pat);
     SETDIR(dfd)
     tw_vptr_start(&completions);
     tw_env = NULL;
@@ -616,8 +625,11 @@ tw_logname_start(dfd, pat)
     DIR *dfd;
     Char *pat;
 {
+    USE(pat);
     SETDIR(dfd)
+#ifndef _VMS_POSIX
     (void) setpwent();	/* Open passwd file */
+#endif /* atp vmsposix */
 } /* end tw_logname_start */
 
 
@@ -638,9 +650,13 @@ tw_logname_next(dir, flags)
      * and if we call endpwent() immediatetely after
      * (in pintr()) we may be freeing an invalid pointer
      */
+    USE(flags);
+    USE(dir);
     TW_HOLD();
+#ifndef _VMS_POSIX
     /* ISC does not declare getpwent()? */
     pw = (struct passwd *) getpwent();
+#endif /* atp vmsposix */
     TW_RELS();
 
     if (pw == NULL) {
@@ -663,7 +679,9 @@ tw_logname_end()
 #ifdef YPBUGS
     fix_yp_bugs();
 #endif
+#ifndef _VMS_POSIX
    (void) endpwent();
+#endif /* atp vmsposix */
 } /* end tw_logname_end */
 
 
@@ -677,6 +695,7 @@ tw_file_start(dfd, pat)
     Char *pat;
 {
     struct varent *vp;
+    USE(pat);
     SETDIR(dfd)
     if ((vp = adrof(STRcdpath)) != NULL)
 	tw_env = vp->vec;
@@ -812,6 +831,7 @@ tw_wl_next(dir, flags)
     Char *dir;
     int *flags;
 {
+    USE(flags);
     if (tw_word == NULL || tw_word[0] == '\0')
 	return NULL;
     
@@ -834,6 +854,7 @@ tw_bind_start(dfd, pat)
     DIR *dfd;
     Char *pat;
 {
+    USE(pat);
     SETDIR(dfd)
     tw_bind = FuncNames;
 } /* end tw_bind_start */
@@ -849,8 +870,10 @@ tw_bind_next(dir, flags)
     int *flags;
 {
     char *ptr;
+    USE(flags);
     if (tw_bind && tw_bind->name) {
-	for (ptr = tw_bind->name, dir = tw_retname; (*dir++ = *ptr++) != '\0';)
+	for (ptr = tw_bind->name, dir = tw_retname;
+	     (*dir++ = (Char) *ptr++) != '\0';)
 	    continue;
 	tw_bind++;
 	return(tw_retname);
@@ -868,6 +891,7 @@ tw_limit_start(dfd, pat)
     DIR *dfd;
     Char *pat;
 {
+    USE(pat);
     SETDIR(dfd)
 #ifndef HAVENOLIMIT
     tw_limit = limits;
@@ -888,12 +912,13 @@ tw_limit_next(dir, flags)
     char *ptr;
     if (tw_limit && tw_limit->limname) {
 	for (ptr = tw_limit->limname, dir = tw_retname; 
-	     (*dir++ = *ptr++) != '\0';)
+	     (*dir++ = (Char) *ptr++) != '\0';)
 	    continue;
 	tw_limit++;
 	return(tw_retname);
     }
 #endif /* ! HAVENOLIMIT */
+    USE(flags);
     return NULL;
 } /* end tw_limit_next */
 
@@ -907,6 +932,7 @@ tw_sig_start(dfd, pat)
     DIR *dfd;
     Char *pat;
 {
+    USE(pat);
     SETDIR(dfd)
     tw_index = 0;
 } /* end tw_sig_start */
@@ -922,13 +948,15 @@ tw_sig_next(dir, flags)
     int *flags;
 {
     char *ptr;
-    for (;tw_index < NSIG; tw_index++) {
+    extern int nsig;
+    USE(flags);
+    for (;tw_index < nsig; tw_index++) {
 
 	if (mesg[tw_index].iname == NULL)
 	    continue;
 
 	for (ptr = mesg[tw_index].iname, dir = tw_retname; 
-	     (*dir++ = *ptr++) != '\0';)
+	     (*dir++ = (Char) *ptr++) != '\0';)
 	    continue;
 	tw_index++;
 	return(tw_retname);
@@ -946,6 +974,7 @@ tw_job_start(dfd, pat)
     DIR *dfd;
     Char *pat;
 {
+    USE(pat);
     SETDIR(dfd)
     tw_index = 1;
 } /* end tw_job_start */
@@ -962,6 +991,8 @@ tw_job_next(dir, flags)
 {
     Char *ptr;
     struct process *j;
+
+    USE(flags);
     for (;tw_index <= pmaxindex; tw_index++) {
 	for (j = proclist.p_next; j != NULL; j = j->p_next)
 	    if (j->p_index == tw_index && j->p_procid == j->p_jobid)
