@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.02/RCS/tw.parse.c,v 3.33 1992/05/09 04:03:53 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.04/RCS/tw.parse.c,v 3.54 1993/07/03 23:47:53 christos Exp $ */
 /*
  * tw.parse.c: Everyone has taken a shot in this futile effort to
  *	       lexically analyze a csh line... Well we cannot good
@@ -39,7 +39,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tw.parse.c,v 3.33 1992/05/09 04:03:53 christos Exp $")
+RCSID("$Id: tw.parse.c,v 3.54 1993/07/03 23:47:53 christos Exp $")
 
 #include "tw.h"
 #include "ed.h"
@@ -55,13 +55,13 @@ RCSID("$Id: tw.parse.c,v 3.33 1992/05/09 04:03:53 christos Exp $")
 /*  TW_FILE,	       TW_DIRECTORY,   TW_VARLIST,     TW_USER,		*/
 /*  TW_COMPLETION,     TW_ALIAS,       TW_SHELLVAR,    TW_ENVVAR,	*/
 /*  TW_BINDING,        TW_WORDLIST,    TW_LIMIT,       TW_SIGNAL	*/
-/*  TW_JOB,	       TW_EXPLAIN					*/
+/*  TW_JOB,	       TW_EXPLAIN,     TW_PATHNAME,    TW_TEXT		*/
 static void (*tw_start_entry[]) __P((DIR *, Char *)) = {
     tw_file_start,     tw_cmd_start,   tw_var_start,   tw_logname_start, 
     tw_file_start,     tw_file_start,  tw_vl_start,    tw_logname_start, 
     tw_complete_start, tw_alias_start, tw_var_start,   tw_var_start,     
     tw_bind_start,     tw_wl_start,    tw_limit_start, tw_sig_start,
-    tw_job_start,      tw_file_start
+    tw_job_start,      tw_file_start,  tw_file_start,  tw_file_start
 };
 
 static Char * (*tw_next_entry[]) __P((Char *, int *)) = {
@@ -69,7 +69,7 @@ static Char * (*tw_next_entry[]) __P((Char *, int *)) = {
     tw_file_next,      tw_file_next,   tw_var_next,    tw_logname_next,  
     tw_var_next,       tw_var_next,    tw_shvar_next,  tw_envvar_next,   
     tw_bind_next,      tw_wl_next,     tw_limit_next,  tw_sig_next,
-    tw_job_next,       tw_file_next
+    tw_job_next,       tw_file_next,   tw_file_next,   tw_file_next
 };
 
 static void (*tw_end_entry[]) __P((void)) = {
@@ -77,7 +77,7 @@ static void (*tw_end_entry[]) __P((void)) = {
     tw_dir_end,        tw_dir_end,     tw_dir_end,    tw_logname_end, 
     tw_dir_end,        tw_dir_end,     tw_dir_end,    tw_dir_end,
     tw_dir_end,        tw_dir_end,     tw_dir_end,    tw_dir_end,
-    tw_dir_end,	       tw_dir_end
+    tw_dir_end,	       tw_dir_end,     tw_dir_end,    tw_dir_end
 };
 
 /* #define TDEBUG */
@@ -85,6 +85,9 @@ static void (*tw_end_entry[]) __P((void)) = {
 /* Set to TRUE if recexact is set and an exact match is found
  * along with other, longer, matches.
  */
+
+int curchoice = -1;
+
 int non_unique_match = FALSE;
 static bool SearchNoDirErr = 0;	/* t_search returns -2 if dir is unreadable */
 
@@ -116,11 +119,15 @@ static	int	 isadirectory		__P((Char *, Char *));
 static  int      tw_collect_items	__P((COMMAND, int, Char *, Char *, 
 					     Char *, Char *, int));
 static  int      tw_collect		__P((COMMAND, int, Char *, Char *, 
-					     Char *, Char *, int, DIR *));
+					     Char **, Char *, int, DIR *));
 static	Char 	 tw_suffix		__P((int, Char *, Char *, Char *, 
 					     Char *));
 static	void 	 tw_fixword		__P((int, Char *, Char *, Char *, int));
 static	void	 tw_list_items		__P((int, int, int));
+static 	void	 add_scroll_tab		__P((Char *));
+static 	void 	 choose_scroll_tab	__P((Char **, int));
+static	void	 free_scroll_tab	__P((void));
+static	int	 find_rows		__P((Char *[], int, int));
 
 #ifdef notdef
 /*
@@ -168,14 +175,14 @@ tenematch(inputline, num_read, command)
     word = cmd_start = wp = qline;
     for (cp = inputline; cp < str_end; cp++) {
         if (!cmap(qu, _ESC)) {
-	    if (cmap(*cp, _Q|_ESC)) {
+	    if (cmap(*cp, _QF|_ESC)) {
 		if (qu == 0 || qu == *cp) {
 		    qu ^= *cp;
 		    continue;
 		}
 	    }
-	    if (qu != '\'' && cmap(*cp, _Q1)) {
-		if (backq ^= 1) {
+	    if (qu != '\'' && cmap(*cp, _QB)) {
+		if ((backq ^= 1) != 0) {
 		    ocmd_start = cmd_start;
 		    oword_start = word_start;
 		    oword = word;
@@ -205,10 +212,6 @@ tenematch(inputline, num_read, command)
       }
     *wp = 0;
 
-    /* move the word_start after the quote */
-    if (*word_start == qu)
-	word_start++;
-
 #ifdef masscomp
     /*
      * Avoid a nasty message from the RTU 4.1A & RTU 5.0 compiler concerning
@@ -228,13 +231,13 @@ tenematch(inputline, num_read, command)
 
 #ifdef TDEBUG
     xprintf("starting_a_command %d\n", looking);
-    xprintf("\ncmd_start:%s:\n", short2str(cmd_start));
-    xprintf("qline:%s:\n", short2str(qline));
+    xprintf("\ncmd_start:%S:\n", cmd_start);
+    xprintf("qline:%S:\n", qline);
     xprintf("qline:");
     for (wp = qline; *wp; wp++)
 	xprintf("%c", *wp & QUOTE ? '-' : ' ');
     xprintf(":\n");
-    xprintf("word:%s:\n", short2str(word));
+    xprintf("word:%S:\n", word);
     xprintf("word:");
     /* Must be last, so wp is still pointing to the end of word */
     for (wp = word; *wp; wp++)
@@ -242,13 +245,16 @@ tenematch(inputline, num_read, command)
     xprintf(":\n");
 #endif
 
+    if (command == RECOGNIZE || command == LIST || command == SPELL ||
+	command == RECOGNIZE_SCROLL || command == RECOGNIZE_ALL) {
 #ifdef TDEBUG
-    xprintf("complete %d ", looking);
+	xprintf("complete %d ", looking);
 #endif
-    looking = tw_complete(cmd_start, &wordp, &pat, looking, &suf);
+	looking = tw_complete(cmd_start, &wordp, &pat, looking, &suf);
 #ifdef TDEBUG
-    xprintf("complete %d %s\n", looking, short2str(pat));
+	xprintf("complete %d %S\n", looking, pat);
 #endif
+    }
 
     switch ((int) command) {
 	Char    buffer[FILSIZ + 1], *bptr;
@@ -257,6 +263,8 @@ tenematch(inputline, num_read, command)
 	int     i, count;
 
     case RECOGNIZE:
+    case RECOGNIZE_SCROLL:
+    case RECOGNIZE_ALL:
 	if (adrof(STRautocorrect)) {
 	    if ((slshp = Strrchr(wordp, '/')) != NULL && slshp[1] != '\0') {
 		SearchNoDirErr = 1;
@@ -284,16 +292,16 @@ tenematch(inputline, num_read, command)
 	    (void) Strcpy(rword, slshp);
 	    if (slshp != STRNULL)
 		*slshp = '\0';
-	    search_ret = spell_me(wordp, QLINESIZE - (wordp - qline), 
-				  looking == TW_COMMAND);
+	    search_ret = spell_me(wordp, QLINESIZE - (wordp - qline), looking);
 	    if (search_ret == 1) {
 		/* get rid of old word */
 		DeleteBack(str_end - word_start);
+		qu = 0;
 		(void) Strcat(wordp, rword);
 		/* insert newly spelled word */
 		if (InsertStr(quote_meta(wordp, qu, 0)) < 0)	
 		    return -1;	/* error inserting */
-		wp = wordp + Strlen(wordp);
+		wp = wordp + (int) Strlen(wordp);
 		search_ret = t_search(wordp, wp, command, space_left,
 				      looking, 1, pat, suf);
 	    }
@@ -306,7 +314,7 @@ tenematch(inputline, num_read, command)
 	 * addsuffix was set. Otherwise the last space was part of a word.
 	 */
 	if (*wp && InsertStr(quote_meta(wp, qu, search_ret == 1 &&
-					     (bool) is_set(STRaddsuffix))) < 0)
+			     is_set(STRaddsuffix) != NULL)) < 0)
 	    /* put it in the input buffer */
 	    return -1;		/* error inserting */
 	return search_ret;
@@ -320,13 +328,20 @@ tenematch(inputline, num_read, command)
 	    if (isglob(*bptr))
 		return 0;
 	}
-	search_ret = spell_me(wordp, QLINESIZE - (wordp - qline), 
-			      looking == TW_COMMAND);
+	search_ret = spell_me(wordp, QLINESIZE - (wordp - qline), looking);
 	if (search_ret == 1) {
 	    /* get rid of old word */
 	    DeleteBack(str_end - word_start);	
 	    /* insert newly spelled word */
-	    if (InsertStr(quote_meta(wordp, qu, 0)) < 0)
+#ifdef notdef
+	    /* 
+	     * We don't want to quote spelling stuff, otherwise
+	     * $OHME -> \$HOME 
+	     */
+	    if (InsertStr(quote_meta(wordp, 0, 0)) < 0)
+		return -1;	/* error inserting */
+#endif
+	    if (InsertStr(wordp) < 0)
 		return -1;	/* error inserting */
 	}
 	return search_ret;
@@ -351,7 +366,7 @@ tenematch(inputline, num_read, command)
 		DeleteBack(str_end - word_start);/* get rid of old word */
 		for (i = 0; i < count; i++)
 		    if (ptr[i] && *ptr[i]) {
-			if (InsertStr(quote_meta(ptr[i], qu, 0)) < 0 ||
+			if (InsertStr(quote_meta(ptr[i], 0, 0)) < 0 ||
 			    InsertStr(STRspace) < 0) {
 			    blkfree(ptr);
 			    return (-1);
@@ -377,14 +392,15 @@ tenematch(inputline, num_read, command)
 	    (void) Strcpy(buffer, bptr);
 	    xfree((ptr_t) bptr);
 	    DeleteBack(str_end - word_start);
-	    if (InsertStr(quote_meta(buffer, qu, 0)) < 0)
+	    if (InsertStr(quote_meta(buffer, 0, 0)) < 0)
 		return (-1);
 	    return (1);
 	}
 	return (0);
 
     case LIST:
-	search_ret = t_search(wordp, wp, command, space_left, looking, 1, 
+    case LIST_ALL:
+	search_ret = t_search(wordp, wp, LIST, space_left, looking, 1, 
 			      pat, suf);
 	return search_ret;
 
@@ -404,7 +420,7 @@ t_glob(v, cmd)
     register Char ***v;
     int cmd;
 {
-    jmp_buf osetexit;
+    jmp_buf_t osetexit;
 
     if (**v == 0)
 	return (0);
@@ -505,12 +521,12 @@ quote_meta(word, qu, trail_space)
 	    break;
 	  
 	if (qu && *wptr == qu) {
-	    *bptr++ = qu;
+	    *bptr++ = (Char) qu;
 	    *bptr++ = '\\';
-	    *bptr++ = qu;
+	    *bptr++ = (Char) qu;
 	}
 	if (!qu &&
-	    (cmap(*wptr, _META | _DOL | _Q | _Q1 | _ESC | _GLOB) ||
+	    (cmap(*wptr, _META | _DOL | _QF | _QB | _ESC | _GLOB) ||
 	     *wptr == HIST || *wptr == HISTSUB) &&
 	    (*wptr != ' ' || !trail_space || *(wptr + 1) != '\0') &&
              *wptr != '#')
@@ -519,6 +535,12 @@ quote_meta(word, qu, trail_space)
 	if (cmap(qu, _ESC))
 	  qu = 0;
     }
+    if (qu && trail_space && wptr > word && *wptr == 0 && wptr[-1] == ' ')
+      {
+        *bptr = bptr[-1];
+        bptr[-1] = (Char) qu;
+        bptr++;
+      }
     *bptr = '\0';
     return (buffer);
 } /* end quote_meta */
@@ -567,11 +589,11 @@ is_suffix(check, template)
 
 
 /* ignored():
- *	Return true if this is an ignored entry
+ *	Return true if this is an ignored item
  */
 static int
-ignored(entry)
-    register Char *entry;
+ignored(item)
+    register Char *item;
 {
     struct varent *vp;
     register Char **cp;
@@ -579,7 +601,7 @@ ignored(entry)
     if ((vp = adrof(STRfignore)) == NULL || (cp = vp->vec) == NULL)
 	return (FALSE);
     for (; *cp != NULL; cp++)
-	if (is_suffix(entry, *cp))
+	if (is_suffix(item, *cp))
 	    return (TRUE);
     return (FALSE);
 } /* end ignored */
@@ -662,23 +684,23 @@ starting_a_command(wordstart, inputline)
 /* recognize():
  *	Object: extend what user typed up to an ambiguity.
  *	Algorithm:
- *	On first match, copy full entry (assume it'll be the only match)
+ *	On first match, copy full item (assume it'll be the only match)
  *	On subsequent matches, shorten exp_name to the first
- *	character mismatch between exp_name and entry.
+ *	character mismatch between exp_name and item.
  *	If we shorten it back to the prefix length, stop searching.
  */
 static int
-recognize(exp_name, entry, name_length, numitems)
-    Char   *exp_name, *entry;
+recognize(exp_name, item, name_length, numitems)
+    Char   *exp_name, *item;
     int     name_length, numitems;
 {
     if (numitems == 1)		/* 1st match */
-	copyn(exp_name, entry, MAXNAMLEN);
+	copyn(exp_name, item, MAXNAMLEN);
     else {			/* 2nd and subsequent matches */
 	register Char *x, *ent;
 	register int len = 0;
 
-	for (x = exp_name, ent = entry;
+	for (x = exp_name, ent = item;
 	     *x && (*x & TRIM) == (*ent & TRIM); x++, len++, ent++)
 	    continue;
 	*x = '\0';		/* Shorten at 1st char diff */
@@ -710,17 +732,21 @@ tw_collect_items(command, looking, exp_dir, exp_name, target, pat, flags)
     int showdots;			 /* Style to show dot files */
     int nignored = 0;			 /* Number of fignored items */
     int numitems = 0;			 /* Number of matched items */
-    int	name_length = Strlen(target);	 /* Length of prefix (file name) */
+    int name_length = (int) Strlen(target); /* Length of prefix (file name) */
     int exec_check = flags & TW_EXEC_CHK;/* need to check executability	*/
     int dir_check  = flags & TW_DIR_CHK; /* Need to check for directories */
+    int text_check = flags & TW_TEXT_CHK;/* Need to check for non-directories */
     int dir_ok     = flags & TW_DIR_OK;  /* Ignore directories? */
     int gpat       = flags & TW_PAT_OK;	 /* Match against a pattern */
     int ignoring   = flags & TW_IGN_OK;	 /* Use fignore? */
-    int d = 4, nd = 0;			 /* Spelling distance */
-    Char *entry, *ptr;
+    int d = 4, nd;			 /* Spelling distance */
+    Char *item, *ptr;
     Char buf[MAXPATHLEN+1];
     struct varent *vp;
     int len;
+    int cnt = 0;
+
+
     flags = 0;
 
     if ((vp = adrof(STRshowdots)) != NULL) {
@@ -733,19 +759,20 @@ tw_collect_items(command, looking, exp_dir, exp_name, target, pat, flags)
     else
 	showdots = DOT_NONE;
 
-    while (!done && (entry = (*tw_next_entry[looking])(exp_dir, &flags))) {
+    while (!done && (item = (*tw_next_entry[looking])(exp_dir, &flags))) {
 #ifdef TDEBUG
-	xprintf("entry = %s\n", short2str(entry));
+	xprintf("item = %S\n", item);
 #endif
 	switch (looking) {
 	case TW_FILE:
 	case TW_DIRECTORY:
+	case TW_TEXT:
 	    /*
 	     * Don't match . files on null prefix match
 	     */
-	    if (showdots == DOT_NOT && (ISDOT(entry) || ISDOTDOT(entry)))
+	    if (showdots == DOT_NOT && (ISDOT(item) || ISDOTDOT(item)))
 		done = TRUE;
-	    if (name_length == 0 && entry[0] == '.' && showdots == DOT_NONE)
+	    if (name_length == 0 && item[0] == '.' && showdots == DOT_NONE)
 		done = TRUE;
 	    break;
 
@@ -772,19 +799,19 @@ tw_collect_items(command, looking, exp_dir, exp_name, target, pat, flags)
 		done = TRUE;
 		break;
 	    }
-	    if (gpat && !Gmatch(entry, pat))
+	    if (gpat && !Gmatch(item, pat))
 		break;
-	    nd = spdist(entry, target);	/* test the entry against original */
+	    nd = spdist(item, target);	/* test the item against original */
 	    if (nd <= d && nd != 4) {
-		if (!(exec_check && !executable(exp_dir, entry, dir_ok))) {
-		    (void) Strcpy(exp_name, entry);
+		if (!(exec_check && !executable(exp_dir, item, dir_ok))) {
+		    (void) Strcpy(exp_name, item);
 		    d = nd;
 		    if (d == 0)	/* if found it exactly */
 			done = TRUE;
 		}
 	    }
 	    else if (nd == 4) {
-		if (spdir(exp_name, exp_dir, entry, target)) {
+		if (spdir(exp_name, exp_dir, item, target)) {
 		    if (exec_check && !executable(exp_dir, exp_name, dir_ok)) 
 			break;
 #ifdef notdef
@@ -802,30 +829,37 @@ tw_collect_items(command, looking, exp_dir, exp_name, target, pat, flags)
 
 	case LIST:
 	case RECOGNIZE:
+	case RECOGNIZE_ALL:
+	case RECOGNIZE_SCROLL:
 
-	    if (!is_prefix(target, entry)) 
+	    if (!is_prefix(target, item)) 
 		break;
 
-	    if (exec_check && !executable(exp_dir, entry, dir_ok))
+	    if (exec_check && !executable(exp_dir, item, dir_ok))
 		break;
 
-	    if (dir_check && !isadirectory(exp_dir, entry))
+	    if (dir_check && !isadirectory(exp_dir, item))
 		break;
 
-	    if (gpat && !Gmatch(entry, pat) && !isadirectory(exp_dir, entry))
+	    if (text_check && isadirectory(exp_dir, item))
+		break;
+
+	    if (gpat && !Gmatch(item, pat) && !isadirectory(exp_dir, item))
 		break;
 
 	    /*
 	     * Remove duplicates in command listing and completion
+             * AFEB added code for TW_LOGNAME and TW_USER cases
 	     */
-	    if (looking == TW_COMMAND || command == LIST) {
-		copyn(buf, entry, MAXPATHLEN);
-		len = Strlen(buf);
+	    if (looking == TW_COMMAND || looking == TW_LOGNAME
+		|| looking == TW_USER || command == LIST) {
+		copyn(buf, item, MAXPATHLEN);
+		len = (int) Strlen(buf);
 		switch (looking) {
 		case TW_COMMAND:
 		    if (!(dir_ok && exec_check))
 			break;
-		    if (filetype(exp_dir, entry) == '/') {
+		    if (filetype(exp_dir, item) == '/') {
 			buf[len++] = '/';
 			buf[len] = '\0';
 		    }
@@ -833,14 +867,15 @@ tw_collect_items(command, looking, exp_dir, exp_name, target, pat, flags)
 
 		case TW_FILE:
 		case TW_DIRECTORY:
-		    buf[len++] = filetype(exp_dir, entry);
+		    buf[len++] = filetype(exp_dir, item);
 		    buf[len] = '\0';
 		    break;
 
 		default:
 		    break;
 		}
-		if (looking == TW_COMMAND && tw_item_find(buf))
+		if ((looking == TW_COMMAND || looking == TW_USER
+                     || looking == TW_LOGNAME) && tw_item_find(buf))
 		    break;
 		else {
 		    /* maximum length 1 (NULL) + 1 (~ or $) + 1 (filetype) */
@@ -851,22 +886,29 @@ tw_collect_items(command, looking, exp_dir, exp_name, target, pat, flags)
 		}
 	    }
 		    
-	    if (command == RECOGNIZE) {
-		if (ignoring && ignored(entry)) {
+	    if (command == RECOGNIZE || command == RECOGNIZE_ALL ||
+		command == RECOGNIZE_SCROLL) {
+		if (ignoring && ignored(item)) {
 		    nignored++;
 		    break;
+		} 
+		else if (command == RECOGNIZE_SCROLL) {
+		    add_scroll_tab(item);
+		    cnt++;
 		}
+		
 		if (is_set(STRrecexact)) {
-		    if (StrQcmp(target, entry) == 0) {	/* EXACT match */
-			copyn(exp_name, entry, MAXNAMLEN);
+		    if (StrQcmp(target, item) == 0) {	/* EXACT match */
+			copyn(exp_name, item, MAXNAMLEN);
 			numitems = 1;	/* fake into expanding */
 			non_unique_match = TRUE;
 			done = TRUE;
 			break;
 		    }
 		}
-		if (recognize(exp_name, entry, name_length, ++numitems)) 
-		    done = TRUE;
+		if (recognize(exp_name, item, name_length, ++numitems)) 
+		    if (command != RECOGNIZE_SCROLL)
+			done = TRUE;
 	    }
 	    break;
 
@@ -874,9 +916,25 @@ tw_collect_items(command, looking, exp_dir, exp_name, target, pat, flags)
 	    break;
 	}
 #ifdef TDEBUG
-	xprintf("done entry = %s\n", short2str(entry));
+	xprintf("done item = %S\n", item);
 #endif
     }
+
+
+    if (command == RECOGNIZE_SCROLL) {
+	if ((cnt <= curchoice) || (curchoice == -1)) {
+	    curchoice = -1;
+	    nignored = 0;
+	    numitems = 0;
+	} else if (numitems > 1) {
+	    if (curchoice < -1)
+		curchoice = cnt - 1;
+	    choose_scroll_tab(&exp_name, cnt);
+	    numitems = 1;
+	}
+    }
+    free_scroll_tab();
+
     if (command == SPELL)
 	return d;
     else {
@@ -909,13 +967,11 @@ tw_suffix(looking, exp_dir, exp_name, target, name)
 	 * Don't consider array variables or empty variables
 	 */
 	if ((vp = adrof(exp_name)) != NULL) {
-	    if ((ptr = vp->vec[0]) != NULL || vp->vec[0][0] == '\0' ||
-		vp->vec[1]) 
+	    if ((ptr = vp->vec[0]) == NULL || *ptr == '\0' ||
+		vp->vec[1] != NULL) 
 		return ' ';
-	    else
-		ptr = vp->vec[0];
 	}
-	else if ((ptr = Getenv(exp_name)) == NULL)
+	else if ((ptr = tgetenv(exp_name)) == NULL || *ptr == '\0')
 	    return ' ';
 	*--target = '\0';
 	(void) Strcat(exp_dir, name);
@@ -940,6 +996,7 @@ tw_suffix(looking, exp_dir, exp_name, target, name)
     case TW_SIGNAL:
     case TW_JOB:
     case TW_COMPLETION:
+    case TW_TEXT:
 	return ' ';
 
     default:
@@ -973,6 +1030,7 @@ tw_fixword(looking, word, dir, exp_name, max_word_length)
 
     case TW_DIRECTORY:
     case TW_FILE:
+    case TW_TEXT:
 	copyn(word, dir, max_word_length);	/* put back dir part */
 	break;
 
@@ -997,15 +1055,15 @@ static int
 tw_collect(command, looking, exp_dir, exp_name, target, pat, flags, dir_fd)
     COMMAND command;
     int looking;
-    Char *exp_dir, *exp_name, *target, *pat;
+    Char *exp_dir, *exp_name, **target, *pat;
     int flags;
     DIR *dir_fd;
 {
     static int ni;	/* static so we don't get clobbered */
-    jmp_buf osetexit;
+    jmp_buf_t osetexit;
 
 #ifdef TDEBUG
-    xprintf("target = %s\n", short2str(target));
+    xprintf("target = %S\n", *target);
 #endif
     ni = 0;
     getexit(osetexit);
@@ -1017,17 +1075,37 @@ tw_collect(command, looking, exp_dir, exp_name, target, pat, flags, dir_fd)
 	    resexit(osetexit);
 	    InsideCompletion = 0;
 	    haderr = 0;
+
+#if defined(SOLARIS2) && defined(i386)
+	    /* Compiler bug? (from PWP) */
+	    if ((looking == 3) || (looking == 7))
+		tw_logname_end();
+	    else
+		tw_dir_end();
+#else /* !(SOLARIS2 && i386) */
 	    (*tw_end_entry[looking])();
+#endif /* !(SOLARIS2 && i386) */
+
 	    /* flag error */
 	    return(-1);
 	}
         if ((ni = tw_collect_items(command, looking, exp_dir, exp_name,
-			           target, pat, 
+			           *target, pat, 
 				   ni >= 0 ? flags : 
 					flags & ~TW_IGN_OK)) >= 0) {
 	    resexit(osetexit);
 	    InsideCompletion = 0;
+
+#if defined(SOLARIS2) && defined(i386)
+	    /* Compiler bug? (from PWP) */
+	    if ((looking == 3) || (looking == 7))
+		tw_logname_end();
+	    else
+		tw_dir_end();
+#else /* !(SOLARIS2 && i386) */
 	    (*tw_end_entry[looking])();
+#endif /* !(SOLARIS2 && i386) */
+
 	    return(ni);
 	}
     }
@@ -1036,6 +1114,11 @@ tw_collect(command, looking, exp_dir, exp_name, target, pat, flags, dir_fd)
 
 /* tw_list_items():
  *	List the items that were found
+ *
+ *	NOTE instead of looking at numerical vars listmax and listmaxrows
+ *	we can look at numerical var listmax, and have a string value
+ *	listmaxtype (or similar) than can have values 'items' and 'rows'
+ *	(by default interpreted as 'items', for backwards compatibility)
  */
 static void
 tw_list_items(looking, numitems, list_max)
@@ -1043,8 +1126,9 @@ tw_list_items(looking, numitems, list_max)
 {
     Char *ptr;
     int max_items = 0;
+    int max_rows = 0;
 
-    if ((ptr = value(STRlistmax)) != NULL) {
+    if ((ptr = value(STRlistmax)) != STRNULL) {
 	while (*ptr) {
 	    if (!Isdigit(*ptr)) {
 		max_items = 0;
@@ -1052,12 +1136,48 @@ tw_list_items(looking, numitems, list_max)
 	    }
 	    max_items = max_items * 10 + *ptr++ - '0';
 	}
+	if ((max_items > 0) && (numitems > max_items) && list_max)
+	    max_items = numitems;
+	else
+	    max_items = 0;
     }
 
-    if ((max_items > 0) && (numitems > max_items) && list_max) {
-	char    tc;
+    if (max_items == 0 && (ptr = value(STRlistmaxrows)) != STRNULL) {
+	int rows = 0;
 
-	xprintf("There are %d items, list them anyway? [n/y] ", numitems);
+	while (*ptr) {
+	    if (!Isdigit(*ptr)) {
+		max_rows = 0;
+		break;
+	    }
+	    max_rows = max_rows * 10 + *ptr++ - '0';
+	}
+	if (max_rows != 0 && looking != TW_JOB)
+	    rows = find_rows(tw_item_get(), numitems, TRUE);
+	else
+	    rows = numitems; /* underestimate for lines wider than the termH */
+	if ((max_rows > 0) && (rows > max_rows) && list_max)
+	    max_rows = rows;
+	else
+	    max_rows = 0;
+    }
+
+
+    if (max_items || max_rows) {
+	char    tc;
+	char*	name;
+	int maxs;
+
+	if (max_items) {
+	    name = "items";
+	    maxs = max_items;
+	}
+	else {
+	    name = "rows";
+	    maxs = max_rows;
+	}
+
+	xprintf("There are %d %s, list them anyway? [n/y] ", maxs, name);
 	flush();
 	/* We should be in Rawmode here, so no \n to catch */
 	(void) read(SHIN, &tc, 1);
@@ -1080,7 +1200,7 @@ tw_list_items(looking, numitems, list_max)
 	Char **w = tw_item_get();
 
 	for (i = 0; i < numitems; i++) {
-	    xprintf("%s", short2str(w[i]));
+	    xprintf("%S", w[i]);
 	    if (Tty_raw_mode)
 		xputchar('\r');
 	    xputchar('\n');
@@ -1106,7 +1226,7 @@ t_search(word, wp, command, max_word_length, looking, list_max, pat, suf)
     Char   *pat;
     int     suf;
 {
-    int     numitems = 0,		/* Number of items matched */
+    int     numitems,			/* Number of items matched */
 	    flags = 0,			/* search flags */
 	    gpat = pat[0] != '\0',	/* Glob pattern search */
 	    nd;				/* Normalized directory return */
@@ -1114,9 +1234,11 @@ t_search(word, wp, command, max_word_length, looking, list_max, pat, suf)
             dir[FILSIZ + 1],		/* /x/y/z/ part in /x/y/z/f */
             exp_name[MAXNAMLEN + 1],	/* the recognized (extended) */
             name[MAXNAMLEN + 1],	/* f part in /d/d/d/f name */
-           *target = NULL;		/* Target to expand/correct/list */
+           *target;			/* Target to expand/correct/list */
     DIR    *dir_fd = NULL;	
 
+    USE(wp);
+
     /*
      * bugfix by Marty Grossman (grossman@CC5.BBN.COM): directory listing can
      * dump core when interrupted
@@ -1126,6 +1248,7 @@ t_search(word, wp, command, max_word_length, looking, list_max, pat, suf)
     non_unique_match = FALSE;	/* See the recexact code below */
 
     extract_dir_and_name(word, dir, name);
+    exp_dir[0] = '\0';
 
     /*
      * Try to figure out what we should be looking for
@@ -1151,6 +1274,12 @@ t_search(word, wp, command, max_word_length, looking, list_max, pat, suf)
 	    return (-1);
 #endif
 	break;
+    case TW_PATHNAME:
+	gpat = 0;	/* pattern holds the pathname to be used */
+	copyn(exp_dir, pat, MAXNAMLEN);
+	catn(exp_dir, dir, MAXNAMLEN);
+	/* dir[0] = '\0'; */
+	break;
 
     case TW_VARLIST:
     case TW_WORDLIST:
@@ -1159,7 +1288,7 @@ t_search(word, wp, command, max_word_length, looking, list_max, pat, suf)
 
     case TW_EXPLAIN:
 	if (command == LIST && pat != NULL) {
-	    xprintf("%s", short2str(pat));
+	    xprintf("%S", pat);
 	    if (Tty_raw_mode)
 		xputchar('\r');
 	    xputchar('\n');
@@ -1170,27 +1299,28 @@ t_search(word, wp, command, max_word_length, looking, list_max, pat, suf)
 	break;
     }
 
-    /*
-     * let fignore work only when we are not using a pattern
-     */
-    flags |= (gpat == 0) ? TW_IGN_OK : TW_PAT_OK;
-
     if ((*word == '~') && (Strchr(word, '/') == NULL)) {
 	looking = TW_LOGNAME;
 	target = name;
+	gpat = 0;	/* Override pattern mechanism */
     }
     else if ((target = Strrchr(name, '$')) != 0 && 
 	     (Strchr(name, '/') == NULL)) {
 	target++;
 	looking = TW_VARIABLE;
+	gpat = 0;	/* Override pattern mechanism */
     }
     else
 	target = name;
 
+    /*
+     * let fignore work only when we are not using a pattern
+     */
+    flags |= (gpat == 0) ? TW_IGN_OK : TW_PAT_OK;
+
 #ifdef TDEBUG
     xprintf("looking = %d\n", looking);
 #endif
-    exp_dir[0] = '\0';
 
     switch (looking) {
     case TW_ALIAS:
@@ -1246,10 +1376,20 @@ t_search(word, wp, command, max_word_length, looking, list_max, pat, suf)
 	    return nd;
 	break;
 
+    case TW_TEXT:
+	flags |= TW_TEXT_CHK;
+	/*FALLTHROUGH*/
     case TW_FILE:
 	if ((nd = expand_dir(dir, exp_dir, &dir_fd, command)) != 0)
 	    return nd;
 	break;
+    case TW_PATHNAME:
+	if ((dir_fd = opendir(short2str(exp_dir))) == NULL) {
+	    xprintf("%S: %s\n", exp_dir, strerror(errno));
+	    return -1;
+	}
+	looking = TW_FILE;
+	break;
 
     case TW_LOGNAME:
 	word++;
@@ -1263,8 +1403,6 @@ t_search(word, wp, command, max_word_length, looking, list_max, pat, suf)
 #ifdef YPBUGS
 	    fix_yp_bugs();
 #endif /* YPBUGS */
-	    if (looking == TW_LOGNAME)
-		word--;
 	    return (0);
 	}
 	copyn(name, word, MAXNAMLEN);	/* name sans ~ */
@@ -1285,12 +1423,14 @@ t_search(word, wp, command, max_word_length, looking, list_max, pat, suf)
     }
 
     numitems = tw_collect(command, looking, exp_dir, exp_name, 
-			  target, pat, flags, dir_fd);
+			  &target, pat, flags, dir_fd);
     if (numitems == -1)
 	return -1;
 
     switch (command) {
     case RECOGNIZE:
+    case RECOGNIZE_ALL:
+    case RECOGNIZE_SCROLL:
 	if (numitems <= 0) 
 	    return (numitems);
 
@@ -1309,7 +1449,7 @@ t_search(word, wp, command, max_word_length, looking, list_max, pat, suf)
 		return numitems;
 
 	    default:	/* completion specified suffix */
-		suffix[0] = suf;
+		suffix[0] = (Char) suf;
 		break;
 	    }
 	    catn(word, suffix, max_word_length);
@@ -1363,8 +1503,10 @@ static Char *
 dollar(new, old)
     Char   *new, *old;
 {
-    Char   *var, *val, *p, save;
+    Char    var[MAXVARLEN];
+    Char   *val, *p;
     int     space;
+    int	    i;
 
     for (space = FILSIZ, p = new; *old && space > 0;)
 	if (*old != '$') {
@@ -1375,30 +1517,33 @@ dollar(new, old)
 	    struct varent *vp;
 
 	    /* found a variable, expand it */
-	    for (var = ++old; alnum(*old); old++)
-		continue;
-	    save = *old;
-	    *old = '\0';
-	    vp = adrof(var);
-	    val = (!vp) ? Getenv(var) : NULL;
-	    *old = save;
-	    /*
-	     * Don't expand array variables
-	     */
-	    if (vp) {
-		if (!vp->vec[0] || vp->vec[1]) {
-		    *new = '\0';
-		    return (NULL);
+	    for (i = 0; i < MAXVARLEN; i++) {
+		var[i] = *++old & TRIM;
+		if (!alnum(var[i])) {
+		    var[i] = '\0';
+		    break;
 		}
-		else
-		    val = vp->vec[0];
 	    }
-	    else if (!val) {
+	    vp = adrof(var);
+	    val = (!vp) ? tgetenv(var) : NULL;
+	    if (vp) {
+		for (i = 0; vp->vec[i] != NULL; i++) {
+		    for (val = vp->vec[i]; space > 0 && *val; space--)
+			*p++ = *val++;
+		    if (vp->vec[i+1] && space > 0) {
+			*p++ = ' ';
+			space--;
+		    }
+		}
+	    }
+	    else if (val) {
+		for (;space > 0 && *val; space--)
+		    *p++ = *val++;
+	    }
+	    else {
 		*new = '\0';
 		return (NULL);
 	    }
-	    for (; space > 0 && *val; space--)
-		*p++ = *val++;
 	}
     *p = '\0';
     return (new);
@@ -1468,9 +1613,9 @@ expand_dir(dir, edir, dfd, cmd)
 	 * From: Amos Shapira <amoss@cs.huji.ac.il>
 	 * Print a better message when completion fails
 	 */
-	xprintf("\n%s %s\n",
-		*edir ? short2str(edir) :
-		(*tdir ? short2str(tdir) : short2str(dir)),
+	xprintf("\n%S %s\n",
+		*edir ? edir :
+		(*tdir ? tdir : dir),
 		(errno == ENOTDIR ? "not a directory" :
 		(errno == ENOENT ? "not found" : "unreadable")));
 	NeedsRedraw = 1;
@@ -1518,11 +1663,14 @@ nostat(dir)
     register Char **cp;
 
     if ((vp = adrof(STRnostat)) == NULL || (cp = vp->vec) == NULL)
-	return (FALSE);
-    for (; *cp != NULL; cp++)
-	if (eq(*cp, dir))
-	    return (TRUE);
-    return (FALSE);
+	return FALSE;
+    for (; *cp != NULL; cp++) {
+	if (Strcmp(*cp, STRstar) == 0)
+	    return TRUE;
+	if (Gmatch(dir, *cp))
+	    return TRUE;
+    }
+    return FALSE;
 } /* end nostat */
 
 
@@ -1539,10 +1687,10 @@ filetype(dir, file)
 	char   *ptr;
 	struct stat statb;
 
-	if (nostat(dir)) return('/');
+	if (nostat(dir)) return(' ');
 
 	(void) Strcpy(path, dir);
-	catn(path, file, sizeof(path) / sizeof(Char));
+	catn(path, file, (int) (sizeof(path) / sizeof(Char)));
 
 	if (lstat(ptr = short2str(path), &statb) != -1)
 	    /* see above #define of lstat */
@@ -1581,12 +1729,18 @@ filetype(dir, file)
 	    if (S_ISNWK(statb.st_mode)) /* Network Special [hpux] */
 		return (':');
 #endif
+#ifdef S_ISCHR
 	    if (S_ISCHR(statb.st_mode))	/* char device */
 		return ('%');
+#endif
+#ifdef S_ISBLK
 	    if (S_ISBLK(statb.st_mode))	/* block device */
 		return ('#');
+#endif
+#ifdef S_ISDIR
 	    if (S_ISDIR(statb.st_mode))	/* normal Directory */
 		return ('/');
+#endif
 	    if (statb.st_mode & 0111)
 		return ('*');
 	}
@@ -1606,10 +1760,8 @@ isadirectory(dir, file)		/* return 1 if dir/file is a directory */
 	Char    path[MAXPATHLEN];
 	struct stat statb;
 
-	if (nostat(dir)) return(1);
-
 	(void) Strcpy(path, dir);
-	catn(path, file, sizeof(path) / sizeof(Char));
+	catn(path, file, (int) (sizeof(path) / sizeof(Char)));
 	if (stat(short2str(path), &statb) >= 0) {	/* resolve through
 							 * symlink */
 #ifdef S_ISSOCK
@@ -1628,6 +1780,31 @@ isadirectory(dir, file)		/* return 1 if dir/file is a directory */
 } /* end isadirectory */
 
 
+
+/* find_rows():
+ * 	Return how many rows needed to print sorted down columns
+ */
+static int
+find_rows(items, count, no_file_suffix)
+    Char *items[];
+    int     count, no_file_suffix;
+{
+    register int i, columns, rows;
+    unsigned int maxwidth = 0;
+
+    for (i = 0; i < count; i++)	/* find widest string */
+	maxwidth = max(maxwidth, (unsigned int) Strlen(items[i]));
+
+    maxwidth += no_file_suffix ? 1 : 2;	/* for the file tag and space */
+    columns = (TermH + 1) / maxwidth;	/* PWP: terminal size change */
+    if (!columns)
+	columns = 1;
+    rows = (count + (columns - 1)) / columns;
+
+    return rows;
+} /* end rows_needed_by_print_by_column */
+
+
 /* print_by_column():
  * 	Print sorted down columns
  */
@@ -1642,7 +1819,7 @@ print_by_column(dir, items, count, no_file_suffix)
     lbuffed = 0;		/* turn off line buffering */
 
     for (i = 0; i < count; i++)	/* find widest string */
-	maxwidth = max(maxwidth, Strlen(items[i]));
+	maxwidth = max(maxwidth, (unsigned int) Strlen(items[i]));
 
     maxwidth += no_file_suffix ? 1 : 2;	/* for the file tag and space */
     columns = (TermH + 1) / maxwidth;	/* PWP: terminal size change */
@@ -1655,15 +1832,15 @@ print_by_column(dir, items, count, no_file_suffix)
 	    i = c * rows + r;
 
 	    if (i < count) {
-		w = Strlen(items[i]);
+		w = (unsigned int) Strlen(items[i]);
 
 		if (no_file_suffix) {
 		    /* Print the command name */
-		    xprintf("%s", short2str(items[i]));
+		    xprintf("%S", items[i]);
 		}
 		else {
 		    /* Print filename followed by '/' or '*' or ' ' */
-		    xprintf("%s%c", short2str(items[i]),
+		    xprintf("%S%c", items[i],
 			    filetype(dir, items[i]));
 		    w++;
 		}
@@ -1690,7 +1867,8 @@ int
 StrQcmp(str1, str2)
     register Char *str1, *str2;
 {
-    for (; *str1 && (*str1 & TRIM) == (*str2 & TRIM); str1++, str2++)
+    for (; *str1 && samecase(*str1 & TRIM) == samecase(*str2 & TRIM); 
+	 str1++, str2++)
 	continue;
     /*
      * The following case analysis is necessary so that characters which look
@@ -1715,14 +1893,7 @@ int
 fcompare(file1, file2)
     Char  **file1, **file2;
 {
-#if defined(NLS) && !defined(NOSTRCOLL)
-    char    buf[2048];
-
-    (void) strcpy(buf, short2str(*file1));
-    return ((int) strcoll(buf, short2str(*file2)));
-#else
-    return (StrQcmp(*file1, *file2));
-#endif
+    return (int) collate(*file1, *file2);
 } /* end fcompare */
 
 
@@ -1761,18 +1932,18 @@ copyn(des, src, count)
 } /* end copyn */
 
 
-/* Getenv():
+/* tgetenv():
  *	like it's normal string counter-part
  *	[apollo uses that in tc.os.c, so it cannot be static]
  */
 Char *
-Getenv(str)
+tgetenv(str)
     Char   *str;
 {
     Char  **var;
     int     len, res;
 
-    len = Strlen(str);
+    len = (int) Strlen(str);
     for (var = STR_environ; var != NULL && *var != NULL; var++)
 	if ((*var)[len] == '=') {
 	    (*var)[len] = '\0';
@@ -1782,4 +1953,54 @@ Getenv(str)
 		return (&((*var)[len + 1]));
 	}
     return (NULL);
-} /* end Getenv */
+} /* end tgetenv */
+
+
+struct scroll_tab_list *scroll_tab = 0;
+
+static void
+add_scroll_tab(item)
+    Char *item;
+{
+    struct scroll_tab_list *new_scroll;
+
+    new_scroll = (struct scroll_tab_list *) xmalloc((size_t)
+	    sizeof(struct scroll_tab_list));
+    new_scroll->element = Strsave(item);
+    new_scroll->next = scroll_tab;
+    scroll_tab = new_scroll;
+}
+
+static void
+choose_scroll_tab(exp_name, cnt)
+    Char **exp_name;
+    int cnt;
+{
+    struct scroll_tab_list *loop;
+    int tmp = cnt;
+    Char **ptr;
+
+    ptr = (Char **) xmalloc(sizeof(Char *) * cnt);
+
+    for(loop = scroll_tab; loop && (tmp >= 0); loop = loop->next)
+	ptr[--tmp] = loop->element;
+
+    qsort((ptr_t) ptr, (size_t) cnt, sizeof(Char *), 
+	  (int (*) __P((const void *, const void *))) fcompare);
+	    
+    copyn(*exp_name, ptr[curchoice], (int) Strlen(ptr[curchoice]));	
+    xfree((ptr_t) ptr);
+}
+
+static void
+free_scroll_tab()
+{
+    struct scroll_tab_list *loop;
+
+    while(scroll_tab) {
+	loop = scroll_tab;
+	scroll_tab = scroll_tab->next;
+	xfree((ptr_t) loop->element);
+	xfree((ptr_t) loop);
+    }
+}
